{**************************************************************************** 
ADSP-2101 - AD1849(SOUNDPORT)  INTERFACE PROGRAM 
Oli Josefsson & Kapriel Karagozyan, Analog Devices     Sept, 1991 
 
The AD1849 will be set up as follows: 
(NC indicates a no care state, all control reg are 1 bit unless indicated  
 by [#bits]) 
 
Control time slot control bits: 
DCB=0 followed by 1 
AC=1    autocalibrate 
DFR=5   Data conversion frequency,44.1kH [3] 
ST=1    stereo mode 
DF=0    dataformat is 16 bit twos complement [2] 
MCK=2   16.9344 MHz is the master clock [2] 
FSEL=0  frame size, 64 bits [2] 
MS=1    master mode (i.e. receive external serial clock) 
TXDIS=0 enable serial output 
ENL=0   disable loopback testing 
ADL=NC  loopback mode analog/digital (disabled) 
PIO=NC  paralell I/O bits not used [2] 
REVID=NC 
 
Data Timeslot Control Bits: 
OM0=1    enable line 0 output 
OM1=1    enable line 1 output 
LO=0    no attenuation of left channel output [6] 
SM=0    mono output is muted 
RO=0    no attenuation of right channel output [6] 
PIO=NC  paralell I/O bits not used [2] 
OVR=NC  overrange INPUT 
IS=0    line-level stereo input selected 
LG=8    12db gain for left channel [4] 
MA=15   no monitor mix (i.e. ADC output is not mixed with DAC input 
RG=8    12db gain for right channel [4] 
 
***************************************************************************** 
 
This program makes use of the multi-channel mode that is available on the 
SPORT0 and it also uses autobuffering to reduce interrupt service overhead. 
For a description of the multi-channel and autobuffering features, please 
refer to the ADSP-2111 or ADSP-2101 architecture user's manuals. 
 
In its current condition, this program can be used "as is" to perform straight 
talkthrough (at 44.1 KHz sample rate) on both left and right channels of the  
AD1849. The incoming audio data from the 16bit ADCs is placed in a short buffer  
and immediately sent out to the 16bit DACs. 
 
The initial setup and handshaking with the AD1849 starts with the "START" 
routine. A state machine to perform the handshaking is executed in software. 
Once the AD1849 is configured properly, the processor enters the WAIT_DATA 
loop and waits for serial port interrupt requests. 
 
This program is booted into the ADSP-2101 on power-up. 
 
*****************************************************************************} 
 
.module ad1849; 
 
.var/circ  ctrlin[4];             {circular buffers for data input and} 
.var/circ  ctrlout[4];            {                                                                                                                                  output for data mode and control mode} 
.var/circ  datain[4]; 
.var/circ  dataout[8]; 
.var   	   first_flg; 
.var       dcb_flg; 
.var       dmode_flg; 
 
#define STACK_SIZE 100 
.var		stack[STACK_SIZE]; 
 
{ ************************************************************************** } 
 
start:	                        { insure `c` runtime environment } 
	l0=0;l1=0;l2=0;l3=0;	{all length registers } 
	l4=0;l5=0;l6=0;l7=0;	{ should be set to 0} 
	m0=0;m1=1;m2=0;m3=-1;	{modify registers set } 
	m5=1;m6=0;m7=-1;	{ to various constants} 
	ax0 = ^stack; 
	ay0 = STACK_SIZE - 1; 
	ar = ax0 + ay0;	 
	m4 = ar;		{frame pointer = top of stack} 
	i4 = ar;		{stack pointer = top of stack} 
 
	  	i6 = 0x8;                        { setup interrupt vectors } 
		m6 = 0x4; 
		ax0 = ^setupcontrol; 
		ay0 = 0xf; 
		ar = ax0 and ay0; 
		sr = lshift ar by 0x4(lo); 
		ay0 = 0xf; 
		ar = sr0 or ay0; 
		px = ar; 
		ay0 = 0x3ff0; 
		ar = ax0 and ay0; 
		mr0 = ar; 
		ax0 = 4; 
		ar = -ax0;                                                                                                                                   
		se = ar; 
		sr = lshift mr0 (lo); 
		ay0 = 0x1800; 
		ar = sr0 or ay0; 
		pm (i6,m6) = ar; 
 
		ax0 = ^newdata; 
		ay0 = 0xf; 
		ar = ax0 and ay0; 
		sr = lshift ar by 0x4(lo); 
		ay0 = 0xf; 
		ar = sr0 or ay0; 
		px = ar; 
		ay0 = 0x3ff0; 
		ar = ax0 and ay0; 
		mr0 = ar; 
		ax0 = 4; 
		ar = -ax0; 
		se = ar; 
		sr = lshift mr0 (lo); 
		ay0 = 0x1800; 
		ar = sr0 or ay0; 
		pm (i6,m6) = ar; 
 
                 ax0=0x0000;    {set the ad1849 d/c pin low} 
                 dm(0x300d)=ax0; 
                 ax0=0x0058; 
                 dm(0x304a)=ax0; {set sport mux} 
                 mx0=0x5555; 
                 mx1=0xaaaa; 
                 l2=%ctrlin; 
                 i2=^ctrlin; 
      		 l3=%ctrlout; 
      		 i3=^ctrlout; 
                 ax0=1; 
                 dm(first_flg)=ax0; 
                 dm(dcb_flg)=ax0; 
                 ax0=0; 
                 dm(dmode_flg)=ax0; 
                 ay0=b#0010000100101100;  
 
{initialize control mode output buffer} 
                                                                                                                                                    
                 dm(i3,m1)= b#0010000100101100; {dcb=0,ac=1,dfr=05,st=1,df=00} 
                 dm(i3,m1)= b#0010001000000000; {mck=02,fsel=0,ms=1,txdis=0} 
                                {enl=0,adl=0} 
                 dm(i3,m1)= b#0000000000000000; {pio = 00} 
                 dm(i3,m1)= b#0000000000000000; {revid=nc} 
                 l6 = 0;                  {linear addressing for register } 
                 i6 = 0x3fef;             {pt. to last dm cntrl reg} 
{3fef}           dm(i6,m5) = 0x0000;      {sport1 not used} 
{3ff0}           dm(i6,m5) = 0x0000; 
{3ff1}           dm(i6,m5) = 0x0000; 
{3ff2}           dm(i6,m5) = 0x0000; 
{3ff3}           dm(i6,m5) = 0x06a7; {autobuf. rx:i2,m1 tx:i3,m1} 
{3ff4}           dm(i6,m5) = 383;    {sport rfsdiv, sets up frame sync freq} 
{3ff5}           dm(i6,m5) = 849;    {sport0 sclkdiv, sclk will be less than} 
                                     {8khz from control mode} 
{3ff6}           dm(i6,m5) = b#1100010100011111; 
                                                                                                                                                                      {sport0 control register: 
                                       multi-channel mode, 24 channels 
                                       internal sclk & rfs 
                                       normal framing mode 
                                       frame sync not inverted 
                                       16 bit word length} 
{3ff7}           dm(i6,m5) = 0x000f;  {first 4 xmit multichannels used} 
{3ff8}           dm(i6,m5) = 0x0000; 
{3ff9}           dm(i6,m5) = 0x000f;  {first 4 rx multichannels used} 
{3ffa}           dm(i6,m5) = 0x0000; 
{3ffb}           dm(i6,m5) = 0x0000;  {tscale register} 
{3ffc}           dm(i6,m5) = 0x0000;  {tcount register} 
{3ffd}           dm(i6,m5) = 0x0000;  {tperiod register,initializing value 
                                       for tcount after every interrupt} 
{3ffe}           dm(i6,m5) = 0x0800;  {external data memory waits=0} 
                                      {wait state 3 = 4 waits} 
 
                                                                                                                                                   icntl=0x0; 
                 imask=b#110000;    {only sport transmit intrpt enabled 
                                       initially while in control mode} 
 
                 ax0=dm(i3,m1);       {send first 16bits of ctrl word} 
                 tx0=ax0; 
{3fff}           dm(i6,m5) = 0x1418;  {system control reg: sport0 enabled} 
 
{wait for an interrupt indicating that transmit register is ready for new data 
 and that the 2101 has received a 16bit word} 
 
wait1:           ax1=dm(dmode_flg); 
                 ar=pass ax1; 
                 if gt jump go_dmode; 
                 jump wait1; 
 
setupcontrol:    ax0=dm(first_flg); 
                 af=pass ax0; 
                 if ne jump decr_first; 
                 ax0=dm(dcb_flg); 
                 ar=pass ax0; 
                 if eq jump dcbflg_set; 
                 ax0=dm(ctrlin);	{dcb_flg has not been set yet} 
                 ar=ax0 xor ay0;        {check all incoming bits including dcb bit} 
                 if                                                                                                                                   eq jump set_dcb;	{set flag if dcb was 0} 
                 rti; 
 
dcbflg_set:      ax0=dm(ctrlin);    {dcb_flg was set} 
                 ar=ax0 and ay0;    {only check for dcb bit} 
                 if ne jump setdmode;   {if dbc=1 ready for datamode} 
                 rti; 
 
set_dcb:         ax0=0; 
                 dm(dcb_flg)=ax0; 
                 ax0=b#0010010100101100; {dcb was 0, prepare to send dcb=1} 
                 dm(ctrlout)=ax0; 
                 ay0=b#0000010000000000; 
                 rti; 
 
decr_first:      ax0=0; 
                 dm(first_flg)=ax0; 
                 rti; 
 
setdmode:        imask=b#100000; 
                 ax0=0x0818;              {disable sport0} 
                 dm(0x3fff)=ax0; 
{ at this point we could boot page#1 and do the following data mode setup after 
  the reboot occurs.  this would free up some pm ram for other uses} 
                  
                 i3 = ^dataout; 
                 l3=0; 
                 dm(i3,m1) = 0x0000;                                                                                                                                               {reset output & input data buffers} 
                 dm(i3,m1) = 0x0000;             {initialize embedded control bits} 
                 dm(i3,m1) = b#1100000000000000; {om1=1,om0=1,lo=0,sm=0,ro=0} 
                 dm(i3,m1) = b#0001100011111000; {pio=00,ovr=0,is=0,lg=8,ma=15,rg=8} 
                 dm(i3,m1) = 0x0000;             {reset output & input data buffers} 
                 dm(i3,m1) = 0x0000;             {initialize embedded control bits} 
                 dm(i3,m1) = b#1100000000000000; {om1=1,om0=1,lo=0,sm=0,ro=0} 
                 dm(i3,m1) = b#0001100011111000; {pio=00,ovr=0,is=0,lg=8,ma=15,rg=8} 
                 ax0=0x861f; 
                 dm(0x3ff6)=ax0;        {sport0 control: 
               				multi-channel mode 
			                external sclk & rfs 
 			                32 word frames, 16bit words} 
                 ax0=0x0002;    {set the ad1849 d/c pin high} 
                 dm(0x300d)=ax0; 
                 ax0=1; 
                 dm(dmode_fl                                                                                                                                  g)=ax0; 
                 rti; 
 
 
go_dmode:        l2=%datain; 
                 i2=^datain; 
                 l3=%dataout; 
                 i3=^dataout; 
                 m6=3; 
        	 ax0=0xffff; 
		 dm(0x3ff7)=ax0;        {enable all multi-channel words} 
		 dm(0x3ff8)=ax0; 
		 dm(0x3ff9)=ax0; 
		 dm(0x3ffa)=ax0; 
                 ax0=dm(i3,m1);         {send first 16bits of data} 
                 tx0=ax0; 
                 ax0=0x1418; 
                 dm(0x3fff)=ax0;        {turn on sport0} 
                 ifc=b#000000111111;	{clear all pending interrupts} 
                 imask=b#101000;      {sport0 rx interrupt on} 
 
wait_data:       {dm(0x100)=mx0;}    {cause digital noise} 
                 {dm(0x100)=mx1; } 
                 jump wait_data;   {wait for sport0 rx autobuffer interrupt} 
 
newdata:	 i6=i3; 
		 modify(i6,m6);    
		 ax0=dm(datain);    {this routine sends the incoming} 
                 dm(i6,m5)=ax0;     {a/d data straight to the d/a by} 
                                                                                                                                                   ax0=dm(datain+1);  {copying newly arrived data words from} 
                 dm(i6,m5)=ax0;     {datain into the dataout buffer} 
                 rti; 
 
{the dataout buffer is twice as long as the datain buffer, since it contains} 
{control words, as well as output data. this program ignores the control} 
{words arriving back from the ad1849 during the data mode}       		     
                 
.endmod; 
