;****************************************************************
;
; ADAPT.ASM - 15-TAP ADAPTIVE LEAST MEAN SQUARE FILTER
; FOR THE TMS320C25 DSP
; THIS PROGRAM IS AN ADAPTATION OF THE ADAPTIVE FILTER PROGRAM
; FOUND ON PAGE 5-65 OF THE "SECOND GENERATION TMS320 USER'S
; GUIDE" BY TEXAS INSTRUMENTS, SPRU014.
; "ADAPT.ASM" IS CONFIGURED TO RUN ON THE "DXP-25" PC/XT CARD
;
;****************************************************************
; DATA PAGE 0 VARIABLES
TMP0    EQU     96             ; TEMPORARY VARIABLE
ONE     EQU     97             ; CONSTANT, 1
MU      EQU     98             ; ADAPTATION CONSTANT
ERROR   EQU     99             ; ERROR=Y-X
ERRF    EQU     100            ; VAR USED FOR INTERMEDIATE RESULT
Y       EQU     101            ; OUTPUT OF FILTER
X       EQU     102            ; NEWEST INPUT SAMPLE
; SYSTEM EQUATES
COFPRG  EQU     0FF00H         ; COEFFICIENTS PROGRAM MEMORY LOC
COFDAT  EQU     0200H          ; COEFFICIENTS DATA MEMORY LOC
FIRSTX  EQU     0300H          ; NEWEST X, X(N) IN BLOCK B1
LASTX   EQU     030EH          ; OLDEST X, X(N-14) IN BLOCK B1
        ;***** START OF PROGRAM *****
STRT:   LDPK    0
        LALK    0E00H           ;SET STATUS 0 REG
        SACL    TMP0
        LST     TMP0
        LALK    03F0H           ;SET STATUS 1 REG
        SACL    TMP0
        LST1    TMP0
        ; LOAD ADAPTATION CONSTANT, MU
        LALK    04AAH
        SACL    MU
        ; LOAD CONSTANT, ONE
        LACK    1
        SACL    ONE
        ; CLEAR DATA BUFFER
        ZAC
        LARP    AR2
        LRLK    AR2,FIRSTX
        RPTK    14
        SACL    *+
        ; SET COEFFICIENTS INITIALLY TO 0
        CNFD
        ZAC
        LARP    AR2
        LRLK    AR2,COFDAT
        RPTK    14
        SACL    *+
        ; SET A/D SAMPLE RATE
        LACK    020H            ; ALLOW ONLY TXINT
        SACL    IMR
        EINT
        LALK    3
        SACL    DXR             ;SIGNAL SECONDARY COM
        LALK    064CAH          ;set Fs TO 10000 HZ
        IDLE                    ;Rb=50
        ZAC
        EINT
        IDLE
        ; SET INTERRUPT MASK
        LACK    012H           ;INTERRUPT MASK
        SACL    IMR            ;RINT AND INT1 ONLY
        EINT
        LDPK    0
LOOP:   B       LOOP
;*************************************************************
; RINT - SERIAL RX INTERRUPT
;        OCCURS WHEN A NEW A/D SAMPLE FROM THE TLS32044 AIC
;        IS READY.
;*************************************************************
RINT:
        LAC     DRR            ; READ A/D SAMPLE INTO ACC
        SACL    X              ; SAVE IN X
        ; ***** ADAPTIVE FILTER *****
        CNFP                   ; MAKE B0 PROGRAM MEMORY
        MPYK    0              ; CLEAR P REG
        LAC     ONE,14         ; LOAD ROUNDING BIT
        LARP    AR3            ; SELECT AR3
        LRLK    AR3,LASTX      ; POINT AR3 TO LAST SAMPLE
        RPTK    14             ; PERFORM FIR FILTER FUNCTION
        MACD    COFPRG,*-      ; MULTIPLY/ACCUMULATE/MOVE DATA
        CNFD                   ; NOW, MAKE B0 DATA MEMORY
                               ; SO WE CAN WRITE TO IT
        APAC                   ; ACCUMULATE LAST PRODUCT
        SACH    Y,1            ; STORE OUTPUT
        NEG                    ; MAKE NEGATIVE
        ADD     X,15           ; ADD NEWEST INPUT
        SACH    ERROR,1        ; ERROR=X-Y
        ; ***** UPDATE FILTER COEFFICIENTS *****
        LT      ERROR          ; PERFORM ERRF=ERROR*MU
        MPY     MU
        PAC
        ;ADD     ONE,14         ; ROUND
        SACH    ERRF,1
        MAR     *+             ; POINT AR3 TO NEWEST X(n) B1 LOC
        LAC     X              ; PLACE X SAMPLE IN B1
        SACL    *
        LRLK    AR2,COFDAT     ; POINT AR2 TO DATA COEFFICIENTS
        LRLK    AR3,LASTX      ; POINT AR3 TO LAST DATA SAMPLE
        LT      ERRF           ; GET INTERMEDIATE ADAPT RESULT
        MPY     *-,AR2         ; Update=2*Mu*ERROR(n)*X(n-14)
        ;***** UPDATE COEFFICIENTS *****
ADAPT:  ;******h15*******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h14******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h13******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h12******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h11******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h10******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h9******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h8******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h7******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h6******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h5******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h4******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h3******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h2******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;******h1******
        ZALR    *,AR3
        MPYA    *-,AR2
        SACH    *+
        ;************15
        ZALR    *
        APAC
        SACH    *+
        LAC     ERROR       ; OUTPUT ERROR TO D/A
        ANDK    0FFFCH      ; MASK OFF LOWER 2 BITS
        SACL    DXR         ; SEND IT
        EINT
        RET
TINT:   SACL    DXR
        EINT
        RET
        END


