{ Cascaded IIR Biquad Sections 
  (Direct Form II or Transposed Direct Form I)
  
      w(n) = x(n) + a1*w(n-1) + a2*w(n-2)       beware of signs here!
      y(n) = w(n) + b1*w(n-1) + b2*w(n-2)       (single biquad structure)
  
  Each section consists of: b2,b1,a2,a1,w(n-1),w(n-2)
  Notice that coefs have been normalized such that b0=1.0

  Calling Parameters
    --- cascaded_biquad ---
    l0, l1, l8 = 0
    m1, m8 = 1
    f8 = first x(n)
    r0 = number of biquad sections (filter order / 2)
    b0 = address of delay line buffer
    b8 = address of coefficient buffer
    --- cascaded_biquad_init ---
    l0 = 0
    r0 = number of biquad sections (filter order / 2)
    b0 = address of delay line buffer

  Returned Values
    f8 = last y(n)              cascaded_biquad
    delay line zeroed           cascaded_biquad_init

  Register File Usage:
    f2, f3, f4, f8, f12         cascaded_biquad
    f2                          cascaded_biquad_init

  Data Address Generator Usage
    i0, b1, i1, i8, m1, m8      cascaded_biquad
    i0                          cascaded_biquad_init

  Benchmark
    6 + 4*(sections) cycles     cascaded_biquad
    5 + 2*(sections) cycles     cascaded_biquad_init (incl. non-delayed rts)

  Memory Usage
    --- cascaded_biquad ---
    10 words        instructions in PM
    4*(sections)    coefficients in PM
    2*(sections)    delay line storage in DM
    --- cascaded_biquad_init ---
    5 words         instructions in PM
    2*(sections)    delay line storage in DM
  
  "cascade.asm"
  Analog Devices, Inc.  DSP Applications  P.O.Box 9106  Norwood, MA 02062
  Christoph D. Cavigioli ... 25-Apr-1991 
}
.GLOBAL cascaded_biquad, cascaded_biquad_init;
.EXTERN coefs, dline;

.SEGMENT /PM    pm_code;
cascaded_biquad_init:   { --- call this once to set up initial conditions --- }
    f2=0;
    lcntr=r0, do clear until lce;                       {for each section, do:}
        dm(i0,1)=f2;                                    {clear w`` storage}
clear:  dm(i0,1)=f2;                                    {clear w`  storage}
    rts;

    { --- comments for subroutine called cascaded_biquad ---                }
    {TERMINOLOGY: w` = w(n-1), w`` = w(n-2), NEXT = "of next biquad section"}
    { #1    clear f12,             rd w``,      rd a2         loop prologue }
    { #2    for each section, do:                                           }
    { #3    w``a2, 1st=x+0,else=y, rd w`,       rd a1         loop body     }
    { #4    w`a1,  x+w``a2,        wr new w`,   rd b2         loop body     }
    { #5    w``b2, new w,          rd NEXT w``, rd b1         loop body     }
    { #6    w`b1,  new w+(w``b2),  wr new w,    rd NEXT a2    loop body     }
    { #7    calc last y after dropping out of loop            loop epilogue }

cascaded_biquad:        { --- call this for every sample to be filtered --- }
    b1=b0;
        f12=f12-f12,          f2=dm(i0,m1), f4=pm(i8,m8);   { #1 }
    lcntr=r0, do quads until lce;                           { #2 }
        f12=f2*f4, f8=f8+f12, f3=dm(i0,m1), f4=pm(i8,m8);   { #3 }
        f12=f3*f4, f8=f8+f12, dm(i1,m1)=f3, f4=pm(i8,m8);   { #4 }
        f12=f2*f4, f8=f8+f12, f2=dm(i0,m1), f4=pm(i8,m8);   { #5 }
quads:  f12=f3*f4, f8=f8+f12, dm(i1,m1)=f8, f4=pm(i8,m8);   { #6 }
    rts (db),      f8=f8+f12;                               { #7 }
     nop;
     nop;
.ENDSEG;
