/**************************************************************
  
                 cs4215.c
  
                 staff
  
                 05-13-92
  
           (C) Texas Instruments Inc., 1992 
  
           Refer to the file 'license.txt' included with this 
           this package for usage and license information. 
  
*************************************************************/
/**************************************************************************/
/*   CS4215.C                                                             */
/*                                                                        */
/*   TMS320C3x - CRYSTAL 4215 MM CODEC                                    */
/*      :TMS320C3x CODE                                                   */
/*       Compile and archive into CS4215.lib                              */
/*                                                                        */
/*   Leor Brenman, DSP Applications                                       */
/*   (C) 1991 TEXAS INSTRUMENTS, HOUSTON                                  */
/**************************************************************************/
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <cs4215.h>

/**************************************************************************/
/* GLOBAL VARIABLES                                                       */
/**************************************************************************/
int  buffer_size = BLOCK_SIZE; /* SIZE OF I/O BUFFER(S)                   */
VPVF output0;                  /* OUTPUT DATA BUFFER FOR PROCESSOR        */
VPVF input0;                   /* INPUT DATA BUFFER FOR PROCESSOR         */
VPVF output_xfer0;             /* OUTPUT DATA BUFFER FOR ISR/CODEC        */
VPVF input_xfer0;              /* INPUT DATA BUFFER FOR ISR/CEDEC         */
VPVF output1;                  /* OUTPUT DATA BUFFER FOR PROCESSOR        */
VPVF input1;                   /* INPUT DATA BUFFER FOR PROCESSOR         */
VPVF output_xfer1;             /* OUTPUT DATA BUFFER FOR ISR/CEDEC        */
VPVF input_xfer1;              /* INPUT DATA BUFFER FOR ISR/CODEC         */
VI   buffer_rdy     = FALSE;   /* CPU-ISR COMM FLAG (INPUT)               */
VI   buffer_index   = 0;       /* INDEX INTO INPUT AND OUTPUT DATA ARRAYS */
VI   first_half = TRUE;
VI   i;                        /* GENERIC COUNTER VARIABLE                */

CS4215_WORD     data_control;

#if C_ISR
/**************************************************************************/
/* C_INT06() OR C_INT08()                                                 */
/*        SERIAL PORT 0/1 RECEIVE INTERRUPT SERVICE ROUTINE               */
/**************************************************************************/
#if SER_NUM
void c_int06(void) {}
void c_int08(void)
#else
void c_int08(void) {}
void c_int06(void)
#endif
{
    VPVF swap;
    CS4215_WORD     in,out;

    if(first_half) /* First half of the 64 bit transmission */
    {
       first_half = FALSE;

       in._intval[0] = SERIAL_PORT_ADDR(SER_NUM)->r_data;
       input_xfer0[buffer_index] = in.stereo_16._bitval.right;
       input_xfer1[buffer_index] = in.stereo_16._bitval.left;

       out.stereo_16._bitval.left    = output_xfer1[buffer_index];
       out.stereo_16._bitval.right   = output_xfer0[buffer_index];
       SERIAL_PORT_ADDR(SER_NUM)->x_data  = out._intval[0];

       if(++buffer_index == buffer_size)
       {
           swap          = input0;
           input0        = input_xfer0;
           input_xfer0   = swap;

           swap          = input1;
           input1        = input_xfer1;
           input_xfer1   = swap;

           swap           = output0;
           output0        = output_xfer0;
           output_xfer0   = swap;

           swap           = output1;
           output1        = output_xfer1;
           output_xfer1   = swap;

           buffer_index   = 0;
           buffer_rdy     = TRUE;
       }
    }
    else /* Second half of transmission */
    {
       SERIAL_PORT_ADDR(SER_NUM)->r_data;
       SERIAL_PORT_ADDR(SER_NUM)->x_data  = data_control._intval[1];
       first_half = TRUE;
    }
}
#endif /* C_ISR */

/*========================================================================*/
/* INIT_ARRAYS(): INITIALIZE DATA ARRAY PARAMETERS                        */
/*========================================================================*/
void init_arrays(int buffer_size)
{
     int i;
     /*----------------------------------------------------------------------*/
     /* INITIALIZE AND ZERO FILL ARRAYS                                      */
     /*--------------------------------------------------------------------  */
     if(!(input0 = (float *) calloc(buffer_size,sizeof(float))))
        heap_overflow();
     if(!(output0 = (float *) calloc(buffer_size,sizeof(float))))
        heap_overflow();
     if(!(input_xfer0 = (float *) calloc(buffer_size,sizeof(float))))
        heap_overflow();
     if(!(output_xfer0 = (float *) calloc(buffer_size,sizeof(float))))
        heap_overflow();
     if(!(input1 = (float *) calloc(buffer_size,sizeof(float))))
        heap_overflow();
     if(!(output1 = (float *) calloc(buffer_size,sizeof(float))))
        heap_overflow();
     if(!(input_xfer1 = (float *) calloc(buffer_size,sizeof(float))))
        heap_overflow();
     if(!(output_xfer1 = (float *) calloc(buffer_size,sizeof(float))))
        heap_overflow();

     for(i = 0; i < buffer_size; i++)
     {
        output0[i] = output_xfer0[i] = 0.0;
        output1[i] = output_xfer1[i] = 0.0;
     }
}

/**************************************************************************/
/* INIT_4215(): INITIALIZE COMMUNICATIONS TO CS4215                       */
/*             NOTE: i IS A VOLATILE TO FORCE TIME DELAYS AND TO FORCE    */
/*                  READS OF SERIAL PORT DATA RECEIVE REGISTER TO CLEAR   */
/*                  THE RECEIVE INTERRUPT FLAG                            */
/**************************************************************************/
void init_4215(int crystal, int sample_rate)
{
    VI i,j,dummy;
    CS4215_WORD     temp,in,out;

    RESET_CODEC;                                             /* RESET AIC */
    WAIT(50);                   /* KEEP RESET LOW FOR SOME PERIOD OF TIME */

    /**********************************************************************/
    /* CONFIGURE SERIAL PORT 1                                            */
    /**********************************************************************/
    SERIAL_PORT_ADDR(SER_NUM)->gcontrol = 0x0;

    SERIAL_PORT_ADDR(SER_NUM)->s_x_control = CLKXFUNC | DXFUNC | FSXFUNC;
    SERIAL_PORT_ADDR(SER_NUM)->s_r_control = CLKRFUNC | DRFUNC | FSRFUNC;

    SERIAL_PORT_ADDR(SER_NUM)->s_rxt_control = XGO | XHLD_ | XCP_ | XCLKSRC;

    /* THE FOLLOWING PERIOD REGISTER VALUE HAS BEEN TESTED ON A 50 MHz C30 */
    SERIAL_PORT_ADDR(SER_NUM)->s_rxt_period_bit.x_period = 0x3;

    SERIAL_PORT_ADDR(SER_NUM)->gcontrol = XCLKSRCE | XLEN_32 | XFSM | RFSM |
                                          RLEN_32 | XINT | RINT |
                                          FSXOUT | RRESET | XRESET;


    /* BUILD CONTROL WORDS */
    /* ALL BITS ARE 0 EXCEPT THOSE DEFINED OTHERWISE */

    temp._intval[0] = temp._intval[1] = 0;
    temp.control._bitval.st   = STEREO_MODE;
    temp.control._bitval.dfr  = sample_rate;
    temp.control._bitval.xclk = 1;
    temp.control._bitval.mckf = crystal;
    temp.control._bitval.pio  = 3;

    /* BUILD DATA CONTROL WORD */
    data_control._intval[0] = data_control._intval[1] = 0;
    data_control.stereo_16._bitval.lo   = ATT_0;
    data_control.stereo_16._bitval.le   = ON;
    data_control.stereo_16._bitval.ro   = ATT_0;
    data_control.stereo_16._bitval.ovr  = ON;
    data_control.stereo_16._bitval.ma   = MATT_15;

    UN_RESET_CODEC;                      /* PULL 4215 OUT OF RESET        */


    DCB_LOW;
    /* Write out control word until dcb bit is low */
    do
    {
       out = temp;
       for(i=0;i<5;i++)
       {
          while(SERIAL_PORT_ADDR(SER_NUM)->gcontrol_bit.xsrempty == 1);

          SERIAL_PORT_ADDR(SER_NUM)->gcontrol = 0x0;

          /* See note on XRESET/RRESET and three cycle delay in C3x U.G. */
          for(j=0;j<3;j++);

          SERIAL_PORT_ADDR(SER_NUM)->gcontrol = XCLKSRCE | XLEN_32 | XFSM |
                                                RFSM | RLEN_32 | XINT | RINT |
                                                FSXOUT | RRESET | XRESET;

          dummy = SERIAL_PORT_ADDR(SER_NUM)->r_data;

          SERIAL_PORT_ADDR(SER_NUM)->x_data = out._intval[0];

          /* See note on XRDY and three cycle delay in C3x U.G. */
          for(j=0;j<3;j++);

          while(SERIAL_PORT_ADDR(SER_NUM)->gcontrol_bit.xrdy == 0);

          SERIAL_PORT_ADDR(SER_NUM)->x_data = out._intval[1];




          while(SERIAL_PORT_ADDR(SER_NUM)->gcontrol_bit.rrdy == 0);

          in._intval[0] = SERIAL_PORT_ADDR(SER_NUM)->r_data;

          /* See note on RRDY and three cycle delay in C3x U.G. */
          for(j=0;j<3;j++);

          while(SERIAL_PORT_ADDR(SER_NUM)->gcontrol_bit.rrdy == 0);

          in._intval[1] = SERIAL_PORT_ADDR(SER_NUM)->r_data;
      }
    } while(in.control._bitval.dcb != 0);

    /* Write out control word twice with the dcb bit high */
    temp.control._bitval.dcb  = 1;
    out = temp;
    for(i=0;i<2;i++)
    {
       while(SERIAL_PORT_ADDR(SER_NUM)->gcontrol_bit.xsrempty == 1);

       SERIAL_PORT_ADDR(SER_NUM)->gcontrol = 0x0;

       /* See note on XRESET/RRESET and three cycle delay in C3x U.G. */
       for(j=0;j<3;j++);

       SERIAL_PORT_ADDR(SER_NUM)->gcontrol = XCLKSRCE | XLEN_32 | XFSM |
                                             RFSM | RLEN_32 | XINT | RINT |
                                             FSXOUT | RRESET | XRESET;

       dummy = SERIAL_PORT_ADDR(SER_NUM)->r_data;

       SERIAL_PORT_ADDR(SER_NUM)->x_data = out._intval[0];

       /* See note on XRDY and three cycle delay in C3x U.G. */
       for(j=0;j<3;j++);

       while(SERIAL_PORT_ADDR(SER_NUM)->gcontrol_bit.xrdy == 0);

       SERIAL_PORT_ADDR(SER_NUM)->x_data = out._intval[1];




       while(SERIAL_PORT_ADDR(SER_NUM)->gcontrol_bit.rrdy == 0);

       in._intval[0] = SERIAL_PORT_ADDR(SER_NUM)->r_data;

       /* See note on RRDY and three cycle delay in C3x U.G. */
       for(j=0;j<3;j++);

       while(SERIAL_PORT_ADDR(SER_NUM)->gcontrol_bit.rrdy == 0);

       in._intval[1] = SERIAL_PORT_ADDR(SER_NUM)->r_data;
    }

    SERIAL_PORT_ADDR(SER_NUM)->gcontrol = 0x0;
    SERIAL_PORT_ADDR(SER_NUM)->gcontrol = XLEN_32 | RLEN_32 | XFSM | RFSM |
                                          RRESET | XRESET | XCLKSRCE;


    while(SERIAL_PORT_ADDR(SER_NUM)->gcontrol_bit.xrdy == 0);
    SERIAL_PORT_ADDR(SER_NUM)->x_data = 0;
    /* See note on XRDY and three cycle delay in C3x U.G. */
    for(j=0;j<3;j++);

    while(SERIAL_PORT_ADDR(SER_NUM)->gcontrol_bit.xrdy == 0);

    SERIAL_PORT_ADDR(SER_NUM)->x_data = data_control._intval[1];

    dummy = SERIAL_PORT_ADDR(SER_NUM)->r_data;

    SERIAL_PORT_ADDR(SER_NUM)->gcontrol |= XINT | RINT;

    SERIAL_PORT_ADDR(SER_NUM)->gcontrol &= ~XCLKSRCE;

    SERIAL_PORT_ADDR(SER_NUM)->s_rxt_control = 0;

    CL_INT_FL_REG;

#if SER_NUM
    EN_SER_PORT_RCV_INT_1;
#else
    EN_SER_PORT_RCV_INT_0;
#endif

    EN_GLOBAL_INTS;

    DCB_HI;
}

