/**************************************************************************/
/*   CS4215.H                                                             */
/*                                                                        */
/*   TMS320C3x - CRYSTAL 4215 MM CODEC                                    */
/*      :TMS320C3x CODE                                                   */
/*                                                                        */
/*   Leor Brenman, DSP Applications                                       */
/*   (C) 1991 TEXAS INSTRUMENTS, HOUSTON                                  */
/**************************************************************************/
/*                                                                        */
/*   DEVICE INTERFACE:                                                    */
/*                                                                        */
/*                                                                        */
/*     TMS320C30 DSP                   CS4215 MM CODEC                    */
/*                                                                        */
/*                                                                        */
/*                              /_\  5 V                                  */
/*                               |                                        */
/*                               - Pull up Resistor                       */
/*                               |                                        */
/*         DR     ---------<---------- SDOUT                              */
/*                                                                        */
/*         DX     --------->---------- SDIN                               */
/*                                                                        */
/*         SCLK   -------------------- CLKX                               */
/*                                 |__ CLKR                               */
/*                                                                        */
/*         FSYNC  -------------------- FSX                                */
/*         TSIN   __|            |____ FSR                                */
/*                                                                        */
/*         D/C_   ---------<---------- XF                                 */
/*                                                                        */
/*         RESET  ---------<---------- TCLK                               */
/*                                                                        */
/*                                                                        */
/**************************************************************************/
#include <math.h>
#include <stdlib.h>
#include <c30_per.h>
#include <commdrvr.h>

/*========================================================================*/
/* MACROS *===============================================================*/
/*========================================================================*/
#define BLOCK_SIZE       64
#define SER_NUM          SERIAL_PORT_ONE
#define TIMER_NUM        TIMER_ONE
#define XF_NUM           1

#define INIT_ARRAYS      init_arrays(buffer_size)
#define WAIT_BUFFERS     while(!buffer_rdy);
#define RESET_FLAGS      buffer_rdy = FALSE
#define RESET_CODEC      TIMER_ADDR(TIMER_NUM)->gcontrol = I_O | HLD_
#define UN_RESET_CODEC   TIMER_ADDR(TIMER_NUM)->gcontrol = I_O | HLD_ | DATOUT
#if XF_NUM
#define DCB_LOW          asm("   AND  2fh,IOF"); asm("  OR 20h,IOF")
#define DCB_HI           asm("   OR   60h,IOF")
#else
#define DCB_LOW          asm("   AND 0F2h,IOF"); asm("  OR 2h,IOF")
#define DCB_HI           asm("   OR   6h,IOF")
#endif

#define WAIT(A)          for(i=0;i<A;i++);

#define C_ISR            ON

/**************************************************************************/
/*         CS4215 DATA COMMAND BIT FIELD DATA STRUCTURES                  */
/**************************************************************************/
/**************************************************************************/
/*   CONTROL COMMAND                                                      */
/**************************************************************************/
typedef union
{
  unsigned int _intval[2];
  struct
  {
    /* Time slot 4 */
    unsigned int adl     :1;     /* Loopback mode                         */
    unsigned int enl     :1;     /* Enable loopback testing               */
    unsigned int d_r5    :6;     /* Unused - don't care bits: 2 - 7       */

    /* Time slot 3 */
    unsigned int xen     :1;     /* Transmitter enable                    */
    unsigned int xclk    :1;     /* Transmit clock                        */
    unsigned int bsel    :2;     /* Select bit rate                       */
    unsigned int mckf    :2;     /* Clock source select                   */
    unsigned int d_r4    :2;     /* Unused - don't care bits: 6 - 7       */

    /* Time slot 2 */
    unsigned int df      :2;     /* Data format selection                 */
    unsigned int st      :1;     /* Stereo bit: 0-mono, 1-stereo          */
    unsigned int dfr     :3;     /* Data conversion freq selection        */
    unsigned int d_r3    :2;     /* Unused - don't care bits: 6 - 7       */

    /* Time slot 1 */
    unsigned int d_r1    :2;     /* Unused - don't cares bits: 0 - 1      */
    unsigned int dcb     :1;     /* Data control handshake bit            */
    unsigned int d_r2    :5;     /* Unused - don't cares bits: 3 - 7      */

    /* Time slot 8 */
    unsigned int d_r9    :8;     /* Unused - don't care bits: 0 - 7       */

    /* Time slot 7 */
    unsigned int rv      :4;     /* Revision level of the CS4215          */
    unsigned int d_r8    :4;     /* Unused - don't care bits: 4 - 7       */

    /* Time slot 6 */
    unsigned int d_r7    :8;     /* Unused - don't care bits: 0 - 7       */

    /* Time slot 5 */
    unsigned int d_r6    :6;     /* Unused - don't care bits: 0 - 5       */
    unsigned int pio     :2;     /* Parallel port control                 */
  } _bitval;
} CONTROL;

/**************************************************************************/
/*   DATA COMMANDS                                                        */
/**************************************************************************/
typedef union
{
  unsigned int _intval[2];
  struct
  {
    /* Time slots 3 & 4 */
      signed int right   :16;    /* Right channel 16 bit                  */

    /* Time slots 1 & 2 */
      signed int left    :16;    /* Left channel 16 bit                   */

    /* Time slot 8 */
    unsigned int rg      :4;     /* Right input gain settings             */
    unsigned int ma      :4;     /* Monitor path selection                */

    /* Time slot 7 */
    unsigned int lg      :4;     /* Left input gain settings              */
    unsigned int is      :1;     /* Input selection                       */
    unsigned int ovr     :1;     /* Overange                              */
    unsigned int pio     :2;     /* Parallel I/O bits                     */

    /* Time slot 6 */
    unsigned int ro      :6;     /* Right output attenuation setting      */
    unsigned int se      :1;     /* Speaker output enable control         */
    unsigned int d_r1    :1;     /* Unused - don't care bit 7             */

    /* Time slot 5 */
    unsigned int lo      :6;     /* Left output attenuation setting       */
    unsigned int le      :1;     /* Parallel output enable control        */
    unsigned int he      :1;     /* Headphone output enable control       */
  } _bitval;
} STEREO_16;

typedef union
{
  unsigned int _intval[2];
  struct
  {
    /* Time slots 3 & 4 */
      signed int d_r1    :16;    /* Unused - don't care bits 0 - 15       */

    /* Time slots 1 & 2 */
      signed int left    :16;    /* Left channel 16 bit                   */

    /* Time slot 8 */
    unsigned int d_r3    :4;     /* Unused - don't care bits: 0 - 3       */
    unsigned int ma      :4;     /* Monitor path selection                */

    /* Time slot 7 */
    unsigned int lg      :4;     /* Left input gain settings              */
    unsigned int is      :1;     /* Input selection                       */
    unsigned int ovr     :1;     /* Overange                              */
    unsigned int pio     :2;     /* Parallel I/O bits                     */

    /* Time slot 6 */
    unsigned int ro      :6;     /* Right output attenuation setting      */
    unsigned int se      :1;     /* Speaker output enable control         */
    unsigned int d_r2    :1;     /* Unused - don't care bit 7             */

    /* Time slot 5 */
    unsigned int lo      :6;     /* Left output attenuation setting       */
    unsigned int le      :1;     /* Parallel output enable control        */
    unsigned int he      :1;     /* Headphone output enable control       */
  } _bitval;
} MONO_16;

typedef union
{
  unsigned int _intval[2];
  struct
  {
    /* Time slots 4 */
      signed int d_r2    :8;     /* Unused - don't care bits 0 - 7        */

    /* Time slot 3 */
      signed int right   :8;     /* Right channel 8 bit                   */

    /* Time slots 2 */
      signed int d_r1    :8;     /* Unused - don't care bits 0 - 7        */

    /* Time slot 1 */
      signed int left    :8;     /* Left channel 8 bit                    */

    /* Time slot 8 */
    unsigned int rg      :4;     /* Right input gain settings             */
    unsigned int ma      :4;     /* Monitor path selection                */

    /* Time slot 7 */
    unsigned int lg      :4;     /* Left input gain settings              */
    unsigned int is      :1;     /* Input selection                       */
    unsigned int ovr     :1;     /* Overange                              */
    unsigned int pio     :2;     /* Parallel I/O bits                     */

    /* Time slot 6 */
    unsigned int ro      :6;     /* Right output attenuation setting      */
    unsigned int se      :1;     /* Speaker output enable control         */
    unsigned int d_r3    :1;     /* Unused - don't care bit 7             */

    /* Time slot 5 */
    unsigned int lo      :6;     /* Left output attenuation setting       */
    unsigned int le      :1;     /* Parallel output enable control        */
    unsigned int he      :1;     /* Headphone output enable control       */
  } _bitval;
} STEREO_8;

typedef union
{
  unsigned int _intval[2];
  struct
  {
    /* Time slots 2 - 4 */
      signed int d_r1    :24;    /* Unused - don't care bits 0 - 23       */

    /* Time slot 1 */
      signed int left    :8;     /* Left channel 8 bit                    */

    /* Time slot 8 */
    unsigned int d_r3    :4;     /* Unused - don't care bits: 0 - 3       */
    unsigned int ma      :4;     /* Monitor path selection                */

    /* Time slot 7 */
    unsigned int lg      :4;     /* Left input gain settings              */
    unsigned int is      :1;     /* Input selection                       */
    unsigned int ovr     :1;     /* Overange                              */
    unsigned int pio     :2;     /* Parallel I/O bits                     */

    /* Time slot 6 */
    unsigned int ro      :6;     /* Right output attenuation setting      */
    unsigned int se      :1;     /* Speaker output enable control         */
    unsigned int d_r2    :1;     /* Unused - don't care bit 7             */

    /* Time slot 5 */
    unsigned int lo      :6;     /* Left output attenuation setting       */
    unsigned int le      :1;     /* Parallel output enable control        */
    unsigned int he      :1;     /* Headphone output enable control       */
  } _bitval;
} MONO_8;

typedef union
{
  unsigned int     _intval[2];
  CONTROL           control;
  STEREO_16         stereo_16;
  MONO_16           mono_16;
  STEREO_8          stereo_8;
  MONO_8            mono_8;
} CS4215_WORD;

/*========================================================================*/
/* GLOBAL VARIABLES *=====================================================*/
/*========================================================================*/
extern int  buffer_size;      /* SIZE OF I/O BUFFER(S)                   */
extern VPVF output0;          /* OUTPUT DATA BUFFER FOR PROCESSOR        */
extern VPVF input0;           /* INPUT DATA BUFFER FOR PROCESSOR         */
extern VPVF output_xfer0;     /* OUTPUT DATA BUFFER FOR ISR/AIC          */
extern VPVF input_xfer0;      /* INPUT DATA BUFFER FOR ISR/AIC           */
extern VPVF output1;          /* OUTPUT DATA BUFFER FOR PROCESSOR        */
extern VPVF input1;           /* INPUT DATA BUFFER FOR PROCESSOR         */
extern VPVF output_xfer1;     /* OUTPUT DATA BUFFER FOR ISR/AIC          */
extern VPVF input_xfer1;      /* INPUT DATA BUFFER FOR ISR/AIC           */
extern VI   buffer_rdy;       /* CPU-ISR COMM FLAG (INPUT)               */
extern VI   buffer_index;     /* INDEX INTO INPUT AND OUTPUT DATA ARRAYS */
extern VI   i;                /* GENERIC COUNTER VARIABLE                */
extern VI   first_half;

extern CS4215_WORD     data_control;

/**************************************************************************/
/*	   FUNCTION PROTOTYPES						  */
/**************************************************************************/
/***************************/
/* CS4215 DRIVER FUNCTIONS */
/***************************/
void init_arrays(int buffer_size);
void init_4215(int crystal, int sample_rate);
#if SER_NUM
void c_int07(void);
#else
void c_int05(void);
#endif

/**************************************************************************/
/*         CS4215 DATA COMMAND BIT FIELD MACROS                           */
/**************************************************************************/
/**************************************************************************/
/*   CONTROL COMMAND MACROS                                               */
/**************************************************************************/
#define DATA               1
#define COMM               0
#define SIXTEEN_BIT_LINEAR 0
#define EIGHT_BIT_U_LAW    1
#define EIGHT_BIT_A_LAW    2
#define MONO_MODE          0
#define STEREO_MODE        1

/* Data conversion Frequency Selections Assumes that XTAL1 = 24.576 MHz   */
/* And XTAL2 = 16.9344 MHz.                                               */
/*                                            XTAL1 (kHz) | XTAL2 (kHz)   */
/*                                            =========================   */
#define CONV_FREQ_0        0           /*         8.00000 |  5.5125       */
#define CONV_FREQ_1        1           /*        16.00000 | 11.0250       */
#define CONV_FREQ_2        2           /*        27.42857 | 18.9000       */
#define CONV_FREQ_3        3           /*        32.00000 | 22.0500       */
#define CONV_FREQ_4        4           /*        NA       | 37.8000       */
#define CONV_FREQ_5        5           /*        NA       | 44.1000       */
#define CONV_FREQ_6        6           /*        48.00000 | 33.0750       */
#define CONV_FREQ_7        7           /*         9.60000 |  6.6150       */

#define CS_ENABLE          0                       /* Data output enabled */
#define CS_DISABLE         1                       /* Data output disabled*/

#define CS_TCLOCK_EXT      0                  /* FSYNC and SCLK are inputs*/
#define CS_TCLOCK_INT      1                 /* FSYNC and SCLK are outputs*/


#define BPF_64             0                         /* 64 bits per frame */
#define BPF_128            1                        /* 128 bits per frame */
#define BPF_256            2                        /* 256 bits per frame */

#define CS_CLOCK_SCLK      0                 /* Clock source select: SCLK */
#define CS_CLOCK_XTAL1     1                 /* Clock source select: XTAL1*/
#define CS_CLOCK_XTAL2     2                 /* Clock source select: XTAL2*/
#define CS_CLOCK_EXT       3                 /* Clock source select: Ext  */

#define DIGITAL_LOOPBACK   0
#define ANALOG_LOOPBACK    1

#define LOOP_ENABLE        1
#define LOOP_DISABLE       0

/**************************************************************************/
/*   DATA COMMAND MACROS                                                  */
/**************************************************************************/
/* Output attenuation is 1.5 dB per unit integer value                    */
/*                                        Attenuation (dB)                */
/*                                        ================                */
#define ATT_0              0           /*      0.0                        */
#define ATT_1              1           /*      1.5                        */
#define ATT_2              2           /*      3.0                        */
#define ATT_3              3           /*      4.5                        */
#define ATT_4              4           /*      6.0                        */
#define ATT_5              5           /*      7.5                        */
#define ATT_6              6           /*      9.0                        */
#define ATT_7              7           /*     10.5                        */
#define ATT_8              8           /*     12.0                        */
#define ATT_9              9           /*     13.5                        */
#define ATT_10             10          /*     15.0                        */
#define ATT_11             11          /*     16.5                        */
#define ATT_12             12          /*     18.0                        */
#define ATT_13             13          /*     19.5                        */
#define ATT_14             14          /*     21.0                        */
#define ATT_15             15          /*     22.5                        */
#define ATT_16             16          /*     24.0                        */
#define ATT_17             17          /*     25.5                        */
#define ATT_18             18          /*     27.0                        */
#define ATT_19             19          /*     28.5                        */
#define ATT_20             20          /*     30.0                        */
#define ATT_21             21          /*     31.5                        */
#define ATT_22             22          /*     33.0                        */
#define ATT_23             23          /*     34.5                        */
#define ATT_24             24          /*     36.0                        */
#define ATT_25             25          /*     37.5                        */
#define ATT_26             26          /*     39.0                        */
#define ATT_27             27          /*     40.5                        */
#define ATT_28             28          /*     42.0                        */
#define ATT_29             29          /*     43.5                        */
#define ATT_30             30          /*     45.0                        */
#define ATT_31             31          /*     46.5                        */
#define ATT_32             32          /*     48.0                        */
#define ATT_33             33          /*     49.5                        */
#define ATT_34             34          /*     51.0                        */
#define ATT_35             35          /*     52.5                        */
#define ATT_36             36          /*     54.0                        */
#define ATT_37             37          /*     55.5                        */
#define ATT_38             38          /*     57.0                        */
#define ATT_39             39          /*     58.5                        */
#define ATT_40             40          /*     60.0                        */
#define ATT_41             41          /*     61.5                        */
#define ATT_42             42          /*     63.0                        */
#define ATT_43             43          /*     64.5                        */
#define ATT_44             44          /*     66.0                        */
#define ATT_45             45          /*     67.5                        */
#define ATT_46             46          /*     69.0                        */
#define ATT_47             47          /*     70.5                        */
#define ATT_48             48          /*     72.0                        */
#define ATT_49             49          /*     73.5                        */
#define ATT_50             50          /*     74.0                        */
#define ATT_51             51          /*     75.5                        */
#define ATT_52             52          /*     77.0                        */
#define ATT_53             53          /*     78.5                        */
#define ATT_54             54          /*     80.0                        */
#define ATT_55             55          /*     81.5                        */
#define ATT_56             56          /*     83.0                        */
#define ATT_57             57          /*     84.5                        */
#define ATT_58             58          /*     87.0                        */
#define ATT_59             59          /*     88.5                        */
#define ATT_60             60          /*     90.0                        */
#define ATT_61             61          /*     91.5                        */
#define ATT_62             62          /*     93.0                        */
#define ATT_63             63          /*     94.5                        */

#define HEADPHONE_OFF      0
#define HEADPHONE_ON       1

#define LINE_OUT_OFF       0
#define LINE_OUT_ON        1

#define SPEAKER_OFF        0
#define SPEAKER_ON         1

/* Input gain is 1.5 dB per unit integer value                            */
/*                                        Gain (dB)                       */
/*                                        =========                       */
#define GAIN_0             0           /*      0.0                        */
#define GAIN_1             1           /*      1.5                        */
#define GAIN_2             2           /*      3.0                        */
#define GAIN_3             3           /*      4.5                        */
#define GAIN_4             4           /*      6.0                        */
#define GAIN_5             5           /*      7.5                        */
#define GAIN_6             6           /*      9.0                        */
#define GAIN_7             7           /*     10.5                        */
#define GAIN_8             8           /*     12.0                        */
#define GAIN_9             9           /*     13.5                        */
#define GAIN_10            10          /*     15.0                        */
#define GAIN_11            11          /*     16.5                        */
#define GAIN_12            12          /*     18.0                        */
#define GAIN_13            13          /*     19.5                        */
#define GAIN_14            14          /*     21.0                        */
#define GAIN_15            15          /*     22.5                        */

#define LINE_IN            0
#define MIKE_IN            1

#define OVERANGE_ENABLE    1
#define OVERANGE_CLEAR     0

/* Monitor path attenuation = 6 dB per unit integer value                 */
/*                                        Gain (dB)                       */
/*                                        =========                       */
#define MATT_0             0           /*      6.0                        */
#define MATT_1             1           /*     12.0                        */
#define MATT_2             2           /*     18.0                        */
#define MATT_3             3           /*     24.0                        */
#define MATT_4             4           /*     30.0                        */
#define MATT_5             5           /*     36.0                        */
#define MATT_6             6           /*     42.0                        */
#define MATT_7             7           /*     48.0                        */
#define MATT_8             8           /*     54.0                        */
#define MATT_9             9           /*     60.0                        */
#define MATT_10            10          /*     66.0                        */
#define MATT_11            11          /*     72.0                        */
#define MATT_12            12          /*     78.0                        */
#define MATT_13            13          /*     84.0                        */
#define MATT_14            14          /*     90.0                        */
#define MATT_15            15          /*     96.0  (Mute Monitor Path)   */
