/****************************************************************/
/*   INTERFACING THE WE(R) DSP16 DIGITAL SIGNAL PROCESSOR	*/
/*   TO MULTIPLE A/D AND D/A CONVERTERS				*/
/*   November, 1989  Doc# AP89-015DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1989 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*			csport.s				*/
/****************************************************************/
/* This program segment illustrates the use of the		*/
/* Control/Status port.  Each time an interrupt occurs, this	*/
/* program segment reads the control switches and writes the	*/
/* settings to the status LEDs.  Then, the program branches to	*/
/* the task corresponding to the selected switch.		*/

#define  ADDR_PORT	pdx0
#define  DATA_PORT	pdx1
#define  PORT_0		0x001		/* bit A0 */
#define  PORT_1		0x002		/* bit A1 */
#define  PORT_2		0x004		/* bit A2 */
#define  PORT_3		0x008		/* bit A3 */
#define  PORT_CS	0x010		/* bit A4 */
#define  NOP		a0=a1		/* used to separate PIO read/writes */

.ram
partsum: int		/* partial sum	*/
.endram

rst:	goto start			/* jump to main program		*/

intdriver:

	/* Interrupt service routine.  First, determine which bit of	*/
	/* the CS port is set.						*/
	y = 0xff00
	ADDR_PORT = PORT_CS		/* select CS channel		*/
	NOP
	a1 = DATA_PORT			/* read dummy, D0 in buffer	*/
	a1 = DATA_PORT			/* read D0, D0 in buffer	*/
	a1 = a1 << 8			/* shift up 8 bits		*/
	a1 = a1^y			/* exclusive OR with 0xff00	*/
	DATA_PORT = a1			/* write inverted settings to LEDs */
	a1 = a1^y			/* exclusive OR with 0xff00	*/

    /*  Test bits - find the first one that is set	*/

	a1 = a1 << 4
	if mi goto intcs3		/* if CS3 is set, goto intcs3	*/
	a1 = a1 << 1
	if mi goto intcs2		/* if CS2 is set, goto intcs2	*/
	a1 = a1 << 1
	if mi goto intcs1		/* if CS1 is set, goto intcs1	*/
	a1 = a1 << 1
	if mi goto intcs0		/* if CS0 is set, goto intcs0	*/

	goto intend			/* else, exit interrupt routine	*/


/* CS3 ******************************************************************/
	/* This is where the code corresponding to switch 3 would be.	*/
	/* For now, do nothing but exit					*/
intcs3: goto intend


/* CS2 ******************************************************************/
	/* This is where the code corresponding to switch 2 would be.	*/
	/* For now, do nothing but exit					*/
intcs2: goto intend


/* CS1 ******************************************************************/
	/* This is where the code corresponding to switch 1 would be.	*/
	/* For now, do nothing but exit					*/
intcs1: goto intend


/* CS0 ******************************************************************/
	/* This is where the code corresponding to switch 0 would be.	*/
	/* For now, do nothing but exit					*/
intcs0: goto intend


intend: ireturn

/* start of main program ************************************************/

start:	pioc = 0x3820		/* pids, pods active, int active	*/
	auc = 0x00

noploop: a0 = a1
	 goto noploop

