;**************************************************************
;  
;                 c_int.asm
;  
;                 Leor Brenman
;  
;                 03-16-92
;  
;           (C) Texas Instruments Inc., 1992 
;  
;           Refer to the file 'license.txt' included with this 
;           this package for usage and license information. 
;  
;**************************************************************
**********************************************************
* C_INT08(VOID)
*
* Hand-coded assembly language interrupt service routine.
* This serial port transmitt ISR supports the CS4215 zero
* chip I/F to the C3x serial port
* This ISR has been hand-coded for speed optimization.
*
* Leor Brenman, DSP Applications
* (C) 1991 TEXAS INSTRUMENTS, HOUSTON
**********************************************************

        .globl  _c_int08

******************
* global variables
******************
        .global _first_half, _input_xfer0, _input_xfer1, _buffer_size
        .global _buffer_index, _output_xfer0
        .global _output_xfer1, _output0, _output1, _data_control
        .global _buffer_rdy, _input0, _input1

******************
* global variables
******************
        .data
SER_1   .word   808050h    ;place in same page as .bss
                           ;to eliminate push/pop of DP when loading
                           ;serial port one's base address

******************************************************
* FUNCTION DEF : _c_int08
******************************************************
        .text
_c_int08:
	PUSH	ST
	PUSH	R0
	PUSHF	R0
        PUSH    AR0

*******************************************************************
* if this is the first half of the transmission then goto FRST_HALF
*******************************************************************

	LDI	@_first_half,R0
        BNZ     FRST_HALF

************************************************
* else, this the second half of the transmission
************************************************
SCND_HALF:

*********************************************************************
* load AR0 with serial port base address
* do dummy read of serial port to empty control info from serial port
*********************************************************************

        LDI     @SER_1,AR0
        LDI     *+AR0(12),R0

**************************************************************************
* get control value and write to serial port while branching to end of ISR
* and set first_half flag to TRUE
**************************************************************************

	LDI	@_data_control+1,R0
        BD      FIN_S
        STI     R0,*+AR0(8)
        LDI     1,R0
        STI     R0,@_first_half

******************************************
* This the second half of the transmission
******************************************
FRST_HALF:

**************************
* push remaining registers
**************************

	PUSH	R1
	PUSHF	R1
        PUSH    AR1
	PUSH	IR0

******************************
* set first_half flag to FALSE
******************************

	LDI	0,R0
	STI	R0,@_first_half

******************************************
* load AR0 with serial port 1 base address
* read serial port data received
******************************************

        LDI     @SER_1,AR0
        LDI     *+AR0(12),R0

****************************************************
* parse left and right channels and convert to float
* put into buffers: input_xfer0 and intput_xfer1
****************************************************

        LDI     R0,R1
	ASH	16,R1
	ASH	-16,R1
	FLOAT	R1
        LDI     @_input_xfer0,AR1
        LDI     @_buffer_index,IR0
        STF     R1,*+AR1(IR0)

        LDI     R0,R1
	ASH	-16,R1
	FLOAT	R1
        LDI     @_input_xfer1,AR1
        LDI     @_buffer_index,IR0
        STF     R1,*+AR1(IR0)

***************************************************
* convert from float to int and interleave left and
* right channels from buffers output_xfer0 and 1
***************************************************

        LDI     @_output_xfer1,AR1
        LDI     @_buffer_index,IR0
        FIX     *+AR1(IR0),R0
        ASH     16,R0

        LDI     @_output_xfer0,AR1
        LDI     @_buffer_index,IR0
        FIX     *+AR1(IR0),R1
	AND	0ffffh,R1

        OR      R0,R1,R0
        STI     R0,*+AR2(8)

*********************************************************************
* check to see if output buffers are empty and input buffers are full
* if so, first check to see if real time has been met, if so, swap
* with buffers input0, input1, output0 and output1 else, go to end
*********************************************************************

        LDI     @_buffer_index,R0
        ADDI    1,R0
        STI     R0,@_buffer_index
        LDI     @_buffer_size,R1
        CMPI    R1,R0
        BNZ     FIN_F

******************************************
* check to see if buffer_rdy is still TRUE
* if buffer_rdy is still TRUE, real time
* has not been met. branch to self.
******************************************

        LDI     @_buffer_rdy,R0
SELF    BNZ     SELF

*****************************
* swap input0 and input_xfer0
*****************************

        LDI     @_input0,R0
        LDI     @_input_xfer0,R1
        STI     R0,@_input_xfer0
        STI     R1,@_input0

*****************************
* swap input1 and input_xfer1
*****************************

        LDI     @_input1,R0
        LDI     @_input_xfer1,R1
        STI     R0,@_input_xfer1
        STI     R1,@_input1

*******************************
* swap output0 and output_xfer0
*******************************

        LDI     @_output0,R0
        LDI     @_output_xfer0,R1
        STI     R0,@_output_xfer0
        STI     R1,@_output0

*******************************
* swap output1 and output_xfer1
*******************************

        LDI     @_output1,R0
        LDI     @_output_xfer1,R1
        STI     R0,@_output_xfer1
        STI     R1,@_output1

*************************
* reset buffer_index to 0
*************************

        LDI     0,R0
        STI     R0,@_buffer_index

************************
* set buffer_rdy to TRUE
************************

        LDI     1,R0
        STI     R0,@_buffer_rdy


FIN_F:
	POP	IR0
        POP     AR1
	POPF	R1
	POP	R1

FIN_S:
        POP     AR0
	POPF	R0
	POP	R0
	POP	ST

	RETI
