/*
   DEMO2.C

   This C program is part of the ADSP-21xx Simulator Demonstration.  It
   performs low-pass filtering of a buffer that is first filled with random
   values.  The filtering routine and the random number generator are
   C-callable assembly routines.

*/

/* C Preprocessor Stuff */
#include <stdlib.h>     /* For rand(), srand() */

#define TAPS            22
#define DATA_SIZE       100

/* Function Prototypes */
void initialize_data( int buffer[], int size );

/* Externs */
extern int state[];                                 /* filter state */
extern void fir_init( void );                       /* called once */
extern int fir_filt( int sample, int pm coeffs[] ); /* filter routine */

/* main() */
int main( void )
{
   int i;
   int sample;                  /* input to filtering routine */

                                /* filter coefficients */
   static int pm coeffs[TAPS] = { 0x0015, 0x008F, 0x014B, 0x018D, 0x004A,
                                  0xFD5A, 0xFA64, 0xFA90, 0x008F, 0x0C2C,
                                  0x1965, 0x2239, 0x1965, 0x0C2C, 0x008F,
                                  0xFA90, 0xFA64, 0xFD5A, 0x004A, 0x018D,
                                  0x014B, 0x008F };

   static int data_buffer[DATA_SIZE];   /* data to be filtered (in place) */

   fir_init();       /* initialize filter routine */
   srand( 0xb5ed );  /* seed value for random number generator */
                     
   /* initialize input data */
   initialize_data( data_buffer, DATA_SIZE );

   /* filter data */
   for( i=0; i < DATA_SIZE; i++)
   {
      sample = data_buffer[i];
      data_buffer[i] = fir_filt( sample, coeffs );
   }

   /* done */
   return( 1 );
}

void initialize_data( int buffer[], int size )
{
   int i;

   for( i=0; i < size; i++ )
      buffer[i] = ( 2*rand() ) - 32768;  /* set buffer to random values */
}
