;***********************************************************
; DIAL.ASM
; This program generates a dial tone, (350Hz + 440Hz)
; Dial.asm is an adaptation of:
; "Precision Digital Sine-Wave Generation with the
; TMS32010" by Domingo Garcia, Texas Instruments
; found in "Digital Signal Processing Applications
; with the TMS320 Family, Volume 1.
;
; This program also demonstrates the use of the timer
; interrupt.
; We want a timer interrupt to occur every
; 125 microseconds (an 8 Khz rate)
; The PRD, period register, determines the number of
; clk cycles which are counted down prior to a timer
; interrupt.
; For a 40Mhz TMS320C25, PRD ------> 1250,
; For a 36Mhz TMS320C25, PRD ------> 1125,
; For a 20Mhz TMS320C25, PRD ------> 625,
; to yield an 8,000 Hz timeout rate
;
; The tone variables, Ton350 and Ton440, determine the
; frequency of the resulting sine waves by the
; formula:  tone = desired frequency/.25
; For example: Ton350 = 350Hz/.25 = 1400
;      and,    Ton440 = 440Hz/.25 = 1760
;
;**********************************************************
;CONSTANTS
PRDVAL  EQU     1125
TON350  EQU     1400       ; value for 350 Hz
TON440  EQU     1760       ; value for 440 Hz
;PAGE 0 VARIABLES
DEL350  EQU     96
ALP350  EQU     97
SIN350  EQU     98
TEMP    EQU     99
MASK    EQU     100
OFSET   EQU     101
ALP480  EQU     102
DEL480  EQU     103
SIN480  EQU     104
TMP0    EQU     105
; *********   SET UP VECTORS FOR THE DXP-25 BOARD ********
        ORG     8000H
        B       STRT
        ORG     8002H          ;INT0
        NOP
        NOP
        ORG     8004H          ;TIMER
        B       TMRINT
        ORG     8006H          ;SERIAL PORT RX
        NOP
        NOP
        ORG     8008H          ;SERIAL PORT TX
        NOP
        NOP
        ;
STRT:   LDPK    0
        LALK    07FFFH
        SACL    MASK
        LALK    SINE
        SACL    OFSET
        ZAC
        SACL    ALP350
        SACL    ALP480
        LALK    TON350
        SACL    DEL350
        LALK    TON440
        SACL    DEL480
        LALK    0E00H            ;SET STATUS 0 REG
        SACL    TMP0
        LST     TMP0
        LALK    03F0H            ;SET STATUS 1 REG
        SACL    TMP0
        LST1    TMP0
        ZAC
        SACL    TEMP
        LACK    1
        SACL    TMP0
        LALK    PRDVAL           ;SET PRD REGISTER
        SACL    PRD              ;FOR A 8KHZ TIMEOUT RATE
        LACK    008H             ;INTERRUPT MASK FOR
        SACL    IMR              ;TIMER INTR ONLY
        EINT
LOOP:   B       LOOP
TMRINT: LAC     ALP350,8
        SACH    TEMP
        LAC     TEMP
        ADD     OFSET
        TBLR    SIN350
        LAC     ALP350
        ADD     DEL350
        AND     MASK
        SACL    ALP350
        LAC     ALP480,8
        SACH    TEMP
        LAC     TEMP
        ADD     OFSET
        TBLR    SIN480
        LAC     ALP480
        ADD     DEL480
        AND     MASK
        SACL    ALP480
        LAC     SIN350
        ADD     SIN480
        ANDK    0FFFCH
        SACL    DXR
        EINT
        RET
;**********************************************************
; SINE - 128 points of a sine wave in Q14 2's complement
;        format
;**********************************************************
SINE:   DW    0
        DW    0324H
        DW    0646H
        DW    0964H
        DW    0C7CH
        DW    0F8DH
        DW    01294H
        DW    01590H
        DW    0187EH
        DW    01B5DH
        DW    01E2BH
        DW    020E7H
        DW    0238EH
        DW    02620H
        DW    0289AH
        DW    02AFBH
        DW    02D41H
        DW    02F6CH
        DW    03179H
        DW    03368H
        DW    03537H
        DW    036E5H
        DW    03871H
        DW    039DBH
        DW    03B21H
        DW    03C42H
        DW    03D3FH
        DW    03E15H
        DW    03EC5H
        DW    03F4FH
        DW    03FB1H
        DW    03FECH
        DW    04000H
        DW    03FECH
        DW    03FB1H
        DW    03F4FH
        DW    03EC5H
        DW    03E15H
        DW    03D3FH
        DW    03C42H
        DW    03B21H
        DW    039DBH
        DW    03871H
        DW    036E5H
        DW    03537H
        DW    03368H
        DW    03179H
        DW    02F6CH
        DW    02D41H
        DW    02AFBH
        DW    0289AH
        DW    02620H
        DW    0238EH
        DW    020E7H
        DW    01E2BH
        DW    01B5DH
        DW    0187EH
        DW    01590H
        DW    01294H
        DW    0F8DH
        DW    0C7CH
        DW    0964H
        DW    0646H
        DW    0324H
        DW    00H
        DW    0FCDCH
        DW    0F9BAH
        DW    0F69CH
        DW    0F384H
        DW    0F073H
        DW    0ED6CH
        DW    0EA70H
        DW    0E782H
        DW    0E4A3H
        DW    0E1D5H
        DW    0DF19H
        DW    0DC72H
        DW    0D9E0H
        DW    0D766H
        DW    0D505H
        DW    0D2BFH
        DW    0D094H
        DW    0CE87H
        DW    0CC98H
        DW    0CAC9H
        DW    0C91BH
        DW    0C78FH
        DW    0C625H
        DW    0C4DFH
        DW    0C3BEH
        DW    0C2C1H
        DW    0C1EBH
        DW    0C13BH
        DW    0C0B1H
        DW    0C04FH
        DW    0C014H
        DW    0C000H
        DW    0C014H
        DW    0C04FH
        DW    0C0B1H
        DW    0C13BH
        DW    0C1EBH
        DW    0C2C1H
        DW    0C3BEH
        DW    0C4DFH
        DW    0C625H
        DW    0C78FH
        DW    0C91BH
        DW    0CAC9H
        DW    0CC98H
        DW    0CE87H
        DW    0D094H
        DW    0D2BFH
        DW    0D505H
        DW    0D766H
        DW    0D9E0H
        DW    0DC72H
        DW    0DF19H
        DW    0E1D5H
        DW    0E4A3H
        DW    0E782H
        DW    0EA70H
        DW    0ED6CH
        DW    0F073H
        DW    0F384H
        DW    0F69CH
        DW    0F9BAH
        DW    0FCDCH
        END
