
/* These are the DSP32C DS Board Register mappings  
** in DSP32C 8 bit mode PCR[9] = 0, PCR[1] = 1 */

#define   BASE     0x300	/* define default dspbase address 
				* need 32 contiguous addresses described
				* in PC I/O slot address map. */
#define   PAR      dspbase
#define   PARL     dspbase		/* PIO Address Register low byte */
#define   PARH     dspbase+0x01	/* PIO Address Register high byte */
#define   PDRL     dspbase+0x02	/* PIO Data Register low byte */
#define   PDRH     dspbase+0x03	/* PIO Data Register high byte */
#define   EMRL     dspbase+0x04	/* Error Mask Register low byte */
#define   EMRH     dspbase+0x05	/* Error Mask Register high byte */
#define   ESR      dspbase+0x06	/* Error Source Register */
#define   PCRL     dspbase+0x07	/* PIO Control Register low byte */
#define   PIRL     dspbase+0x08	/* PIO Interrupt Register low byte */
#define   PIRH     dspbase+0x09	/* PIO Interrupt Register high byte */

#define   PCRH     dspbase+0x0A	/* PIO Control Register high byte */
#define   PARE     dspbase+0x0B	/* PAR Extended */
#define   PDR2L    dspbase+0x0C	/* PIO Data Register 2 low byte */
#define   PDR2H    dspbase+0x0D	/* PIO Data Register 2 high byte */

#define   STATRD   dspbase+0x10	/* Polled status register */
#define   CNTLWR2  dspbase+0x14	/* SIO port interface control */
#define   CNTLWR   dspbase+0x18	/* Additional SIO port interface control */
#define   INITWR   dspbase+0x1c	/* Board init control register */

/* These are the bit defines for INITWR */
#define   CONTZN   	0x40	/* if set Zn pin is deasserted */
#define   MMD2		0x20	/* Set MMD2 bit of DSP */
#define   MMD1NOT	0x10	/* Set ~MMD1 bit of DSP */
#define   MMD0		0x08	/* Set MMD0 bit of DSP */
#define   IRQENABLE	0x04	/* Enable PC path to IRQ */
#define   IRQ		0x02	/* Simulator input to IRQ for breakpointing */
#define   BPCNTL	0x01	/* if set disables interrupt logic
				to SIO header */

/* memory modes */
#define   MMODE0       ((0 ^ 2) << 3)
#define   MMODE1       ((1 ^ 2) << 3)
#define   MMODE2       ((2 ^ 2) << 3)
#define   MMODE3       ((3 ^ 2) << 3)
#define   MMODE4       ((4 ^ 2) << 3)
#define   MMODE5       ((5 ^ 2) << 3)
#define   MMODE6       ((6 ^ 2) << 3)
#define   MMODE7       ((7 ^ 2) << 3)

/* These are the bit defines for CNTLWR */
#define   ICKIH         0x80	/* If set, ICK is generated externally */
#define   ICKOH         0x40	/* If set, ICK is generated internally */
#define   ILDIH         0x20	/* If set, ILD is generated externally */
#define   ILDOH         0x10	/* If set, ILD is generated internally */
#define   OCKIH         0x08	/* If set, OCK is generated externally */
#define   OCKOH         0x04	/* If set, OCK is generated internally */
#define   OLDIH         0x02	/* If set, OLD is generated externally */
#define   OLDOH         0x01	/* If set, OLD is generated internally */

/* These are the bit defines for CNTLWR2 */
#define   SIOBENH      0x80	/* Set for SIO header operation */
#define   CODBENH      0x40	/* Cleared for SIO header operation */
#define   SYIH         0x20	/* If set, SY is generated externally */
#define   SYOH         0x10	/* If set, SY is generated internally */
#define   CMMD2        0x08	/* MMD2 for the current memory mode */

/* These are the bit defines for STATRD - read only register */
#define  BP            0x80	/* for breakpoint signaling */
#define  PDF           0x40	/* PDR status flag */
#define  PIF           0x20	/* PIF status flag */
#define  SMMD2         0x04	/* memory mode bit 2 */
#define  SMMD1         0x02	/* memory mode bit 1 */
#define  SMMD0         0x01	/* memory mode bit 0 */
/* Note bits 4 and 3 are read as ones */

/* these are the bit defines for the PCR register */

#define  RESET	       0x0001	/* 0 halt chip. 1 start reset sequence */
			/* reset causes certain pcr bits to be cleared */
#define  REGMAP        0x0002 	/* Select PAB register map
				* must be set for development system */
#define  ENI           0x0004	/* PIF status on PIR enable/disable */
/* Issue 1 (R31) DSP32C - "PIC" and "PIF" I/O conditions can only be used
 if ENI set.
*/
#define  DMA           0x0008	/* DMA enable */
#define  AUTO          0x0010	/* PAR autoincrement enable */
#define  PDFS          0x0020	/* PDR status - read only */
#define  PIFS          0x0040	/* PIR status - read only */
#define  DMA32         0x0100	/* 0 - 16 bit DMA, 1 - 32 bit DMA */
#define  PIO16         0x0200	/* 0 - 8 bit PIO, 1 - 16 bit PIO */
#define  FLG           0x0400	/* PDF/PIF leading/trailing edge changes */

#define   TRUE     1
#define   FALSE    0

#define   HIGHP    0
#define   ULAW     1
#define   ALAW     2

extern unsigned dspbase;
extern unsigned dspmode;

int load_dsp(char *dfile);
void reset_dsp(void);
void halt_dsp(void);
void run_dsp(void);
void codec_on(void);
int init_card(void);
void set_base(unsigned addr);
void write_pcr(unsigned short data);
unsigned short int read_pcr(void);
unsigned short read_pdr16(void);
void write_pdr16(unsigned short data);
unsigned long read_pdr32(void);
void write_pdr32(unsigned long data);
void write_par(unsigned long address);
void ul_array(unsigned long address, unsigned int num, int *array);
void dl_array(unsigned long address, unsigned int num, int *array);

