/****************************************************************/
/*   DIGITAL ECHO CANCELLOR and COMPLEX ADAPTIVE EQUALIZERS	*/
/*   Using The WE(R) DSP16/DSP16A DIGITAL SIGNAL PROCESSOR	*/
/*   June, 1989 Doc# AP89-009DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1989 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*	Program Listing 2.		echo-dp.s		*/
/*	Digital Echo Canceler for Two-Wire Data			*/
/*	Transmission - Double Precision Coefficients		*/
/*								*/
/*  by S. Tu, AT&T Bell Laboratories, Allentown, PA  11/28/88	*/
/*								*/

.ram
 dat1:	11*int		/* allocate 12 ram spaces for 12-tap	*/
dat12:	   int		/* delay lines				*/
  cf1:	24*int		/* 12 coef. for sub-canceler 1		*/
  cf2:	24*int		/* 12 coef. for sub-canceler 2		*/
  cf3:	24*int		/* 12 coef. for sub-canceler 3		*/
  cf4:	24*int		/* 12 coef. for sub-canceler 4		*/
  scl:	 2*int
   rx:	   int
.endram

start:	auc=0x02			/* set alignment for Q14 format */
	pt=CF1				/* pt - rom coef. pointer	*/
	r2=cf1				/* r2 - ram coef. pointer	*/

 do 96  {				/* transfer start-up coef.	*/
			y=a0   x=*pt++	/* from rom to ram (y=a0 is a	*/
	*r2++=x				/* dummy fetch)			*/
	}

	r0=scl
	r1=dat12
	r3=rx
	rb=dat1				/* rb points to beginning, re	*/
	re=dat12			/* to end of modulo.		*/
	j=-24
	i=0
	c0=0				/* initialize counter c0 to 0	*/

loop:	*r3=sdx				/* read s(n)+e(n) from SIO	*/

	if c0lt goto cnvl		/* read a(n) from PIO every	*/
	*r1++=pdx0			/* 4th loop iteration.		*/
	c0=-3				/* set c0 to -3			*/
	r2=cf1
					/* start convolution routine	*/
cnvl:	a0=p		x=*r2++		/* load x, y and clear a0,a1	*/
	a0=a0-p		y=*r1++
	a1=a0

  do 11 {				/* perform double precision	*/
		p=x*y	x=*r2++		/* convolution to obtain echo	*/
	a0=a0+p p=x*y	y=*r1++		/* estimate			*/
	a1=a1+p		x=*r2++
	}
		p=x*y	x=*r2++
	a0=a0+p p=x*y			/* a0- convolution output (LSW) */
	a1=a1+p		x=*r2++j	/* a1- convolution output (MSW) */
					/* end of convolution routine	*/

	a0=a0>>16			/* align a0 with a1		*/
	y=a1				/* transfer a1h to yh first,	*/
	a1=a1<<16			/* then a1l to yl.		*/
	yl=a1
	a0=a0+y	 y=*r3			/* add a0 and a1 to yield echo	*/
					/* estimate; fetch s(n)+e(n)	*/
	a0=a0-y				/* subtract estimated echo	*/
	a0=-a0				/* from received signal, then	*/
	a1=rnd(a0)			/* output result to SIO after	*/
	sdx=a1				/* rounding			*/

			y=a1   x=*pt++i /* fetch canceler output to y,	*/
					/* convergence factor to x	*/

					/* start lms adaptation routine */
lms:	p=x*y		x=*r1++		/* multiply x and y to yield 32 */
	a0=p				/* bit product;convert p to two */
	*r0++=a0l			/* 16 bit words and store in	*/
	a0=rnd(a0)			/* ram location pointed to by r0 */
	*r0--=a0
   move y=*r0++

 do 11  {				/* update filter coefficients	*/
		p=x*y	y=*r0--		/* perform 32 x 16 -> 32	*/
	a0=p	p=x*y	x=*r1++		/* multiplication to yield 32	*/
	a0=a0>>16			/* bit coefficient update	*/
	a0=a0+p		y=*r2++

	a1=rnd(a0)			/* a1 - tap update (MSW)	*/
	a0=a0<<16			/* a0 - tap update (LSW)	*/

	a0=a0+y		y=*r2--		/* update the LSW of the coef.	*/
	a0=a0>>16

	a1=a1+y		*r2++=a0l	/* update the MSW of the coef.	*/
	a0=rnd(a0)
	y=a0
	a1=a1+y		y=*r0++
	*r2++=a1
	}

		p=x*y	y=*r0--
	a0=p	p=x*y
	a0=a0>>16
	a0=a0+p		y=*r2++

	a1=rnd(a0)
	a0=a0<<16

	a0=a0+y		y=*r2--
	a0=a0>>16

	a1=a1+y		*r2++=a0l
	a0=rnd(a0)
	y=a0
	a1=a1+y
	*r2++=a1			/* end of adaptation routine	*/

end:	goto loop
			  /* filter final coef. settings */
CF1:	int	0.83484
	int	-0.00024
	int	-1.17218
	int	0.00037
	int	-1.12122
	int	0.00177
	int	-0.19501
	int	0.00336
	int	0.58185
	int	0.00610
	int	-1.71277
	int	0.00873
	int	0.93817
	int	0.00012
	int	-0.39166
	int	0.14514
	int	-1.84900
	int	0.01007
	int	-1.12067
	int	-0.00043
	int	-1.42810
	int	-0.00055
	int	-1.91608
	int	-0.00024
	int	-0.77722
	int	-0.00055
	int	-1.00397
	int	-0.00024
	int	1.43420
	int	0.00104
	int	-1.47412
	int	0.00256
	int	1.48413
	int	0.00464
	int	-1.12616
	int	0.00934
	int	-0.84863
	int	0.02875
	int	-1.06116
	int	0.30914
	int	1.70483
	int	0.01263
	int	0.95929
	int	-0.00055
	int	0.95587
	int	-0.00092
	int	1.71930
	int	-0.00037
	int	-0.64282
	int	-0.00049
	int	0.57837
	int	-0.00024
	int	-0.49121
	int	0.00092
	int	0.26746
	int	0.00220
	int	-0.26996
	int	0.00421
	int	0.10040
	int	0.00818
	int	-0.78595
	int	0.03174
	int	-0.57611
	int	0.24237
	int	-0.25562
	int	-0.03516
	int	1.80664
	int	-0.00372
	int	-1.50757
	int	-0.00055
	int	0.93994
	int	-0.00031
	int	-1.84192
	int	-0.00012
	int	-0.39398
	int	0.
	int	1.40552
	int	0.00092
	int	0.29498
	int	0.00226
	int	1.03375
	int	0.00403
	int	1.17072
	int	0.00757
	int	-1.82428
	int	0.01349
	int	-0.10437
	int	0.06342
	int	-0.08191
	int	-0.02716
	int	0.14935
	int	-0.00543
	int	1.98663
	int	0.
	int	-0.93744
	int	-0.00018
U:	int	0.00025
