/****************************************************************/
/*   NETWORK ECHO CANCELATION WITH THE				*/
/*   WE(R) DSP16 FAMILY OF DIGITAL SIGNAL PROCESSORS		*/
/*   November, 1988 Doc# AP88-10DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1988 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*								*/
/*	ekobgone.s	    Appendix B:  Program Listing 1	*/
/*								*/
/* Last modified: 10/12/90					*/
/*								*/
/* This is a 16 ms network echo canceller based on a 128 tap	*/
/* double precision NLMS adaptive filter with tap leakage. 	*/
/*								*/
/* Includes: near-end speech detector, residual echo suppressor,*/
/* dual port serial I/O, and coefficient display and control.	*/
/*								*/
/* Serial I/O:  						*/
/*	Port A:  Output: fardata				*/
/*		 Input:  nrdata					*/
/*	Port B:  Output: ecout					*/
/*		 Input:  fardata				*/
/*								*/
/* Parallel I/O: Only used for demonstration purposes. That is, */
/* the display and control of the adaptive coefficients of the	*/
/* NLMS adaptive filter by a PC.  All instructions related to 	*/
/* this interface are marked by the comment, DEMO, and may be 	*/
/* removed if PIO is not used.					*/
/*								*/
/* Note: Register r3 is reserved for serial I/O pointer.	*/

#define lngm2 126	/* length of NLMS filter minus 2		*/
#define lngm2d2 63	/* (length of NLMS filter minus 2) over 2	*/
#define lngm4 124	/* length of NLMS filter minus 4		*/
#define lngm4d2 62	/* (length of NLMS filter minus 4) over 2	*/
#define lngm8 120	/* length of NLMS filter minus 8		*/
#define lngt2 256	/* length of NLMS filter times 2		*/
#define lngt2m2 254	/* (length of NLMS filter times 2) minus 2	*/

/*
 * Definition of commands sent by PC
 */

# define	SEND_FRAME	0x46	/* 'F' */
# define	NEW_COEFF	0x4d	/* 'M' */
# define	SYNCHRONIZE	0x53	/* 'S' */
# define	ZERO_COEFFS	0x5a	/* 'Z' */

/*
 *   Necessary addresses in the dual-ports
 */
# define	DSP16AINPUT	0x0000	/* Where DSP reads commands from */
# define	DSP16AOUTPUT	0x0100	/* Where DSP writes data to	 */
# define	SIGNAL		0x0200	/* Used for PC<->DSP signalling  */

/*
 *  The following definition shows the value used
 *  in the SIGNAL dual-port location by the PC
 *  to signal the DSP16A to take some action.
 */
# define	SIGNAL_VALUE	0xffff

/************************************************************************/
/************************************************************************/
/*								 	*/
/*		     Beginning of Source Code		 	 	*/
/*									*/
/************************************************************************/
/************************************************************************/

	goto init		/* jump to initialization routine	*/

/************************************************************************/
/*								 	*/
/*		     Interupt Service Routine				*/
/*									*/
/************************************************************************/


intdriver:


/*	Check SIO Syncronization:  Since the PIO is double buffered       */
/*	two reads are required to transfer data from external pins        */
/*	to accumulator.  Because pods and pids strobe width is set to     */
/*	4T, four instruction cycles are required between active read      */
/*	and data availability in accumulator.				  */

synch:

insynch:sdx=*r3			/*output to SIO port B	*/
	*r3=sdx		/*input from SIO port A	*/

/*	Next, swap the serial I/O and working buffers.	   		   */
/*	During one 125 us frame, the forground performs serial I/O on 	   */
/*	one buffer while the background works on the other.  In the	   */
/*	next frame the roles of the buffers are swapped.		   */


	pi=exec			/*upon return from interrupt -- goto exec   */

/*********************************************************************/	/*DEMO*/
/*								     */	/*DEMO*/
/*		PC<->DSP16A I/O Control Code			     */	/*DEMO*/
/*								     */	/*DEMO*/
/*								     */	/*DEMO*/
/* This routine processes all I/O related requests.		     */	/*DEMO*/
/*						     		     */	/*DEMO*/
/* The C language description of the code for this routine follows.  */	/*DEMO*/
/*						     		     */	/*DEMO*/
/* ----------------------------------------------------------------- */	/*DEMO*/
/*								     */ /*DEMO*/
/* if(signal[0] == 0xffff && signal[1] == 0xffff) {		     */ /*DEMO*/
/* 	switch(DSP16AInput[0]) {				     */ /*DEMO*/
/*		case SYNCHRONIZE:				     */ /*DEMO*/
/*			break;					     */ /*DEMO*/
/* 								     */ /*DEMO*/
/* 		case CHANGE_UPDATE_COEFF:			     */ /*DEMO*/
/* 			update_coeff = DSP16AInput[1];		     */ /*DEMO*/
/* 			break;					     */ /*DEMO*/
/* 								     */ /*DEMO*/
/* 		case ZERO_COEFFS:				     */ /*DEMO*/
/* 			zero_coeffs();				     */ /*DEMO*/
/* 			break;					     */ /*DEMO*/
/* 								     */ /*DEMO*/
/* 		case SEND_NEXT_FRAME:				     */ /*DEMO*/
/* 			for(i = 0; i < 128; ++i) {		     */ /*DEMO*/
/* 				DSP16AOutput[i] = coeffs[i];	     */ /*DEMO*/
/* 			}					     */ /*DEMO*/
/* 			break;					     */ /*DEMO*/
/* 	}							     */ /*DEMO*/
/*	signal[0] = signal[1] = 0;				     */ /*DEMO*/
/* }								     */ /*DEMO*/
/* ----------------------------------------------------------------- */	/*DEMO*/
			

dsp_pc_io:								
	auc  = 0	/* Must clear a0l, a1l, yl on load */		
	y = SIGNAL_VALUE

	pdx1 = SIGNAL
	;
	a0 = pdx0
	;
	a0 = pdx0
	;
	a0 - y

	if ne goto dsp_pc_io_done

	pdx1 = SIGNAL + 1
	;
	a0 = pdx0
	;
	a0 = pdx0
	;
	a0 - y

	if ne goto dsp_pc_io_done

	pdx1 = DSP16AINPUT
	;
	a0 = pdx0
	;
	a0 = pdx0

	y  = SYNCHRONIZE
	a0 - y

	if eq goto clear_signal

	y  = NEW_COEFF							
	a0 - y								

	if ne goto chk1						

	pdx1 = DSP16AINPUT + 1
	;
	a0 = pdx0
	;
	a0 = pdx0

	r1  = delta							
	*r1 = a0							

	goto clear_signal

chk1:									
	y  = ZERO_COEFFS						
	a0 - y								
	if ne goto chk2						

	/* zero the 128 double precision coefficients */						
	r0 = coeffs							
	y  = 0								

	do 64 {								
		*r0++ = y						
		*r0++ = y						
		*r0++ = y						
		*r0++ = y						
	}								
/*
	r0 = junk
	*r0 = y
*/

	goto clear_signal

chk2:									
	y  = SEND_FRAME							
	a0 - y								
	
	if ne goto clear_signal						

send_frame:
	r0 = coeffs_end - 2
	y  = DSP16AOUTPUT - 1
	a0 = y
	j = -2

	/*
	 *   Place the scaled low word of the 128 double-precision
	 *   coefficients into the PIO Dual-Port
	 */
	do 64 {
		a0h = a0h + 1
		pdx1 = a0
		a1 = *r0++j
		a1 = a1 >> 4
		a1 = a1 >> 1
		pdx0 = a1

		a0h = a0h + 1
		pdx1 = a0
		a1 = *r0++j
		a1 = a1 >> 4
		a1 = a1 >> 1
		pdx0 = a1
	}

/*
	r0 = junk
	y = *r0
	a1 = y
	a1h = a1h + 1
	*r0 = a1

	do 64 {
		a0h = a0h + 1
		pdx1 = a0
		a1h = a1h + 1
		pdx0 = a1

		a0h = a0h + 1
		pdx1 = a0
		a1h = a1h + 1
		pdx0 = a1
	}
	;
*/


clear_signal:
	pdx1 = SIGNAL
	;
	pdx0 = 0
	;
	pdx1 = SIGNAL+1
	;
	pdx0 = 0
	;

dsp_pc_io_done:
	ireturn

/************************************************************************/
/*									*/
/*	 	      END OF FORGROUND TASKS 				*/
/*									*/
/************************************************************************/


/************************************************************************/
/*									*/
/*			INITIALIZATION					*/
/*									*/
/************************************************************************/

init:	auc=0x00  	/*clearing of a0l, a1l, and yl enabled, 	*/
			/*saturation on overflow enabled, p=x*y		*/

	sioc=0x0	/*ICK, OCK, ILD, OLD are passive; LSB first;	*/
			/*and 16-bit serial I/O				*/

	pioc=0x3800	/*PODS signal is active, strobe width = 2T,	*/
			/*PIDS is active, interrupts are disabled.	*/

/******************************ZERO ALL RAM******************************/
	r0=0
	y=0
	do 64	{
	*r0++=y
		}
	redo 64
	redo 64
	redo 64
	redo 64
	redo 64
	redo 64
	redo 64

/********Initialize hang-over counter of the near end speech detector******/
	pt=maxcnt
	r1=nearcnt
	y=*r0	x=*pt++
	a0=x
	*r1=a0

/*******Initialize oldest value state pointer of the circular buffer*******/ 
	r0=oldest
	y=states
	*r0=y

/*Initialize updaten flag such that NLMS coefficient updates are enabled*/
	r0=updaten
	y=1
	*r0=y

/*Initialize the adjustable factor (from the PC) of the step size in the   */
/*adaptive filter							   */
	r0=delta
	y=0x0400
	*r0=y

/*Initialize the simulated channel delay "filter" */
	r0=simpnt
	y=chndel
	*r0=y

/*Initialize the pure delay "filter" for the hybrid */
	r0=hybpnt
	y=hybdel
	*r0=y

/***************Initialize Synchronization of Serial Interface***************/
synchro:pioc=0x3800	/* disable interrupts */
	auc=0x0
	sioc=0x0
	tdms=0x200	/* sync pin is active and clocked at 8 Khz*/

	c0=0		
	r3=lowsio	/* initially, foreground controlles lower SIO buffer  */
	r0=iopnt
	y=hisio	/* initially, background controlles higher SIO buffer */
	*r0=y

	pioc=0x3a00	/* enable interrupts  */
	goto loop	/* go and wait for first interrupt */

/************************************************************************/
/*									*/
/*			BACKGROUND TASKS				*/
/*									*/
/************************************************************************/

exec:	call io
	call normlizr
/*	call nearspch*/
	call upncon
	call supress

loop:	3*;		/*wait for interrupt -- start of next 125 us frame */
	goto loop

/************************************************************************/
/*									*/
/*		Background Serial I/O Subroutine			*/
/*									*/
/************************************************************************/

/*io:	r1=iopnt		/*	*/
	r0=*r1		/*  load r0 with serial i/o pointer */

	a0=*r0++	/*  read input for SIO port B	*/

	r2=fardata  	/*  ch B input to fardata */
	*r2=a0

	a1=*r0--	/*  read input for SIO port A	*/

	r2=nrdata
	*r2=a1	    	/*  SIO port A input to nrdata  */

	*r0++=a0	/*  write output for SIO port A	*/
		    	/*  fardata to A out  */

	r2=echout
	a0=*r2	    	/*  echout to B out   */

	*r0++=a0	/*  write output for SIO port B	*/

	*r1=r0		/*  load background SIO buffer pointer with r0 */

	return

/************************************************************************/
/*									*/
/*		Residual Echo Suppression Subroutine			*/
/*									*/
/*	Purpose:  Detect the state when only the far-end signal is	*/
/*		  present and NLMS algorithm has reduced echo to a	*/
/*		  low level then further suppress the residual echo	*/
/*		  by center clipping.					*/
/*									*/
/*	Inputs:   echout --  Output of NLMS algorithm			*/
/*		  sdest --   Standard deviation measure of fardata      */
/*									*/
/*	State Variable:	sdest2 -- Measure of standard deviation of      */
/*			echout calculated using the leaky integrator:	*/
/*		   	sdest2 = a*sdest2 + (1-a)*|echout|.		*/
/*			Where a=.99 .					*/
/*									*/
/*	Output:	echout -- center clipped if sdest2 < sdest*(2**-4)	*/
/*									*/
/*									*/
/************************************************************************/

supress:auc=0x02	/* set auc such that register p=4*x*y  */
	r1=sdest2	
	pt=onemina	/* pt now points to (1-a) -- a is stored next */
	r0=echout
		a0=*r0
		a0=a0
		if mi a0=-a0		/*a0 is |eckout|*/

				y=a0	x=*pt++
			p=x*y	y=*r1	x=*pt++
		a0=p	p=x*y
		a0=a0+p
	*r1=a0			/*sdest2 and a0 contain new measure of	*/
				/*standard deviation of echout		*/

	r1=sdest
	a1=*r1
	a1=a1>>4		/*a1 now contains sdest*(2**-4)		*/
	y=a0
	a1-y
	if mi return		/*if sdest2 > sdest*(2**-4) then do nothing*/

		/* else center clip echout */

	  y=a1			/* y contains the clipping threshold */
	  a1=p
	  a1=a1-p	a0=*r0	/* a1=0 & a0 contains echout	*/
	  a0=a0
	  if mi a1=a1+1		/* if echout was neg then a1=1 	*/
	  a0=a0
	  if mi a0=-a0		/* a0 now contains abs(echout)	*/
	  a0=a0-y
	  y=0
	  if mi a0=y		/* a0 now contains cntr cliped abs(echout)*/
	  a1=a1
	  if gt a0=-a0		/* a0 now contains center clipped echout */
	  *r0=a0		/* echout replaced with center clipped value */
	return

/************************************************************************/
/*									*/
/*			NORMLIZR					*/
/*									*/
/*	Purpose:  Calculate a measure of the standard deviation of	*/
/*		  the far end signal and invert it.  This is		*/
/*		  accomplished by performing leaky integration on	*/
/*		  the absolute value of the far end signal and then	*/
/*		  inverting this value.	 Should the magnitude of a 	*/
/*		  single far end sample exceed the sd measure by __ dB	*/
/*		  then the sd measure will be replaced by that value.	*/
/*		  This prevents instability in the tap update algorithm	*/
/*		  when far end signal strenteh suddenly increases.	*/
/*									*/
/*	Inputs:   fardata --  The far end signal			*/
/*									*/
/*	State Variable:	sdest -- Measure of standard deviation of       */
/*			fardata, calculated using the leaky integrator:	*/
/*		   	sdest = a*sdest + (1-a)*|fardata|.		*/
/*			Where a=.99 .					*/
/*									*/
/*	Output:	normal -- The inverted value of the measure of the	*/
/*			standard deviation of the far end signal.	*/
/*									*/
/************************************************************************/

normlizr:auc=0x02	/* set auc such that register p=4*x*y		*/
	r1=sdest	/* measure of standard deviation of fardata	*/
	r0=fardata
	r2=normal
	pt=onemina	/* pt now points to (1-a) -- a is stored next	*/
		a1=*r0
		a1=a1
		if mi a1=-a1		/*a1 is |fardata|*/
				y=a1	x=*pt++
			p=x*y	y=*r1	x=*pt++
		a0=p	p=x*y
		a0=a0+p			/*a0 now contains new sd measure*/

	a1=a1>>4
	y=a1
	a0-y
	if mi a0=a1<<4		/* if |fardata|/16 > sd measure 	*/
				/* then sd measure = |fardata|		*/


	*r1=a0			/*sdest and a0 contain new sd measure*/

				y=512	/*defined as minimum sdest value*/
		a0-y			/*this prevents instability at  */
		if le a0=y		/*low far end signal strengths	*/

/*	now, invert the estimated standard deviation */

				y=a0	x=*pt++	/* dummy load to x	*/
		x=0x1
		a0=x			/*here, y is divisor, x is dividend*/
	do 15 {
		a0=a0<<1
		a1=a0-y
		if pl a0=a1+1
			}
tstpt0:		a0=a0<<16

	*r2=a0				/*a0 now contains "normal" */

ret2:	return

onemina:	int	.01
aaaaaaa:	int	.99

/************************************************************************/
/*									*/
/*			UPNCON						*/
/*									*/
/*	Purpose:  This subroutine performs the convolution, error	*/
/*		calculation, and optionally, the coefficient update	*/
/*		algorithm of the leaky NLMS adaptive filter.		*/
/*									*/
/*	Inputs:	fardata -- The far end signal				*/
/*		nrdata	-- The near end signal				*/
/*		normal  -- The normalization factor 			*/
/*									*/
/*	State Variables:						*/
/*		states  -- The delay line of the filter			*/
/*		coeffs	-- The filter coefficients			*/
/*		oldest	-- The address of the oldest sample in filter	*/
/*									*/
/*	Output: echout  -- The "canceled" echo				*/
/*									*/
/************************************************************************/
upncon:	auc=0x02	/* set auc such that register p=4*x*y	*/

	r0=coeffs	/* the adaptive filter coefficients	*/

	r1=fardata

	rb=states	/* set the modulo addressing registers	*/
	re=enstates	/* to enclose the state variables	*/

	r2=oldest	/* oldest contains the address of the	*/
	r2=*r2		/* oldest sample in the filter		*/

	j=2		/* only high 16 bits of coeffs used in	*/
			/* convolution				*/

		a0=p		x=*r0++j	/* x=last coeff		   */
		a0=a0-p		y=*r2++		/* y=oldest state variable */
	do 2	{
			p=x*y	x=*r0++j	/* convolve from oldest to */
		a0=a0-p		y=*r2++		/* newest samples in filt  */
		}
	2*;			/* stop cache operations periodically	*/
	redo lngm4d2		/* to allow pending interrupts to be	*/
	2*;			/* serviced				*/
	redo lngm4d2

			p=x*y	x=*r0++j    /* perform multiply/accumulate  */
		a0=a0-p		y=*r1	    /* on newest sample and place   */
			p=x*y	*r2zp:y	    /* that sample into delay line  */

		a0=a0-p		/*a0 now contains prediction of nrdata */

	r0=oldest	/* save address of oldest sample in the filter	*/
	*r0++=r2

	*r0=y		/* save former oldest state variable in oldold	    */
			/* to be used later in coefficient update algorithm */

	r2=nrdata
	y=*r2		/* read in nrdata				*/
	a0=a0+y		/* a0 now contains prediction error of filter	*/
	r0=echout	/* also known as -- return signal, echout	*/
	*r0=a0

/*	read in adaptive step size	*/

	r1=normal
				x=*r1++
				y=a0	
			p=x*y
		a0=p			/* a0 = prediction error times normal*/

			a0=a0<<8	/* shift result to maintain precision*/
			a0=a0<<4

				y=a0
			p=x*y
		a0=p			/* a0 = prediction error times normal*/
					/* squared			     */

			a0=a0<<8	/* once again, shift result to       */
			a0=a0<<1	/* maintain precision		     */

				y=a0	/* apply additional modification of  */
				x=*r1	/* stepsize from PC		     */
			p=x*y
		a0=p
		a0=a0<<4		/* shift again for precision         */
	r0=factor
	*r0=a0				/* store result into RAM loc, factor */


/*	next, update the coefficients		*/

	r0=updaten
	a0=*r0
	a0=a0				/* if update enable flag is not set  */
	if eq return 			/* then do not update coefficients   */

update: auc=0x11	/* auc set such that register p=x*y/4	*/
			/* and clearing of a0l upon loading of	*/
			/* a0h is inhibited			*/

	r0=coeffs	/* the adaptive coefficients		*/

	rb=states	/* set the modulo addressing to surround*/
	re=enstates	/* the state variables of the filter	*/

	r2=oldest	/* let r1 point to oldest state variable	*/
	r1=*r2++	/* in the filter, r2 now points to previous	*/
			/* oldest value					*/

	j=-2		/* set YAAU auto incrementers			*/
	k=3

	i=0		/* set XAAU auto incrementer			*/
	pt=leak		/* pt points to leak factor			*/

				y=*r2		/* read in oldold       */
	r2=factor
				x=*r2		/* read in update factor*/
			p=x*y	a0=*r0++	/* read in last double- */
				a0l=*r0++	/* precision coefficitne*/
	do 2	{
		a1=a0>>8			/*			    */
		a0=a0+p		y=a1	x=*pt++i/*a0 has new coeff sans leak*/
			p=x*y	x=*r2		/*			    */
		a0=a0+p		y=*r1++     /*a0 now has new coeff with leak*/
			p=x*y	*r0jk:a0    /*store previous coeff and bring*/
				*r0jk:a0l   /*in next one		    */
		}
	2*;			/* stop cache operations periodically	*/
	redo lngm2d2		/* to allow pending interrupts to be	*/
	2*;			/* serviced				*/
	redo lngm2d2

	return

leak:	int -0x1

/************************************************************************/
/*									*/
/*			NEARSPCH					*/
/*									*/
/* Purpose:  Detect near end speech and set the flag, updaten, if	*/
/*	none is found.  If updaten is not set then the subroutine,	*/
/*	upncon, will not update the adaptive filter coefficients.	*/
/*	This prevents near end signals from diverging the coefficients	*/
/*	of the echo canceler.  If the statement 			*/
/*									*/
/*	   s(n)**2 >= .3125*(max[x(n)**2, x(n-1)**2, ..., x(n-N)**2])	*/
/*									*/
/*	is true, where s(n) is nrdata and x(n) is fardata, then we	*/
/*	declare the presence of near end speech.  This is Geigel's	*/
/*	algorithm.							*/
/*									*/
/*  Input:  nrdata:	The near-end sample				*/
/*	    fardata:	The new far-end sample				*/ 
/*	    states:	The state variables of the adaptive filter	*/
/*									*/
/*  State Variables:							*/
/*	    nearcnt:	The detector's hang-over counter.		*/
/*									*/
/*  Output: updaten:	Flag to upncon subroutine which enable the	*/
/*			leaky NLMS coefficient update			*/
/*									*/
/************************************************************************/


nearspch:auc=0x10
	r1=nrdata
	r2=fardata
	r0=states
	c1=1
	c2=-1
		a0=p		y=*r2
halt1:		a0=a0-p		x=*r2++	
	do 2 {
			p=x*y	y=*r0	
		a1=a0-p		x=*r0++
	if mi a0=p
			}
	redo lngm8	/* a0 = max[x(n)**2, ..., x(n-N)**2]	*/

	y=a0
	x=0x1000
	a0=a0>>1
	a0=a0>>1

		p=x*y
	a0=a0+p 	/* a0 = .3125*(max[x(n)**2, ..., x(n-N)**2])	*/

			y=*r1
			x=*r1
		p=x*y
	a1=a0-p		/* perform Giegel's test as described above	*/
	ifc mi a1=a1	
	a0=c2
	c1=a0		/* store result in c1. c1=1 if near speech present */
			/* c1=-1 if near speech not present		   */

	r0=nearcnt
	pt=maxcnt
				y=*r0	/*y=nearcnt*/
	a1=p
	a1=a1-p				/*a1=0*/
	a0=y			y=a1  x=*pt++	/*a0=nearcnt,x=maxcnt*/
	if c1ge a0=a1 			/*if 4*neardata**2 larger, nearcnt=0*/
	a0h=a0h+1			/*increment nearcnt*/
	a1=x
	y=a1  				/*y=maxcnt*/
	a1=p
	a1=a1-p				/*a1=0 again*/
	a0-y
	if gt goto stopcnt		/*if nearcnt gt maxcnt stop supres*/

	  				/*else, supress coeff update*/
	  *r0=a0			/*store new nearcnt*/

	  r0=updaten		/*disable tap leakage*/
	  y=0
	  *r0=y		
	  return

stopcnt:a0=y			/*store new nearcnt=maxcnt+1*/
	a0h=a0h+1
	*r0=a0

	r0=updaten		/*enable tap leakage*/
	y=1
	*r0=y	
	return

endrom:
maxcnt:	int	600
/***********************************************************************/



/******************************** RAM **********************************/
.ram

	/*****serial I/O buffer************/

lowsio:		int 0	/*low SIO buffer*/
hisio:			/*high SIO buffer*/
endsio:		int 0

	/*****end of serial I/O buffer****/

iopnt:		int 0		/*pointer to beginning background controlled*/
				/* SIO buffer */

sdest2:		int		/* standard deviation measure of echout	*/
sdest:		int		/* standard deviation measure of fardata*/
normal:		int		/*updated scale factor*/
delta:		int		/*varies scale factor--MUST FOLLOW normal*/
factor:		int		/* */
updaten:	int		/*update enable flag*/
nrdata:		int		/*near end data sample*/
echout:		int		/*output of echo canceler*/
fardata:	int		/*new data from far end echo*/

states:		lngm2*int	/*the echo canceler tap delay line*/
enstates:	int

coeffs_strt:
oldest:		int		/*pointer to oldest sample in states*/
oldold:		int		/*this must immediatly follow oldest*/
coeffs:		lngt2m2*int	/*the time varying coefficients*/
				/*double precision 128 coeffs*/
coeffs_end:	2*int
nearcnt:	int		/*the hang-over near-end signal detector*/
				/*counter*/
state:		int							/*DEMO*/
nnrdata:	int
iirbuf:		6*int
junk:		int
simpnt:		int
chndel:		600*int
chndele:	int
hybpnt:		int
hybdel:		15*int
hybdele:	int
.endram
/*----------------------------I/O Subroutine------------*/

io:	
	
	r0=lowsio

	a0=*r0		/*  read input from sio*/

	r2=fardata  	/*  store into fardata */
	*r2=a0

	r2=nnrdata	
	*r2=a0	    	/*  input to nnrdata  */

	a1=pr
	call channel
	pr=a1

	r2=echout
	a0=*r2	    	/*  output is echout */

	r0=lowsio
	*r0=a0	/*  write output */

/*	test stuff  */
	a1=pr	     /* Ch A output should be tied to ch A input  */
	call puredel
	call hybridi	/*	*/
/*	call hybccitt /* simulate echo path: input=nnrdata output=nrdata */
	pr=a1
/*	end test stuff */

	return

/*---------------------------------------------------------------*/
hybccitt:


dohyb:	r1=oldest
	r1=*r1
	rb=states
	re=enstates
	do 32	{
		*r1++
		}
	a0=*r1
	a0=a0>>1	/*6 db loss*/
	r0=nrdata	/* output */
	*r0=a0
	rb=0
	re=0
	return
/*---------------------------------------------------------------*/
/*	IIR HYBRID SIMULATOR	*/
hybridi:auc=0x02
	j=-2
	k=3
	pt=hycoefi7
	r1=iirbuf
	r0=nrdata		/* output */
	r2=nnrdata		/* input */
		a0=*r2
		a0=a0>>1		/* guarentees 6dB of loss */
				y=*r1++	x=*pt++
			p=x*y	y=*r1--	x=*pt++
	do 3	{				/* 3 for 6'th order */
		a0=a0-p	p=x*y	y=*r1++	x=*pt++
		a0=a0-p	p=x*y	*r1zp:y	x=*pt++
		a0=p	p=x*y	y=a0	x=*pt++
		a0=a0+p	p=x*y	*r1jk:y	x=*pt++
		a0=a0+p	p=x*y	y=*r1--	x=*pt++
		}
	*r0=a0
ret5:	return

hycoefi:int	-0.707106781
	int	0.25
	int	0.0
	int	0.25
	int	1.0
	int	0.707106781
	int	0.25
	int	0.0
	int	0.0
	int	0.25			/* was 1.0*/
hycoefi2:int	-1.272792206
	int	0.81
	int	0.0
	int	0.81
	int	1.0
	int	1.272792206
	int	0.81
	int	0.0
	int	0.0
	int	1.0
hycoefi3:int	.009094
	int	0.959717
	int	-0.044525
	int	0.043182
	int	.043411
	int	.144745
	int	0.959808
	int	0.130348
	int	0.114128
	int	.114700

hycoefi4:int	.296967
	int	.875
	int	.162193
	int	.122398
	int	.122406

	int	.018005
	int	.938599
	int	-.112816
	int	.126892
	int	.126892

	int	.583557
	int	.941345
	int	0.0
	int	-.300598
	int	.300598

hycoefi7:
	int	.733704		/* D1 */
	int	.751190		/* D2 */
	int	.875610		/* N2 */
	int	.733704		/* N1 */
	int	.875610		/* N0 */

	int	.552521		/* D1 */
	int	.862213		/* D2 */
	int	.710510		/* N2 */
	int 	.550812		/* N1 */
	int	.710510		/* N0 */

	int	.990326		/* D1 */
	int	.876526		/* D2 */
	int	1.214783	/* N2 */
	int	1.091797	/* N1 */
	int	1.214783	/* N0 */

channel:r0=simpnt
	r2=*r0
	rb=chndel
	re=chndele
	r1=echout
	a0=*r1
	y=*r2
	*r2++=a0
	*r1=y
	*r0=r2
	return
	
puredel:r0=hybpnt
	r2=*r0
	rb=hybdel
	re=hybdele
	r1=nnrdata
	a0=*r1
	y=*r2
	*r2++=a0
	*r1=y
	*r0=r2
	return
	
