.module/boot=2        encode_parameters; 

#include "lpc.h"

{ ENCODE.DSP - truncates/compresses  the lpc parameters.

  INPUT:
     /dm      k
     /dm      pitch
     /dm      gain     
  
  OUTPUT:
     /dm      k       truncated
     /dm      pitch
     /dm      gain     
  
  The parameters are truncated into the required nr of bits,
  either by linearly (gain) or logarithmic (k's) quantization.

  Logarithmic:   g[i] = log10((1+k[i])/(1-k[i]))/4

}
.const POLY_ORDER = 16; 
.var/pm coeffs[2*POLY_ORDER]; {Ci lsb, Ci msb, Ci-1 lsb .....}
.init coeffs: <enc.cff>; {scaled down by 512 = 2^9}

.var/pm round[WORDS_PR_LPCFRAME];
.init   round: 
               0x000900, {pitch,  7 bit, (9=16-7) shifting NOT rounding}
               0x000900, {gain,   9 bit}
               0x000500, {k1,     5 bit}
               0x000500, {k2,     5 bit}
               0x000400, {k3,     4 bit}
               0x000400, {k4,     4 bit}
               0x000400, {k5,     4 bit}
               0x000400, {k6,     4 bit}
               0x000300, {k7,     3 bit}
               0x000300, {k8,     3 bit}
               0x000200, {k9,     2 bit}
               0x000200; {k10,    2 bit}
                         {      -------
                                 52 bit/frame
                                =============}
.entry    encode;

.external poly_approx;

.external k;
.external gain;
.external pitch;

encode:

  {encode parameters}
    i4 = ^round;       l4 = 0;
    
    si = dm(pitch);
    se = pm(i4,m5); {nr of bits to shift}
    sr = lshift si (lo);
    dm(pitch) = sr0;

    si = dm(gain);
    se = pm(i4,m5); {nr of bits to round to}
    call do_round;
    dm(gain) = sr0;

    i1 = ^k;           l1 = 0;
    cntr = 2 {N};
    do enc2_k until ce;
      my0 = 0x0000;    {k lsb}
      my1 = dm(i1,m0); {k msb}
      ax0 = POLY_ORDER - 1;
      i6  = ^coeffs;   l6 = 0;
      call poly_approx;  {log area ratio function}
      si = mx0;
      sr = lshift si by 9 (lo);  {scale up by 512, comes with the coeff's}
      sr = sr or ashift ar by 9 (hi);
      si = sr1;
      se = pm(i4,m5);
      call do_round;
    enc2_k:  dm(i1,m1) = sr0;

    cntr = N-2;
    do enc_k until ce;
      si = dm(i1,m0); 
      se = pm(i4,m5);           
      call do_round;
    enc_k:  dm(i1,m1) = sr0;

    
rts;
    
    { ROUNDING routine

     Input  si = value to be rounded
            se = nr of bits to round to

     Output sr0 = rounded value}

do_round:
  
  sr  = ashift si (lo);
  mr0 = sr0;
  mr1 = sr1;
  mr  = mr (rnd), ay0 = se;
  ar  = -ay0;
  se  = ar;
  sr  = ashift mr1 (hi);

rts;


.endmod;