/****************************************************************/
/*   DIGITAL ECHO CANCELLOR and COMPLEX ADAPTIVE EQUALIZERS	*/
/*   Using The WE(R) DSP16/DSP16A DIGITAL SIGNAL PROCESSOR	*/
/*   June, 1989 Doc# AP89-009DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1989 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*	Program Listing 4.		equal-df.s		*/
/*	Complex Decision Feedback Equalizer	rev. 1.0	*/
/*  by S. Tu, AT&T Bell Laboratories, Allentown, PA  1/10/89	*/

.ram
 dt1:	5*int	/* Allocate 6 RAM spaces for		*/
edt1:	  int	/* 3-tap forward equalizer.		*/
 dt2:	9*int	/* Allocate 10 RAM spaces for		*/
edt2:	  int	/* 5-tap feedback equalizer.		*/
 cf1:	5*int	/* Allocate 6 RAM spaces for		*/
ecf1:	  int	/* 3-tap forward equalizer coef.	*/
 cf2:	9*int	/* Allocate 10 RAM spaces for		*/
ecf2:	  int	/* 5-tap feedback equalizer coef.	*/
 trh:	3*int	/* decision threshold level		*/
 qtz:	4*int	/* quantized signal level		*/
 pnt:	3*int	/* pointer r1,rb and re			*/
 buf:	5*int	/* temporary data storage buffer	*/
 err:	2*int	/* error signal				*/
.endram

start:	auc=0x02		/* set alignment for Q14 format	*/
	j=2
	i=0
	pt=CI			/* pt -> coef. in ROM		*/
	r0=cf1			/* r2 -> coef. in RAM		*/
  do 23 {			/* Transfer coefficients and	*/
		y=a0   x=*pt++  /* constants from ROM to RAM	*/
	*r0++=x			/* (y=a0 is a dummy fetch).	*/
	}
	r1=dt2			/* r1 -> data pointer		*/
	rb=dt2			/* Configure tapped delay line	*/
	re=edt2			/* using rb, re for feedback	*/
	*r0++=r1		/* equalizer, save pointers	*/
	*r0++=rb		/* in RAM.			*/
	*r0++j=re

	r1=dt1			/* Configure tapped delay line	*/
	rb=dt1			/* for forward equalizer.	*/
	re=edt1

				/* main loop			*/
 loop:	*r0=sdx			/* input real data from SIO	*/
	r2=cf1			/* convolution - forward eqlzr	*/
cnvl1:	a0=p		x=*r2++	/* x = c1			*/
	a0=a0-p	 y=*r1++	/* clear a0; y=x1		*/
	a1=a0			/* clear a1			*/
		p=x*y	x=*r2--	  /* p = x1*c1; x = c2		*/
   do 2 {
	a0=a0+p p=x*y	y=*r1++	  /* p = x1*c2; y = x2		*/
	a1=a1+p p=x*y	x=*r2++j  /* p = x2*c2; x = c1		*/
	a0=a0-p p=x*y	y=*r1++	  /* p = x2*c1; y = x1(-1)	*/
	a1=a1+p	 x=*r2++	  /* x = c1_1			*/
		p=x*y	x=*r2--	  /* p = x1(-1)*c1_1; x = c2_1	*/
	}

	a0=a0+p p=x*y	y=*r1++
	a1=a1+p p=x*y	x=*r2++j
	a0=a0-p p=x*y
	a1=a1+p	 x=*r2++	/* x = b1			*/

	r0=pnt
	*r0zp:r1		/* Exchange rb, re, r1		*/
	*r0zp:rb		/* contents with data in RAM	*/
	*r0zp:re		/* for the feedback equalizer	*/
				/* convolution.			*/
cnvl2:	y=*r1++			  /* y = d1			*/
		p=x*y	x=*r2--	  /* p = d1*b1; x = b2		*/
   do 4 {
	a0=a0+p p=x*y	y=*r1++	  /* p = d1*b2; y = d2		*/
	a1=a1+p p=x*y	x=*r2++j  /* p = d2*b2; x = b1		*/
	a0=a0-p p=x*y	y=*r1++	  /* p = d2*b1; y = d1(-1)	*/
	a1=a1+p	 x=*r2++	  /* x = b1_1			*/
		p=x*y	x=*r2--	  /* p = d1(-1)*b1_1; x = b2_1	*/
	}
	a0=a0+p p=x*y	y=*r1++
	a1=a1+p p=x*y	x=*r2++j
	a0=a0-p p=x*y
	a1=a1+p

	a0=rnd(a0)		/* Roundoff equalizer output	*/
	a1=rnd(a1)		/* a0, a1 to 16-bit word.	*/

	*r0++j=sdx		/* input imag. data from SIO	*/
	sdx=a0			/* output real data to SIO	*/
	*r0++=a1		/* Save imag. output data	*/
				/* for later writing to sdx.	*/

/* error signal calculation for 16-QAM case */
	call dec		/* call subroutine dec		*/
	y=*r2			/* fetch quantized level d1	*/
	a0=a0-y	 *r0++j=y	   /* -e1 = y1-d1; save d1	  */
		   y=a0  x=*pt++i  /* fetch convergence factor u */
		p=x*y		   /* e1 = -u*(-e1)		  */
	a0=p
	a0=rnd(a0)		/* round e1 to 16 bit		*/
	*r0--=a0		/* save e1 in ram (err1)	*/

	a0=a1			/* repeat for e2 calculation	*/

	call dec
	y=*r2
	a0=a0-y	 *r0++j=y
			y=a0	x=*pt++i
		p=x*y
	a0=p
	a0=rnd(a0)
	*r0--=a0

/* feedback equalizer coefficient adaptation using the LMS algorithm */
	r2=err
	r3=cf2
lms1:	x=*r2++			/* x = e1			*/
	y=*r1++			/* y = d1			*/
   do 4 {
		p=x*y	x=*r2--	  /* p = e1*d1; x = e2		*/
	a0=p    p=x*y	y=*r1++	  /* p = e2*d1; y = d2		*/
	a1=p    p=x*y	x=*r2++	  /* p = e2*d2; x = e1		*/
	a0=a0+p p=x*y	y=*r3++	  /* p = e1*d2; y = b1		*/
	a0=rnd(a0)		/* db1=rnd(e1*d1+e2*d2)		*/
	a0=a0+y	 y=*r3--	/* b1(1)=b1(0)+db1; y = b2	*/
	a1=a1-p	 *r3++=a0	/* update b1 in ram		*/

	a1=rnd(a1)		/* db2=rnd(e2*d1-e1*d2)		*/
	a1=a1+y	 y=*r1++	/* b2(1)=b2(0)+db2; y = d1	*/
	*r3++=a1		/* update b2 in ram		*/
	}
		p=x*y	x=*r2--
	a0=p    p=x*y	y=*r1
	a1=p    p=x*y	x=*r2
	a0=a0+p p=x*y	y=*r3++
	a0=rnd(a0)
	a0=a0+y	 y=*r3--
	a1=a1-p	 *r3++=a0
	a1=rnd(a1)
	a1=a1+y	 *r0--
	*r3++=a1

	y=*r0--			/* Update the feedback eqlzr	*/
	*r1--=y			/* delay line elements d1(0),	*/
	y=*r0			/* d2(0).			*/
	*r1=y

	r0=pnt
	*r0zp:r1		/* Exchange the pointer		*/
	*r0zp:rb		/* settings for the forward	*/
	*r0zp:re		/* equalizer tap update.	*/

/* forward equalizer coefficient adaptation using the LMS algorithm */
lms2:	r3=cf1
	x=*r2++			/* x = e1			*/
	y=*r1++			/* y = x1			*/
   do 2 {
		p=x*y	x=*r2--	  /* p = e1*x1; x = e2		*/
	a0=p    p=x*y	y=*r1++	  /* p = e2*x1; y = x2		*/
	a1=p    p=x*y	x=*r2++	  /* p = e2*x2; x = e1		*/
	a0=a0+p p=x*y	y=*r3++	  /* p = e1*x2; y = c1		*/
	a0=rnd(a0)		/* dc1 = rnd(e1*x1+e2*x2)	*/
	a0=a0+y	 y=*r3--	/* c1(1) = c1(0)+dc1; y = c2	*/
	a1=a1-p	 *r3++=a0	/* update c1 in ram		*/
	a1=rnd(a1)		/* dc2 = rnd(e2*x1-e1*x2)	*/
	a1=a1+y	 y=*r1++	/* c2(1) = c2(0)+dc2; y = x1	*/
	*r3++=a1		/* update c2 in ram		*/
	}

		p=x*y	x=*r2--
	a0=p    p=x*y	y=*r1--
	a1=p    p=x*y	x=*r2++
	a0=a0+p p=x*y	y=*r3++
	a0=rnd(a0)
	a0=a0+y	 y=*r3--
	a1=a1-p	 *r3++=a0
	a1=rnd(a1)
	a1=a1+y	 *r0++
	*r3++=a1

	y=*r0--			/* Transfer new data to forward	*/
	*r1++=y			/* equalizer delay line elements*/
	y=*r0++j		/* x1(0), x2(0).		*/
	*r1--=y
	sdx=*r0--		/* output imag. data to SIO	*/
	goto loop		/* return to main loop		*/

/* subroutine: dec - 2-bit quantizer */
dec:	r3=trh			/* r3 -> trh			*/
	r2=qtz			/* r2 -> qtz			*/
	y=*r3++			/* fetch threshold level -0.2	*/
	a0-y			/* to y and compare with a0	*/
	if le return		/* if greater, increment r2	*/

	*r2++
	y=*r3++			/* compare with threshold 0,	*/
	a0-y			/* if greater,increment r2	*/
	if le return

	*r2++
	y=*r3			/* compare with threshold 0.2	*/
	a0-y			/* if greater, increment r2	*/
	if le return

	*r2++
	return

CI:	int	0		/* forward equalizer coef.	*/
CQ:	int	0
	int	0
	int	0
	int	1.0
	int	0
BCI:	int	0		/* feedback equalizer coef.	*/
BCQ:	int	0
	int	0
	int	0
	int	0
	int	0
	int	0
	int	0
	int	0
	int	0
TRH:	int    -0.2		/* threshold levels		*/
	int	0.0
	int	0.2
QTZ:	int    -0.3		/* quantization levels		*/
	int    -0.1
	int	0.1
	int	0.3
U:	int    -0.05		/* -(convergence factor) 	*/

