/****************************************************************/
/*   DIGITAL ECHO CANCELLOR and COMPLEX ADAPTIVE EQUALIZERS	*/
/*   Using The WE(R) DSP16/DSP16A DIGITAL SIGNAL PROCESSOR	*/
/*   June, 1989 Doc# AP89-009DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1989 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*		Program Listing 3.		equal-l.s	*/
/*		Complex Linear Equalizer	rev. 1.0	*/
/*  by S. Tu, AT&T Bell Laboratories, Allentown, PA  12/20/88	*/

.ram
 dat1:	   int		/* allocate 16 ram spaces for	*/
 dat2:	13*int		/* 8-tap complex data		*/
edat1:	   int
edat2:	   int
 err1:	   int		/* err1 - real error signal	*/
 err2:	   int		/* err2 -imag. error signal	*/
  trh:	 3*int		/* decision threshold levels	*/
  qtz:	 4*int		/* quantized signal levels	*/
coef1:	   int		/* allocate 16 ram spaces for	*/
coef2:	13*int		/* 8-tap complex coefficients	*/
ecef1:	   int
ecef2:	   int
  buf:	 3*int		/* temporary data storage	*/
.endram

start:	auc=0x02			/* set alignment for Q14 format	*/
	pt=CI				/* pt -> coef. in rom		*/
	r2=coef1			/* r2 -> coef. in ram		*/
  do 16 {				/* transfer coefficients from	*/
			y=a0   x=*pt++	/* rom to ram (y=a0 is a dummy	*/
	*r2++=x				/* fetch)			*/
	}
	r0=trh				/* r0 -> threshold in ram	*/
   do 7 {				/* transfer threshold levels	*/
			y=a0   x=*pt++	/* and quantization levels	*/
	*r0++=x				/* from rom to ram		*/
	}

	r1=dat1				/* r1 -> data pointer		*/
	rb=dat1				/* set virtual shifting		*/
	re=edat2			/* registers rb, re		*/
	j=2
	i=0

loop:	r0=buf				/* main loop			*/
	*r0++=sdx			/* input real data from SIO	*/
	r2=coef1
	r3=coef2
					/* convolution			*/
cnvl:	a0=p	x=*r2			/* x = c1			*/
	a0=a0-p	 y=*r1++		/* clear a0; y = x1		*/
	a1=a0				/* clear a1			*/
		p=x*y	x=*r3++j	/* p = x1*c1; x = c2		*/
   do 7 {
	a0=a0+p p=x*y	y=*r1++		/* p = x1*c2; y = x2		*/
	a1=a1+p p=x*y	x=*r2++j	/* p = x2*c2; x = c1		*/
	a0=a0-p p=x*y	y=*r1++		/* p = x2*c1; y = x1(-1)	*/
	a1=a1+p	 x=*r2			/* x = c1_1			*/
		p=x*y	x=*r3++j	/* p = x1(-1)*c1_1; x = c2_1	*/
	}
	a0=a0+p p=x*y	y=*r1++
	a1=a1+p p=x*y	x=*r2++j
	a0=a0-p p=x*y
	a1=a1+p

	a0=rnd(a0)			/* roundoff equalizer output	*/
	a1=rnd(a1)			/* a0,a1 to 16-bit word		*/

	*r0++=sdx			/* input imag. data from SIO	*/
	*r0=a1				/* store imag. output data	*/
	sdx=a0				/* output real data to SIO	*/

/* error signal calculation of 16-QAM case */
	r3=err1				/* r3 -> error signal (real)	*/
	call dec			/* call subroutine dec		*/
	y=*r2				/* fetch quantized level d1	*/
	a0=a0-y				/* calculate -e1 = y1 - d1	*/
			y=a0   x=*pt++	/* fetch convergence factor u	*/
		p=x*y			/* calculate  e1 = -u*(-e1)	*/
	a0=p
	a0=rnd(a0)
	*r3++=a0			/* write  e1 to ram (err1)	*/

	a0=a1				/* repeat for  e2 calculation	*/
	call dec
	y=*r2
	a0=a0-y
			y=a0   x=*pt++i
		p=x*y
	a0=p
	a0=rnd(a0)
	*r3=a0				/* write  e2 to ram (err2)	*/

/* coefficient adaptation using the LMS algorithm */
	r2=err1				/* r2 -> err1			*/
	r3=err2				/* r3 -> err2			*/
	r0=coef1			/* r0 -> coef1			*/
lms:	x=*r2				/* x = e1			*/
	y=*r1++				/* y = x1			*/

   do 7 {
		p=x*y	x=*r3		/* p = e1*x1; x = e2		*/
	a0=p	p=x*y	y=*r1++		/* p = e2*x1; y = x2		*/
	a1=p	p=x*y	x=*r2		/* p = e2*x2; x = e1		*/
	a0=a0+p p=x*y	y=*r0++		/* p = e1*x2; y = c1		*/
	a0=rnd(a0)
	a0=a0+y	 y=*r0--		/* a0 = c1(n+1); y = c2		*/
	a1=a1-p	 *r0++=a0		/* update c1			*/
	a1=rnd(a1)
	a1=a1+y	 y=*r1++		/* a1 = c2(n+1); y = x1		*/
	*r0++=a1			/* update c2			*/
	}
		p=x*y	x=*r3
	a0=p	p=x*y	y=*r1--
	a1=p	p=x*y	x=*r2
	a0=a0+p p=x*y	y=*r0++
	a0=rnd(a0)
	a0=a0+y	 y=*r0--
	a1=a1-p	 *r0++=a0
	a1=rnd(a1)
	a1=a1+y
	*r0++=a1

	y=*r0++				/* transfer new data to delay	*/
	*r1++=y				/* line				*/
	y=*r0++
	*r1--=y
	sdx=*r0				/* output imag. data to SIO	*/
	goto loop			/* return to main loop		*/

/* subroutine: dec - 2-bit quantizer	*/
dec:	r0=trh				/* r0 -> trh			*/
	r2=qtz				/* r2 -> qtz			*/
	y=*r0++				/* fetch threshold level -0.2	*/
	a0-y				/* to y and compare with a0	*/
	if le return			/* if greater, increment r2	*/

	*r2++
	y=*r0++				/* compare with threshold 0,	*/
	a0-y				/* if greater,increment r2	*/
	if le return

	*r2++
	y=*r0				/* compare with threshold 0.2	*/
	a0-y				/* if greater, increment r2	*/
	if le return

	*r2++
	return

CI:	int	0		/* start of real coef.		*/
CQ:	int	0		/* start of imaginary coef.	*/
	int	0
	int	0
	int	0
	int	0
	int	1.0
	int	0
	int	0
	int	0
	int	0
	int	0
	int	0
	int	0
	int	0
	int	0
TRH:	int	-0.2		/* threshold levels		*/
	int	0.0
	int	0.2
QTZ:	int	-0.3		/* quantization levels		*/
	int	-0.1
	int	0.1
	int	0.3
U:	int	-0.0625		/* -( convergence factor )	*/

