/* AT&T DSP32,  FFT				  Sep 29, 1988  */
/*								*/
/* fft.s
/*   -- In-place decimation-in-time, radix 2 complex FFT	*/
/*   -- See Rabiner and Gold, Theory and Application of Digital	*/
/*      Signal Processing, p.367. and Cooley, Lewis, and Welsh,	*/
/*      "The Fast Fourier Transform and its Applications," IEEE	*/
/*      Trans. on Education, March 1969, pp. 27-34.		*/
/*   -- Routine requires 372 bytes of memory			*/
/*      Input data requires 8*N bytes where N=2**M, and a max	*/
/*      size of N=4096.	*/
/*   -- Generic routine, can be imbedded in ROM where		*/
/*      minimizing code size is important.			*/
/*   -- Execution time in ms for the 25MHz DSP32 is as follows:	*/
/*								*/
/*	           N       Time (ms)				*/
/*	         -----     ---------				*/
/*                 64         0.564				*/
/*                128         1.205				*/
/*                256         2.570				*/
/*                512         5.454				*/
/*               1024        11.547				*/
/*               2048        24.359				*/
/*               4096        51.270				*/
/*								*/

/* Driver Test Program ------------------------------------------ */

 .global  main, pause, _fftIO
 .global  _fftend, _fft

	.rsect ".bank0"

	goto main
	nop

_fftIO:	128*float 0.707,0.,2.,0.,0.707,0.,-1.,0.,-0.707,0.,0.,0.,-0.707,0.,-1.,0.
.rsect ".hi_ram"

	main:	call _fft (r14)
		nop
	int	1024, 10		/* arguments */
	int	_fftIO
	.align  4

		pir = r1     /* next instr. executed after routine call */
	pause:	if(pif) goto pause
		nop

/* ------------------------------------------------------------------- */

/* The fft.s routnie */

_fft:

#define i r1
#define j r2
#define cnt r5
#define k r3
#define w r4
#define jcnt r6
#define lcnt r7
#define le1 r8
#define leo r9
#define lx r10
#define iz r11
#define kz r12
#define lem r15
#define ur a2
#define ui a3
#define ONE *r13++
#define _ERO *r13--
#define _fftTWO a1

/* In place bit reversal ----------------------------- */

        lx=*r14++		/* N */
        lcnt=*r14++		/* M */
        j=*r14			/* points to _fftIO */
        i=*r14			/* points to _fftIO */
	r17 = j 		/* _fftIO  ********************* */

	r16 = lx * 2		/* N *2 */
        cnt=lx + (-3)		/* NM3 = N -3 */
	r16 = r16 * 2		/* N * 4 */

_fftA:  i-j
        if (ge) goto _fft5
	r4 = i			/* add temp. pointer for i ********** */

        *i++ = a0 = *j++     /* Begin complex exchange */
        *i-- = a0 = *j--     /*    T = _fftIO(J)   */
        *j++ = a0 = *r4++     /*  _fftIO(J) = _fftIO(I) ************ */
        *j-- = a0 = *r4--     /*  _fftIO(I) = _fftIO(J) ************ */

_fft5:  k=r16		/* N4 = N * 4 */
        j=j-r17 	/* ****************************************** */
_fft6:  k-j
        if (gt) goto _fft7     /* ************************************ */
        nop

        j=j-k
        goto _fft6
        k=k/2

_fft7:  j=j+r17                /* ************************************* */
        j=j+k
        if (cnt-- >= 0) goto _fftA
        i=i+8

/* Begin FFT calculation ----------------------------- */

        leo=4
	lcnt = lcnt + (-2)	/* M - 2 */
        le1=1
        w=_fftW
        r13=_ffttwo
        _fftTWO = *r13++

_fft30: lx=lx/2
        leo=leo*2
        lem=leo*2
        lem=lem-4
        j=*r14			/* _fftIO */
        ur=ONE
        ui=_ERO
        jcnt=le1+(-2)
        le1=le1*2

_fft20: i=j
	iz=i
        k=i
        k=k+leo
	kz=k
        cnt=lx+(-2)

/* Butterfly ----------------------------------------- */

_fft10: a0 = *i  + ur * *k++       /* T=_fftIO(i)r+Ur*_fftIO(k)r */
        *iz++=a0=a0- *k-- * ui    /* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
        *kz++=a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)r=-T+2*_fftIO(i)r */
        a0 = *i  + ui * *k++       /* T=_fftIO(i)i+Ui*_fftIO(k)r */
   *iz++lem=a0=a0+ *k++lem * ur    /* _fftIO(i)i=T=T+Ur*_fftIO(k)i */
        if (cnt-- >= 0) goto _fft10
   *kz++lem=a0=-a0+ *i++lem * _fftTWO    /* _fftIO(k)i=-T+2*_fftIO(i)i */

        a0 = ur * *w++          /* Compute new twiddle */
        ur = a0 - ui * *w         /*   U = U * _fftW       */
        a0 = ur * *w--
        ui = a0 + ui * *w
        if (jcnt-- >= 0) goto _fft20
        j=j+8

        if (lcnt-- >= 0) goto _fft30
        w=w+8

 	goto r14+4
	nop

/* Constant table ------------------------------------ */

_ffttwo:    float 2.0, 1.0, 0.0

/* Twiddle table ------------------------------------- */

_fftW:float -1.0000000, 0.0000000   /* cos(pi/1),-sin(pi/1) */
  float  0.0000000, -1.0000000   /* cos(pi/2),-sin(pi/2) */
  float  0.7071068, -0.7071068   /* cos(pi/4),-sin(pi/4) */
  float  0.9238795, -0.3826834   /* cos(pi/8),-sin(pi/8) */
  float  0.9807853, -0.1950903   /*cos(pi/16),-sin(pi/16)*/
  float  0.9951847, -0.0980171   /*cos(pi/32),-sin(pi/32)*/
  float 0.9987955, -4.9067674e-2  /* cos and -sin(pi/64) */
  float 0.9996988, -2.4541228e-2 /* cos and -sin(pi/128) */
  float 0.9999247, -1.2271538e-2 /* cos and -sin(pi/256) */
  float 0.9999812, -6.1358846e-3 /* cos and -sin(pi/512) */

  float 0.9999952938, -3.067956691e-3 /* cos and -sin(pi/1024) */
  float 0.9999988234, -1.533980094e-3 /* cos and -sin(pi/2048) */

_fftend:
/* --------------------------------------------------------------------- */

