/* AT&T DSP32, 1k Point FFT			  Sep 28, 1988  */
/*								*/
/* fft1k.s							*/
/*   -- 1024 pt in-place decimation-in-time, radix 2 complex FFT*/
/*   -- Use stored bit reversal table in same bank as routine	*/
/*   -- First and second stages combined and separate also	*/
/*      the last stage from the intermediate stages.		*/
/*   -- Adapted from Rabiner and Gold, Theory and Application	*/
/*      of Digital Signal Processing, p.367.			*/
/*   -- Routine requires 2536 bytes of memory			*/
/*      Input data requires 8192 bytes				*/
/*   -- Execution time for the 25 MHz DSP32 is 7.425 ms.	*/

/* Driver Test Program -----------------------------------------*/

 .global  main, pause, _fftIO
 .global  _fftend, _fft1k

	.rsect ".bank0"

	goto main
	nop

_fftIO:	128*float 0.707,0.,2.,0.,0.707,0.,-1.,0.,-0.707,0.,0.,0.,-0.707,0.,-1.,0.
.rsect ".hi_ram"

	main:	call _fft1k (r14)
		nop

	int	_fftIO    /* arguments */
	.align  4

		pir = r1    	 /* next instr. executed after routine call */
	pause:	if(pif) goto pause
		nop

/* --------------------------------------------------------------------- */
/* The fft1k.s routine */

_fft1k:

#define  N	1024	/* length of FFT */
#define  M	10	/* log2(N) */
#define  MM2	6	/* counter for M-2 loops */
#define  L	496	/* number of address pairs */
#define  INC	(-8184)

#define i r1
#define j r2
#define k r3
#define iz r4
#define kz r5
#define w r6
#define le1 r7
#define leo r8
#define jcnt r9
#define lx r10
#define Mcnt r12
#define lem r15
#define cnt r16
#define ur a2
#define ui a3
#define ONE *r13++
#define _ERO *r13--
#define _fftTWO a1

	r11 = *r14		/* address of data */

/* Performs bit-reversal prior to FFT, using table lookup */

		r3 = _fft1KY	/* pointer to first address */
		r4 = r3 + 2	/* pointer to second address in pair */
		r5 = L - 2	/* loop counter */
		r15 = 4		/* increment */

        	r1 = *r3++r15	/* get first address (A) */
_fft1kA:	r2 = *r4++r15	/* get second address (B) */
		r1 = r1 + r11	/* optional offsets to array */
		r2 = r2 + r11
		r6 = r1

		*r1++=	a0 = *r2++	/* ReB -> ReA */
		*r1--=	a0 = *r2--	/* ImB -> ImA */
		*r2++=	a0 = *r6++	/* ReA -> ReB */
		*r2--=	a0 = *r6--	/* ImA -> ImB */
		if (r5-- >=0) goto _fft1kA
        	r1 = *r3++r15	/* get first address (A) */


/*  fft - Calculates 1024 point complex in-place DIT radix 2 FFT */

/*  Stage 1 and 2 combined, i.e., use N/4 butterflies  */

		r16 = N/4 - 2	/* loop counter  */
		r1 = r11
		r2 = r1 + 8	/* data item B */
		r3 = r1 + 16	/* data item C */
		r4 = r1 + 24	/* data item D */

		r5 = r1		/* pointer for new A (A') */
		r6 = r2		/* B' */
		r7 = r3		/* C' */
		r8 = r4		/* D' */

		r15 = 28	/* increment for next group of four */

_fft1kB:	a0 = *r1++ + *r2++	/* ReA + ReB */
		a1 = *r3++ + *r4++	/* ReC + ReD */
		a2 = *r1-- + *r2--	/* ImA + ImB */
		a3 = *r3-- + *r4--	/* ImC + ImD */

	*r5++=	a0 = a0 + a1		/* ReA' = ReA+ReB+ReC+ReD */
		a0 = *r1++ - *r2++	/* ReA - ReB */
	*r7++=	a1 = -a1 + a0		/* ReC' = -(ReC+ReD)+(ReA+ReB) */
  	*r5++r15=  a2 = a2 + a3		/* ImA' = ImA+ImB+ImC+ImD */
		a2 = *r3++ - *r4++	/* ReC - ReD */
        *r7++r15=  a3 = -a3 + a2	/* ImC' = -(ImC+ImD)+(ImA+ImB) */
		a3 = *r1++r15 - *r2++r15	/* ImA - ImB */
		a1 = *r3++r15 - *r4++r15	/* ImC - ImD */

	*r6++=	a0 = a1 + a0		/* ReB' = (ImC-ImD)+(ReA-ReB) */
	*r8++=  a1 = -a1 + a0		/* ReD' = -(ImC-ImD)+(ReA-ReB) */
        *r6++r15 = a3 = -a2 + a3	/* ImB' = -(ReC-ReD)+(ImA-ImB) */
		if (r16-- >=0) goto _fft1kB
	*r8++r15 = a2 = a2 + a3		/* ImD' = (ReC-ReD)+(ImA-ImB) */

/* Stage 3 to M, radix 2 ---------------------------------------------- */

	Mcnt = M + (-5)	        /* M - 5, counter for M (stages) */
        r13=_ffttwo
        _fftTWO = *r13++
        w=_fftW			/* Pointer to twiddle table */
        leo=16			/* Butterfly index difference */
        le1=4			/* Number of twiddles for the stage */

	lx = N/4              /* N/4, length of FFT *************** */

/* Stage 3 to M-1 --------------------------------- */

_fft30: lx=lx/2			/* Number of Butterflies */
        leo=leo*2
        lem=leo*2
        lem=lem-4
	r17 = lem + INC		/* Increment for the next twiddle */
        j=*r14			/* _fftIO */
        ur=*w++			/* ************ */
        ui=*w--			/* ************ */
        jcnt=le1+(-3)		/* Counter for number of twiddles */
        le1=le1*2		/* Number of twiddles for next stage */

        i=j
	iz=i
        k=i
        k=k+leo
	kz=k
        cnt=lx+(-3)		/* Counter for number of butterflies */

/* Stages 3 to M-1, Butterfly # 1 --------------------------------- */

_fft10d:
        *iz++=a0=*i + *k++		/* RE1 + RE2 */
        *kz++=a0=-a0 + *i++ * _fftTWO	/* RE1 - RE2 */
   *iz++lem=a0=*i + *k++lem		/* IM1 + IM2 */
        if (cnt-- >= 0) goto _fft10d
   *kz++lem=a0= -a0 + *i++lem  *_fftTWO	/* IM1 - IM2 */

        *iz++=a0=*i + *k++		/* RE1 + RE2 */
        *kz++=a0=-a0 + *i++ * _fftTWO	/* RE1 - RE2 */
   *iz++r17=a0=*i + *k++r17		/* IM1 + IM2 */
   *kz++r17=a0= -a0 + *i++r17  *_fftTWO	/* IM1 - IM2 */


/* Stages 3 to M-1, Butterfly # 2 --------------------------------- */

_fft20: cnt=lx+(-3)
_fft10: a0 = *i  + ur * *k++       /* T=_fftIO(i)r+Ur*_fftIO(k)r */
        *iz++=a0=a0- *k-- * ui    /* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
        *kz++=a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)r=-T+2*_fftIO(i)r */
        a0 = *i  + ui * *k++       /* T=_fftIO(i)i+Ui*_fftIO(k)r */
   *iz++lem=a0=a0+ *k++lem * ur    /* _fftIO(i)i=T=T+Ur*_fftIO(k)i */
        if (cnt-- >= 0) goto _fft10
   *kz++lem=a0=-a0+ *i++lem * _fftTWO    /* _fftIO(k)i=-T+2*_fftIO(i)i */

         a0 = *i  + ur * *k++       /* T=_fftIO(i)r+Ur*_fftIO(k)r */
        *iz++=a0=a0- *k-- * ui    /* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
        *kz++=a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)r=-T+2*_fftIO(i)r */
        a0 = *i  + ui * *k++       /* T=_fftIO(i)i+Ui*_fftIO(k)r */
   *iz++r17=a0=a0+ *k++r17 * ur    /* _fftIO(i)i=T=T+Ur*_fftIO(k)i */
   *kz++r17=a0=-a0+ *i++r17 * _fftTWO    /* _fftIO(k)i=-T+2*_fftIO(i)i */

        a0 = ur * *w++          /* Compute new twiddle */
        ur = a0 - ui * *w         /*   U = U * _fftW       */
        a0 = ur * *w--
        ui = a0 + ui * *w
        if (jcnt-- >= 0) goto _fft20
        j=j+8

        if (Mcnt-- >= 0) goto _fft30
        w=w+8

/* Stage M -------------------------------------------- */
	leo=leo*2
        i=*r14			/* _fftIO */
        ur=ONE
        ui=_ERO
        jcnt=le1+(-2)

	iz=i
        k=i
        k=k+leo
	kz=k

_fft20A: a0 = *i  + ur * *k++       /* T=_fftIO(i)r+Ur*_fftIO(k)r */
        *iz++=a0=a0- *k-- * ui    /* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
        *kz++=a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)r=-T+2*_fftIO(i)r */
        a0 = *i  + ui * *k++       /* T=_fftIO(i)i+Ui*_fftIO(k)r */
   *iz++ =a0=a0+ *k++ * ur    /* _fftIO(i)i=T=T+Ur*_fftIO(k)i */
   *kz++ =a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)i=-T+2*_fftIO(i)i */

        a0 = ur * *w++          /* Compute new twiddle */
        ur = a0 - ui * *w         /*   U = U * _fftW       */
        a0 = ur * *w--
        if (jcnt-- >= 0) goto _fft20A
        ui = a0 + ui * *w

/* --------------------------------------------------- */

 	goto r14+4
	nop

/* Constant table ------------------------------------ */

_ffttwo:    float 2.0, 1.0, 0.0

/* Twiddle table ------------------------------------- */

_fftW:  float  0.7071068, -0.7071068   /* cos(pi/4),-sin(pi/4) */
  float  0.9238795, -0.3826834   /* cos(pi/8),-sin(pi/8) */
  float  0.9807853, -0.1950903   /*cos(pi/16),-sin(pi/16)*/
  float  0.9951847, -0.0980171   /*cos(pi/32),-sin(pi/32)*/
  float 0.9987955, -4.9067674e-2  /* cos and -sin(pi/64) */
  float 0.9996988, -2.4541228e-2 /* cos and -sin(pi/128) */
  float 0.9999247, -1.2271538e-2 /* cos and -sin(pi/256) */
  float 0.9999812, -6.1358846e-3 /* cos and -sin(pi/512) */

/* ---------------------------------------------------------- */

/* Bit reveral table */
_fft1KY:

fltbits 0x10000008,0x08000010,0x18000018,0x04000020,0x14000028,0x0c000030
fltbits 0x1c000038,0x02000040,0x12000048,0x0a000050,0x1a000058,0x06000060
fltbits 0x16000068,0x0e000070,0x1e000078,0x01000080,0x11000088,0x09000090
fltbits 0x19000098,0x050000a0,0x150000a8,0x0d0000b0,0x1d0000b8,0x030000c0
fltbits 0x130000c8,0x0b0000d0,0x1b0000d8,0x070000e0,0x170000e8,0x0f0000f0
fltbits 0x1f0000f8,0x10800108,0x08800110,0x18800118,0x04800120,0x14800128
fltbits 0x0c800130,0x1c800138,0x02800140,0x12800148,0x0a800150,0x1a800158
fltbits 0x06800160,0x16800168,0x0e800170,0x1e800178,0x11800188,0x09800190
fltbits 0x19800198,0x058001a0,0x158001a8,0x0d8001b0,0x1d8001b8,0x038001c0
fltbits 0x138001c8,0x0b8001d0,0x1b8001d8,0x078001e0,0x178001e8,0x0f8001f0
fltbits 0x1f8001f8,0x10400208,0x08400210,0x18400218,0x04400220,0x14400228
fltbits 0x0c400230,0x1c400238,0x12400248,0x0a400250,0x1a400258,0x06400260
fltbits 0x16400268,0x0e400270,0x1e400278,0x11400288,0x09400290,0x19400298
fltbits 0x054002a0,0x154002a8,0x0d4002b0,0x1d4002b8,0x034002c0,0x134002c8
fltbits 0x0b4002d0,0x1b4002d8,0x074002e0,0x174002e8,0x0f4002f0,0x1f4002f8
fltbits 0x10c00308,0x08c00310,0x18c00318,0x04c00320,0x14c00328,0x0cc00330
fltbits 0x1cc00338,0x12c00348,0x0ac00350,0x1ac00358,0x06c00360,0x16c00368
fltbits 0x0ec00370,0x1ec00378,0x11c00388,0x09c00390,0x19c00398,0x05c003a0
fltbits 0x15c003a8,0x0dc003b0,0x1dc003b8,0x13c003c8,0x0bc003d0,0x1bc003d8
fltbits 0x07c003e0,0x17c003e8,0x0fc003f0,0x1fc003f8,0x10200408,0x08200410
fltbits 0x18200418,0x14200428,0x0c200430,0x1c200438,0x12200448,0x0a200450
fltbits 0x1a200458,0x06200460,0x16200468,0x0e200470,0x1e200478,0x11200488
fltbits 0x09200490,0x19200498,0x052004a0,0x152004a8,0x0d2004b0,0x1d2004b8
fltbits 0x132004c8,0x0b2004d0,0x1b2004d8,0x072004e0,0x172004e8,0x0f2004f0
fltbits 0x1f2004f8,0x10a00508,0x08a00510,0x18a00518,0x14a00528,0x0ca00530
fltbits 0x1ca00538,0x12a00548,0x0aa00550,0x1aa00558,0x06a00560,0x16a00568
fltbits 0x0ea00570,0x1ea00578,0x11a00588,0x09a00590,0x19a00598,0x15a005a8
fltbits 0x0da005b0,0x1da005b8,0x13a005c8,0x0ba005d0,0x1ba005d8,0x07a005e0
fltbits 0x17a005e8,0x0fa005f0,0x1fa005f8,0x10600608,0x08600610,0x18600618
fltbits 0x14600628,0x0c600630,0x1c600638,0x12600648,0x0a600650,0x1a600658
fltbits 0x16600668,0x0e600670,0x1e600678,0x11600688,0x09600690,0x19600698
fltbits 0x156006a8,0x0d6006b0,0x1d6006b8,0x136006c8,0x0b6006d0,0x1b6006d8
fltbits 0x076006e0,0x176006e8,0x0f6006f0,0x1f6006f8,0x10e00708,0x08e00710
fltbits 0x18e00718,0x14e00728,0x0ce00730,0x1ce00738,0x12e00748,0x0ae00750
fltbits 0x1ae00758,0x16e00768,0x0ee00770,0x1ee00778,0x11e00788,0x09e00790
fltbits 0x19e00798,0x15e007a8,0x0de007b0,0x1de007b8,0x13e007c8,0x0be007d0
fltbits 0x1be007d8,0x17e007e8,0x0fe007f0,0x1fe007f8,0x10100808,0x18100818
fltbits 0x14100828,0x0c100830,0x1c100838,0x12100848,0x0a100850,0x1a100858
fltbits 0x16100868,0x0e100870,0x1e100878,0x11100888,0x09100890,0x19100898
fltbits 0x151008a8,0x0d1008b0,0x1d1008b8,0x131008c8,0x0b1008d0,0x1b1008d8
fltbits 0x171008e8,0x0f1008f0,0x1f1008f8,0x10900908,0x18900918,0x14900928
fltbits 0x0c900930,0x1c900938,0x12900948,0x0a900950,0x1a900958,0x16900968
fltbits 0x0e900970,0x1e900978,0x11900988,0x19900998,0x159009a8,0x0d9009b0
fltbits 0x1d9009b8,0x139009c8,0x0b9009d0,0x1b9009d8,0x179009e8,0x0f9009f0
fltbits 0x1f9009f8,0x10500a08,0x18500a18,0x14500a28,0x0c500a30,0x1c500a38
fltbits 0x12500a48,0x1a500a58,0x16500a68,0x0e500a70,0x1e500a78,0x11500a88
fltbits 0x19500a98,0x15500aa8,0x0d500ab0,0x1d500ab8,0x13500ac8,0x0b500ad0
fltbits 0x1b500ad8,0x17500ae8,0x0f500af0,0x1f500af8,0x10d00b08,0x18d00b18
fltbits 0x14d00b28,0x0cd00b30,0x1cd00b38,0x12d00b48,0x1ad00b58,0x16d00b68
fltbits 0x0ed00b70,0x1ed00b78,0x11d00b88,0x19d00b98,0x15d00ba8,0x0dd00bb0
fltbits 0x1dd00bb8,0x13d00bc8,0x1bd00bd8,0x17d00be8,0x0fd00bf0,0x1fd00bf8
fltbits 0x10300c08,0x18300c18,0x14300c28,0x1c300c38,0x12300c48,0x1a300c58
fltbits 0x16300c68,0x0e300c70,0x1e300c78,0x11300c88,0x19300c98,0x15300ca8
fltbits 0x0d300cb0,0x1d300cb8,0x13300cc8,0x1b300cd8,0x17300ce8,0x0f300cf0
fltbits 0x1f300cf8,0x10b00d08,0x18b00d18,0x14b00d28,0x1cb00d38,0x12b00d48
fltbits 0x1ab00d58,0x16b00d68,0x0eb00d70,0x1eb00d78,0x11b00d88,0x19b00d98
fltbits 0x15b00da8,0x1db00db8,0x13b00dc8,0x1bb00dd8,0x17b00de8,0x0fb00df0
fltbits 0x1fb00df8,0x10700e08,0x18700e18,0x14700e28,0x1c700e38,0x12700e48
fltbits 0x1a700e58,0x16700e68,0x1e700e78,0x11700e88,0x19700e98,0x15700ea8
fltbits 0x1d700eb8,0x13700ec8,0x1b700ed8,0x17700ee8,0x0f700ef0,0x1f700ef8
fltbits 0x10f00f08,0x18f00f18,0x14f00f28,0x1cf00f38,0x12f00f48,0x1af00f58
fltbits 0x16f00f68,0x1ef00f78,0x11f00f88,0x19f00f98,0x15f00fa8,0x1df00fb8
fltbits 0x13f00fc8,0x1bf00fd8,0x17f00fe8,0x1ff00ff8,0x18081018,0x14081028
fltbits 0x1c081038,0x12081048,0x1a081058,0x16081068,0x1e081078,0x11081088
fltbits 0x19081098,0x150810a8,0x1d0810b8,0x130810c8,0x1b0810d8,0x170810e8
fltbits 0x1f0810f8,0x18881118,0x14881128,0x1c881138,0x12881148,0x1a881158
fltbits 0x16881168,0x1e881178,0x19881198,0x158811a8,0x1d8811b8,0x138811c8
fltbits 0x1b8811d8,0x178811e8,0x1f8811f8,0x18481218,0x14481228,0x1c481238
fltbits 0x1a481258,0x16481268,0x1e481278,0x19481298,0x154812a8,0x1d4812b8
fltbits 0x134812c8,0x1b4812d8,0x174812e8,0x1f4812f8,0x18c81318,0x14c81328
fltbits 0x1cc81338,0x1ac81358,0x16c81368,0x1ec81378,0x19c81398,0x15c813a8
fltbits 0x1dc813b8,0x1bc813d8,0x17c813e8,0x1fc813f8,0x18281418,0x1c281438
fltbits 0x1a281458,0x16281468,0x1e281478,0x19281498,0x152814a8,0x1d2814b8
fltbits 0x1b2814d8,0x172814e8,0x1f2814f8,0x18a81518,0x1ca81538,0x1aa81558
fltbits 0x16a81568,0x1ea81578,0x19a81598,0x1da815b8,0x1ba815d8,0x17a815e8
fltbits 0x1fa815f8,0x18681618,0x1c681638,0x1a681658,0x1e681678,0x19681698
fltbits 0x1d6816b8,0x1b6816d8,0x176816e8,0x1f6816f8,0x18e81718,0x1ce81738
fltbits 0x1ae81758,0x1ee81778,0x19e81798,0x1de817b8,0x1be817d8,0x1fe817f8
fltbits 0x1c181838,0x1a181858,0x1e181878,0x19181898,0x1d1818b8,0x1b1818d8
fltbits 0x1f1818f8,0x1c981938,0x1a981958,0x1e981978,0x1d9819b8,0x1b9819d8
fltbits 0x1f9819f8,0x1c581a38,0x1e581a78,0x1d581ab8,0x1b581ad8,0x1f581af8
fltbits 0x1cd81b38,0x1ed81b78,0x1dd81bb8,0x1fd81bf8,0x1e381c78,0x1d381cb8
fltbits 0x1f381cf8,0x1eb81d78,0x1fb81df8,0x1f781ef8
_fftend:

