/* AT&T DSP32,  64 Point FFT			  Sep 29, 1988  */
/*								*/
/* fft6.s							*/
/*     -- 64 point in-place decimation-in-time, radix 2		*/
/*        complex FFT						*/
/*     -- Use stored bit reversal table in same bank as routine	*/
/*     -- First and second stages combined and separate also	*/
/*        the last stage from the intermediate stages.		*/
/*     -- Adapted from Rabiner and Gold, Theory and Application	*/
/*        of Digital Signal Processing, p.367.			*/
/*     -- Routine requires 632 bytes of memory			*/
/*        Input data requires 512 bytes				*/
/*     -- Execution time for the 25 MHz DSP32 is 0.308 ms.	*/

/* Driver Test Program -----------------------------------------*/

 .global  main, pause, _fftIO
 .global  _fftend, _fft6, _fft1KY

	.rsect ".bank0"

	goto main
	nop

_fftIO:	8*float 0.707,0.,2.,0.,0.707,0.,-1.,0.,-0.707,0.,0.,0.,-0.707,0.,-1.,0.
.rsect ".hi_ram"

	main:	call _fft6 (r14)
		nop

	int	_fftIO    /* arguments */
	.align  4

		pir = r1    	 /* next instr. executed after routine call */
	pause:	if(pif) goto pause
		nop

/* --------------------------------------------------------------------- */
/* The fft6.s routine */

_fft6:

#define  N	64	/* length of FFT */
#define  M	6	/* log2(N) */
#define  MM2	2	/* counter for M-2 loops */
#define  L	28	/* number of address pairs */
#define  INC	(-504)

#define i r1
#define j r2
#define k r3
#define iz r4
#define kz r5
#define w r6
#define le1 r7
#define leo r8
#define jcnt r9
#define lx r10
#define Mcnt r12
#define lem r15
#define cnt r16
#define ur a2
#define ui a3
#define ONE *r13++
#define _ERO *r13--
#define _fftTWO a1

	r11 = *r14		/* address of data */

/* Performs bit-reversal prior to FFT, using table lookup */

		r3 = _fft1KY	/* pointer to first address */
		r4 = r3 + 2	/* pointer to second address in pair */
		r5 = L - 2	/* loop counter */
		r15 = 4		/* increment */

        	r1 = *r3++r15	/* get first address (A) */
_fft6A:	   r2 = *r4++r15	/* get second address (B) */
		r1 = r1 + r11	/* optional offsets to array */
		r2 = r2 + r11
		r6 = r1

		*r1++=	a0 = *r2++	/* ReB -> ReA */
		*r1--=	a0 = *r2--	/* ImB -> ImA */
		*r2++=	a0 = *r6++	/* ReA -> ReB */
		*r2--=	a0 = *r6--	/* ImA -> ImB */
		if (r5-- >=0) goto _fft6A
        	r1 = *r3++r15	/* get first address (A) */


/* fft - Calculates 64 point complex in-place DIT radix 2 FFT*/

/* Stage 1 and 2 combined, i.e., use N/4 butterflies */

		r16 = N/4 - 2	/* loop counter */
		r1 = r11
		r2 = r1 + 8	/* data item B */
		r3 = r1 + 16	/* data item C */
		r4 = r1 + 24	/* data item D */

		r5 = r1		/* pointer for new A (A') */
		r6 = r2		/* B' */
		r7 = r3		/* C' */
		r8 = r4		/* D' */

		r15 = 28	/* increment for next group of four */

_fft6B:	        a0 = *r1++ + *r2++	/* ReA + ReB */
		a1 = *r3++ + *r4++	/* ReC + ReD */
		a2 = *r1-- + *r2--	/* ImA + ImB */
		a3 = *r3-- + *r4--	/* ImC + ImD */

	*r5++=	a0 = a0 + a1		/* ReA' = ReA+ReB+ReC+ReD */
		a0 = *r1++ - *r2++	/* ReA - ReB */
	*r7++=	a1 = -a1 + a0		/* ReC' = -(ReC+ReD)+(ReA+ReB) */
  	*r5++r15=  a2 = a2 + a3		/* ImA' = ImA+ImB+ImC+ImD */
		a2 = *r3++ - *r4++	/* ReC - ReD */
        *r7++r15=  a3 = -a3 + a2	/* ImC' = -(ImC+ImD)+(ImA+ImB) */
		a3 = *r1++r15 - *r2++r15	/* ImA - ImB */
		a1 = *r3++r15 - *r4++r15	/* ImC - ImD */

	*r6++=	a0 = a1 + a0		/* ReB' = (ImC-ImD)+(ReA-ReB) */
	*r8++=  a1 = -a1 + a0		/* ReD' = -(ImC-ImD)+(ReA-ReB) */
        *r6++r15 = a3 = -a2 + a3	/* ImB' = -(ReC-ReD)+(ImA-ImB) */
		if (r16-- >=0) goto _fft6B
	*r8++r15 = a2 = a2 + a3		/* ImD' = (ReC-ReD)+(ImA-ImB) */

/* Stage 3 to M, radix 2 ---------------------------------------------- */

	Mcnt = M + (-5)	        /* M - 5, counter for M (stages) */
        r13=_ffttwo
        _fftTWO = *r13++
        w=_fftW			/* Pointer to twiddle table */
        leo=16			/* Butterfly index difference */
        le1=4			/* Number of twiddles for the stage */

	lx = N/4              /* N/4, length of FFT *************** */

/* Stage 3 to M-1 --------------------------------- */

_fft30: lx=lx/2			/* Number of Butterflies */
        leo=leo*2
        lem=leo*2
        lem=lem-4
	r17 = lem + INC		/* Increment for the next twiddle */
        j=*r14			/* _fftIO */
        ur=*w++			/* ************ */
        ui=*w--			/* ************ */
        jcnt=le1+(-3)		/* Counter for number of twiddles */
        le1=le1*2		/* Number of twiddles for next stage */

        i=j
	iz=i
        k=i
        k=k+leo
	kz=k
        cnt=lx+(-3)		/* Counter for number of butterflies */

/* Stages 3 to M-1, Butterfly # 1 --------------------------------- */

_fft10d:
        *iz++=a0=*i + *k++		/* RE1 + RE2 */
        *kz++=a0=-a0 + *i++ * _fftTWO	/* RE1 - RE2 */
   *iz++lem=a0=*i + *k++lem		/* IM1 + IM2 */
        if (cnt-- >= 0) goto _fft10d
   *kz++lem=a0= -a0 + *i++lem  *_fftTWO	/* IM1 - IM2 */

        *iz++=a0=*i + *k++		/* RE1 + RE2 */
        *kz++=a0=-a0 + *i++ * _fftTWO	/* RE1 - RE2 */
   *iz++r17=a0=*i + *k++r17		/* IM1 + IM2 */
   *kz++r17=a0= -a0 + *i++r17  *_fftTWO	/* IM1 - IM2 */


/* Stages 3 to M-1, Butterfly # 2 --------------------------------- */

_fft20: cnt=lx+(-3)
_fft10: a0 = *i  + ur * *k++       /* T=_fftIO(i)r+Ur*_fftIO(k)r */
        *iz++=a0=a0- *k-- * ui    /* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
        *kz++=a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)r=-T+2*_fftIO(i)r */
        a0 = *i  + ui * *k++       /* T=_fftIO(i)i+Ui*_fftIO(k)r */
   *iz++lem=a0=a0+ *k++lem * ur    /* _fftIO(i)i=T=T+Ur*_fftIO(k)i */
        if (cnt-- >= 0) goto _fft10
   *kz++lem=a0=-a0+ *i++lem * _fftTWO    /* _fftIO(k)i=-T+2*_fftIO(i)i */

         a0 = *i  + ur * *k++       /* T=_fftIO(i)r+Ur*_fftIO(k)r */
        *iz++=a0=a0- *k-- * ui    /* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
        *kz++=a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)r=-T+2*_fftIO(i)r */
        a0 = *i  + ui * *k++       /* T=_fftIO(i)i+Ui*_fftIO(k)r */
   *iz++r17=a0=a0+ *k++r17 * ur    /* _fftIO(i)i=T=T+Ur*_fftIO(k)i */
   *kz++r17=a0=-a0+ *i++r17 * _fftTWO    /* _fftIO(k)i=-T+2*_fftIO(i)i */

        a0 = ur * *w++          /* Compute new twiddle */
        ur = a0 - ui * *w         /*   U = U * _fftW       */
        a0 = ur * *w--
        ui = a0 + ui * *w
        if (jcnt-- >= 0) goto _fft20
        j=j+8

        if (Mcnt-- >= 0) goto _fft30
        w=w+8

/* Stage M -------------------------------------------- */
	leo=leo*2
        i=*r14			/* _fftIO */
        ur=ONE
        ui=_ERO
        jcnt=le1+(-2)

	iz=i
        k=i
        k=k+leo
	kz=k

_fft20A: a0 = *i  + ur * *k++       /* T=_fftIO(i)r+Ur*_fftIO(k)r */
        *iz++=a0=a0- *k-- * ui    /* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
        *kz++=a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)r=-T+2*_fftIO(i)r */
        a0 = *i  + ui * *k++       /* T=_fftIO(i)i+Ui*_fftIO(k)r */
   *iz++ =a0=a0+ *k++ * ur    /* _fftIO(i)i=T=T+Ur*_fftIO(k)i */
   *kz++ =a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)i=-T+2*_fftIO(i)i */

        a0 = ur * *w++          /* Compute new twiddle */
        ur = a0 - ui * *w         /*   U = U * _fftW       */
        a0 = ur * *w--
        if (jcnt-- >= 0) goto _fft20A
        ui = a0 + ui * *w

/* --------------------------------------------------- */

 	goto r14+4
	nop

/* Constant table ------------------------------------ */

_ffttwo:    float 2.0, 1.0, 0.0

/* Twiddle table ------------------------------------- */

_fftW:  float  0.7071068, -0.7071068   /* cos(pi/4),-sin(pi/4) */
  float  0.9238795, -0.3826834   /* cos(pi/8),-sin(pi/8) */
  float  0.9807853, -0.1950903   /*cos(pi/16),-sin(pi/16)*/
  float  0.9951847, -0.0980171   /*cos(pi/32),-sin(pi/32)*/

/* ---------------------------------------------------------- */

/* Bit reveral table */

_fft1KY:

fltbits 0x01000008, 0x00800010, 0x01800018, 0x00400020, 0x01400028
fltbits 0x00c00030, 0x01c00038, 0x01200048, 0x00a00050, 0x01a00058
fltbits 0x01600068, 0x00e00070, 0x01e00078, 0x01100088, 0x01900098
fltbits 0x015000a8, 0x00d000b0, 0x01d000b8, 0x013000c8, 0x01b000d8
fltbits 0x017000e8, 0x01f000f8, 0x01880118, 0x01480128, 0x01c80138
fltbits 0x01a80158, 0x01e80178, 0x01d801b8

_fftend:

