/* AT&T DSP32C,  1k Point FFT			  Sep 29, 1988  */
/*								*/
/* fftc1k.s							*/
/*	-- 1024 point in-place decimation-in-time, radix 2	*/
/*	   complex FFT for DSP32C				*/
/*	-- Use stored bit reversal table and twiddle factors	*/
/*	-- Adapted from Rabiner and Gold, Theory and Application*/
/*	   of Digital Signal Processing, p.367.			*/
/*	-- Routine requires 6432 bytes of memory (code size 348	*/
/*	   bytes, bit reversal table 1988 bytes, and twiddle	*/
/*	   table 4096 bytes.					*/
/*	   Input data requires 8192 bytes			*/
/*      -- Execution time for the 25 MHz DSP32 is 3.209 ms.	*/

/* Driver Test Program -----------------------------------------*/

.global pause,_fftIO,_fftA,_fftB,butfly,_fftc1k,_fftIOi,exc,bfly1,swap
.global revaddr,_fft20,_fft30
.global revaddrE, _fftend, _fftW, _fftWend

main:	call _fftc1k (r14)
	nop
int	1024, 10		/* arguments */
int	_fftIO
int	_fftIOi
.align  4

	pir = r1     /* next instr. executed after routine call */
pause:	goto pause
	nop

	.rsect ".bank1"

_fftIO:	128*float 0.707,2.,0.707,-1.,-0.707,0.,-0.707,-1. /* real part */

	.rsect ".bank0"
_fftIOi: 1024*float 0.0			/* imaginary part of data */


/* ---------------------------------------------------------------- */
/* The fftc1k.s routine */

	.rsect ".rom2"
_fftc1k:

#define i r1
#define j r18
#define cnt r2
#define h r5
#define k r3
#define w r4
#define jcnt r10
#define lcnt r7
#define le1 r8
#define leo r9
#define lx r22
#define iz r11
#define kz r12
#define lem r15
#define ur a2
#define ui a3
#define _TWO r6
#define ONE *r13++
#define _ERO *r13--


/* Read in parameters */
	lx=*r14++		/* N */
	lcnt=*r14++		/* M */
	r16=*r14++		/* pointer to _fftIO */
	r17 = *r14-- 		/* pointer to _fftIOi */

/* In place bit reversal ----------------------------- */
	r21=lx
	r21=r21*2
	r21=r21*2
	r20=revaddr
_fftA:	r2e=*r20++		/* # of swaps (minus 1) required */
	r17=r17-r16		/* + delta - imag higher in memory */
	r19= -r17		/* - delta */

exc:	do  8, cnt
	i=*r20++
	h=*r20++
	i=i+r16
	h=h+r16
swap:	*i++r17 = a0 = *h++r17	/* Begin complex exchange */
	*i++r19 = a0 = *h++r19	/*    T = _fftIO(J)   */
	nop			/* Z conflict */
	*h++r17 = a0 = *i++r17	/*  _fftIO(J) = _fftIO(I)  */
	*h++r19 = a0 = *i++r19	/*  _fftIO(I) = _fftIO(J)  */
/* end do */

/* Begin FFT calculation ----------------------------- */

_fftB:	leo=2
	lcnt = lcnt + (-3)	/* M - 3 - one less */
	le1=1
	w=_fftW
	r13=_fftone
	_TWO = _ffttwo

_fft30: r16=lx*2
	r16=r16*2
	r16=r16-4
	lx=lx/2
	leo=leo*2
	lem=leo*2
	lem=lem-r17
	r18=lem-r21
	r18=r18+4
	ur = *w++		/*   U = * _fftW       */
	ui = *w++r16
	i=*r14			/* _fftIO */
	jcnt=le1+(-2)
	le1=le1*2
	iz=i
	k=i+leo
	kz=k

_fft20:
	cnt=lx-2		/* one less for do loop, and one less */
				/* for updating differently */

/* Butterfly ----------------------------------------- */

butfly:	do  5, cnt
	a0=*i++r17 + ur * *k++r17	/* T0=_fftIO(i)r+Ur*_fftIO(k)r */
	*iz++r17=a0=a0 - *k++r19 *ui	/* _fftIO(i)r=T0=T0-Ui*_fftIO(k)i */
	a1=*i++r19 + ui**k++r17 	/* T1=_fftIO(i)i+Ui*_fftIO(k)r */
	*kz++r17=a0=-a0+*i++r17* *_TWO 	/* _fftIO(k)r=T0=-T0+2*_fftIO(i)r */
	*iz++lem=a1=a1+ *k++lem*ur	/* _fftIO(i)i=T1=T1+Ur*_fftIO(k)i*/
	*kz++lem=a1=-a1+*i++lem**_TWO	/* _fftIO(k)i=-T1+2*_fftIO(i)i */
/* end do */

/* Last time through - update with different pointer */
	a0=*i++r17 + ur * *k++r17	/* T0=_fftIO(i)r+Ur*_fftIO(k)r */
	*iz++r17=a0=a0 - *k++r19 *ui	/* _fftIO(i)r=T0=T0-Ui*_fftIO(k)i */
	a1=*i++r19 + ui**k++r17 	/* T1=_fftIO(i)i+Ui*_fftIO(k)r */
	*kz++r17=a0=-a0+*i++r17* *_TWO 	/* _fftIO(k)r=T0=-T0+2*_fftIO(i)r */
	*iz++r18=a1=a1+ *k++r18*ur	/* _fftIO(i)i=T1=T1+Ur*_fftIO(k)i*/
	*kz++r18=a1=-a1+*i++r18**_TWO	/* _fftIO(k)i=-T1+2*_fftIO(i)i */

	ur = *w++		  /* Fetch new twiddle  */
	if (jcnt-- >= 0) goto _fft20
	ui = *w++r16

	if (lcnt-- >= 0) goto _fft30
	w=_fftW

	/*Last Stage of Butterflies */
	i=*r14			/* _fftIO */
	ur = *w++		/*   U = * _fftW       */
	ui = *w++
	lx=lx/2
	leo=leo*2
	lem=4+r19		/* -r17 + 4 */
	iz=i
	k=i+leo
	kz=k
bfly1:	cnt=le1 -1

/* Butterfly and Twiddle Factor Update combined --------------------- */

	do  7, cnt
	a0 = *i++r17  + ur * *k++r17	/* T=_fftIO(i)r+Ur*_fftIO(k)r */
	*iz++r17=a0=a0- *k++r19 *ui	/* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
	a1 = *i++r19  +  ui**k++r17	/* T=_fftIO(i)i+Ui*_fftIO(k)r */
	*kz++r17=a0=-a0+*i++r17* *_TWO 	/* _fftIO(k)r=-T+2*_fftIO(i)r */
	ur = *w++			/*   U = * _fftW       */
	*iz++lem=a1=a1+ *k++lem*ur	/* _fftIO(i)i=T=T+Ur*_fftIO(k)i*/
	ui = *w++
	*kz++lem=a1=-a1+*i++lem**_TWO	/* _fftIO(k)i=-T+2*_fftIO(i)i */
/* end do */

 	goto r14+4
	nop

/* Constant table ------------------------------------ */

_ffttwo:    float 2.0
_fftone:     float 1.0, 0.0
_fftend:

/* Full Twiddle table ------------------------------------- */

	.rsect ".bank0"
_fftW:
float	1.000000,0.000000, 0.999981,-0.006136, 0.999925,-0.012272
float	0.999831,-0.018407, 0.999699,-0.024541, 0.999529,-0.030675
float	0.999323,-0.036807, 0.999078,-0.042938, 0.998796,-0.049068
float	0.998476,-0.055195, 0.998118,-0.061321, 0.997723,-0.067444
float	0.997291,-0.073565, 0.996821,-0.079683, 0.996313,-0.085797
float	0.995768,-0.091909, 0.995185,-0.098017, 0.994565,-0.104122
float	0.993907,-0.110222, 0.993212,-0.116319, 0.992480,-0.122411
float	0.991710,-0.128498, 0.990903,-0.134581, 0.990059,-0.140658
float	0.989177,-0.146731, 0.988258,-0.152797, 0.987302,-0.158858
float	0.986309,-0.164913, 0.985278,-0.170962, 0.984211,-0.177005
float	0.983106,-0.183040, 0.981965,-0.189069, 0.980786,-0.195091
float	0.979570,-0.201105, 0.978318,-0.207112, 0.977029,-0.213111
float	0.975703,-0.219102, 0.974340,-0.225084, 0.972941,-0.231059
float	0.971505,-0.237024, 0.970032,-0.242981, 0.968523,-0.248928
float	0.966977,-0.254866, 0.965395,-0.260795, 0.963777,-0.266713
float	0.962122,-0.272622, 0.960431,-0.278520, 0.958704,-0.284408
float	0.956941,-0.290285, 0.955142,-0.296152, 0.953307,-0.302007
float	0.951436,-0.307851, 0.949529,-0.313683, 0.947587,-0.319503
float	0.945608,-0.325311, 0.943595,-0.331107, 0.941545,-0.336891
float	0.939460,-0.342662, 0.937340,-0.348420, 0.935184,-0.354165
float	0.932994,-0.359896, 0.930768,-0.365614, 0.928507,-0.371318
float	0.926211,-0.377009, 0.923881,-0.382685, 0.921515,-0.388346
float	0.919115,-0.393993, 0.916680,-0.399626, 0.914211,-0.405243
float	0.911707,-0.410845, 0.909169,-0.416431, 0.906597,-0.422002
float	0.903990,-0.427557, 0.901350,-0.433096, 0.898676,-0.438618
float	0.895967,-0.444124, 0.893226,-0.449613, 0.890450,-0.455085
float	0.887641,-0.460541, 0.884798,-0.465978, 0.881923,-0.471399
float	0.879013,-0.476801, 0.876071,-0.482186, 0.873096,-0.487552
float	0.870088,-0.492900, 0.867047,-0.498230, 0.863974,-0.503541
float	0.860868,-0.508833, 0.857730,-0.514105, 0.854559,-0.519358
float	0.851356,-0.524592, 0.848121,-0.529806, 0.844855,-0.535000
float	0.841556,-0.540174, 0.838226,-0.545328, 0.834864,-0.550461
float	0.831471,-0.555573, 0.828046,-0.560665, 0.824590,-0.565735
float	0.821104,-0.570784, 0.817586,-0.575811, 0.814037,-0.580817
float	0.810458,-0.585801, 0.806849,-0.590763, 0.803208,-0.595703
float	0.799538,-0.600620, 0.795838,-0.605515, 0.792107,-0.610387
float	0.788347,-0.615235, 0.784558,-0.620061, 0.780738,-0.624863
float	0.776889,-0.629642, 0.773011,-0.634397, 0.769104,-0.639129
float	0.765168,-0.643836, 0.761203,-0.648519, 0.757209,-0.653177
float	0.753188,-0.657811, 0.749137,-0.662420, 0.745058,-0.667005
float	0.740952,-0.671564, 0.736817,-0.676097, 0.732655,-0.680606
float	0.728465,-0.685088, 0.724248,-0.689546, 0.720003,-0.693976
float	0.715731,-0.698381, 0.711433,-0.702760, 0.707107,-0.707112
float	0.702755,-0.711437, 0.698377,-0.715736, 0.693972,-0.720008
float	0.689541,-0.724252, 0.685084,-0.728470, 0.680601,-0.732660
float	0.676093,-0.736822, 0.671559,-0.740957, 0.667000,-0.745063
float	0.662416,-0.749142, 0.657807,-0.753193, 0.653173,-0.757215
float	0.648514,-0.761208, 0.643832,-0.765173, 0.639124,-0.769110
float	0.634393,-0.773017, 0.629638,-0.776895, 0.624859,-0.780744
float	0.620057,-0.784563, 0.615231,-0.788353, 0.610383,-0.792113
float	0.605511,-0.795844, 0.600616,-0.799544, 0.595699,-0.803214
float	0.590759,-0.806855, 0.585797,-0.810464, 0.580813,-0.814043
float	0.575808,-0.817592, 0.570780,-0.821110, 0.565731,-0.824597
float	0.560661,-0.828052, 0.555570,-0.831477, 0.550457,-0.834870
float	0.545324,-0.838232, 0.540171,-0.841563, 0.534997,-0.844861
float	0.529803,-0.848128, 0.524589,-0.851363, 0.519355,-0.854566
float	0.514102,-0.857737, 0.508829,-0.860875, 0.503537,-0.863981
float	0.498226,-0.867054, 0.492897,-0.870095, 0.487549,-0.873103
float	0.482182,-0.876078, 0.476798,-0.879021, 0.471395,-0.881930
float	0.465975,-0.884806, 0.460537,-0.887648, 0.455082,-0.890458
float	0.449610,-0.893233, 0.444120,-0.895975, 0.438615,-0.898683
float	0.433092,-0.901358, 0.427553,-0.903998, 0.421998,-0.906605
float	0.416428,-0.909177, 0.410841,-0.911715, 0.405239,-0.914219
float	0.399622,-0.916688, 0.393990,-0.919123, 0.388343,-0.921524
float	0.382681,-0.923889, 0.377005,-0.926220, 0.371315,-0.928516
float	0.365611,-0.930777, 0.359893,-0.933002, 0.354161,-0.935193
float	0.348416,-0.937349, 0.342658,-0.939469, 0.336887,-0.941554
float	0.331103,-0.943604, 0.325307,-0.945617, 0.319499,-0.947596
float	0.313679,-0.949538, 0.307847,-0.951445, 0.302003,-0.953316
float	0.296148,-0.955151, 0.290281,-0.956951, 0.284404,-0.958714
float	0.278516,-0.960441, 0.272618,-0.962132, 0.266709,-0.963787
float	0.260790,-0.965405, 0.254862,-0.966987, 0.248924,-0.968533
float	0.242976,-0.970042, 0.237020,-0.971515, 0.231054,-0.972951
float	0.225080,-0.974350, 0.219097,-0.975713, 0.213106,-0.977039
float	0.207107,-0.978329, 0.201100,-0.979581, 0.195086,-0.980797
float	0.189064,-0.981975, 0.183035,-0.983117, 0.177000,-0.984221
float	0.170957,-0.985289, 0.164909,-0.986320, 0.158853,-0.987313
float	0.152792,-0.988269, 0.146726,-0.989188, 0.140653,-0.990070
float	0.134576,-0.990914, 0.128493,-0.991722, 0.122406,-0.992491
float	0.116313,-0.993224, 0.110217,-0.993919, 0.104116,-0.994576
float	0.098012,-0.995197, 0.091903,-0.995780, 0.085792,-0.996325
float	0.079677,-0.996832, 0.073559,-0.997303, 0.067438,-0.997735
float	0.061315,-0.998130, 0.055189,-0.998488, 0.049062,-0.998808
float	0.042932,-0.999090, 0.036801,-0.999335, 0.030669,-0.999542
float	0.024535,-0.999711, 0.018400,-0.999843, 0.012265,-0.999937
float	0.006129,-0.999994, -0.000007,-1.000013, -0.006143,-0.999994
float	-0.012278,-0.999937, -0.018414,-0.999843, -0.024548,-0.999712
float	-0.030682,-0.999542, -0.036814,-0.999335, -0.042946,-0.999091
float	-0.049075,-0.998808, -0.055203,-0.998489, -0.061328,-0.998131
float	-0.067452,-0.997736, -0.073572,-0.997304, -0.079690,-0.996834
float	-0.085805,-0.996326, -0.091917,-0.995781, -0.098025,-0.995198
float	-0.104130,-0.994578, -0.110230,-0.993920, -0.116327,-0.993225
float	-0.122419,-0.992493, -0.128507,-0.991723, -0.134589,-0.990916
float	-0.140667,-0.990072, -0.146739,-0.989190, -0.152806,-0.988271
float	-0.158867,-0.987315, -0.164922,-0.986322, -0.170971,-0.985291
float	-0.177013,-0.984224, -0.183049,-0.983119, -0.189078,-0.981978
float	-0.195100,-0.980799, -0.201114,-0.979584, -0.207121,-0.978331
float	-0.213120,-0.977042, -0.219111,-0.975716, -0.225094,-0.974353
float	-0.231068,-0.972954, -0.237034,-0.971518, -0.242990,-0.970045
float	-0.248938,-0.968536, -0.254876,-0.966990, -0.260805,-0.965408
float	-0.266724,-0.963790, -0.272632,-0.962135, -0.278531,-0.960445
float	-0.284419,-0.958717, -0.290296,-0.956954, -0.296162,-0.955155
float	-0.302017,-0.953320, -0.307861,-0.951449, -0.313693,-0.949542
float	-0.319514,-0.947600, -0.325322,-0.945621, -0.331118,-0.943608
float	-0.336902,-0.941558, -0.342673,-0.939473, -0.348431,-0.937353
float	-0.354176,-0.935198, -0.359908,-0.933007, -0.365626,-0.930781
float	-0.371330,-0.928520, -0.377020,-0.926224, -0.382697,-0.923894
float	-0.388358,-0.921528, -0.394005,-0.919128, -0.399638,-0.916693
float	-0.405255,-0.914224, -0.410857,-0.911720, -0.416443,-0.909182
float	-0.422014,-0.906610, -0.427569,-0.904003, -0.433108,-0.901363
float	-0.438631,-0.898688, -0.444137,-0.895980, -0.449626,-0.893238
float	-0.455098,-0.890463, -0.460554,-0.887653, -0.465992,-0.884811
float	-0.471412,-0.881935, -0.476814,-0.879026, -0.482199,-0.876084
float	-0.487566,-0.873109, -0.492914,-0.870101, -0.498243,-0.867060
float	-0.503554,-0.863986, -0.508846,-0.860881, -0.514119,-0.857742
float	-0.519372,-0.854572, -0.524606,-0.851369, -0.529820,-0.848134
float	-0.535014,-0.844867, -0.540188,-0.841568, -0.545342,-0.838238
float	-0.550475,-0.834876, -0.555588,-0.831483, -0.560679,-0.828058
float	-0.565749,-0.824603, -0.570798,-0.821116, -0.575826,-0.817598
float	-0.580832,-0.814049, -0.585816,-0.810470, -0.590778,-0.806861
float	-0.595718,-0.803221, -0.600635,-0.799550, -0.605530,-0.795850
float	-0.610402,-0.792119, -0.615250,-0.788359, -0.620076,-0.784569
float	-0.624879,-0.780750, -0.629657,-0.776901, -0.634413,-0.773023
float	-0.639144,-0.769116, -0.643851,-0.765180, -0.648534,-0.761215
float	-0.653193,-0.757221, -0.657827,-0.753199, -0.662436,-0.749149
float	-0.667020,-0.745070, -0.671579,-0.740963, -0.676113,-0.736829
float	-0.680622,-0.732666, -0.685105,-0.728476, -0.689562,-0.724259
float	-0.693993,-0.720014, -0.698398,-0.715743, -0.702776,-0.711444
float	-0.707128,-0.707118, -0.711454,-0.702766, -0.715753,-0.698388
float	-0.720024,-0.693983, -0.724269,-0.689552, -0.728487,-0.685095
float	-0.732677,-0.680612, -0.736839,-0.676104, -0.740974,-0.671570
float	-0.745081,-0.667011, -0.749159,-0.662427, -0.753210,-0.657818
float	-0.757232,-0.653184, -0.761226,-0.648525, -0.765191,-0.643842
float	-0.769127,-0.639135, -0.773034,-0.634404, -0.776912,-0.629649
float	-0.780761,-0.624870, -0.784581,-0.620067, -0.788371,-0.615242
float	-0.792131,-0.610393, -0.795861,-0.605521, -0.799562,-0.600626
float	-0.803232,-0.595709, -0.806872,-0.590769, -0.810482,-0.585807
float	-0.814062,-0.580823, -0.817610,-0.575818, -0.821128,-0.570790
float	-0.824615,-0.565741, -0.828071,-0.560671, -0.831496,-0.555579
float	-0.834889,-0.550467, -0.838251,-0.545334, -0.841581,-0.540180
float	-0.844880,-0.535006, -0.848147,-0.529812, -0.851382,-0.524598
float	-0.854585,-0.519364, -0.857756,-0.514111, -0.860894,-0.508838
float	-0.864000,-0.503546, -0.867074,-0.498235, -0.870114,-0.492906
float	-0.873123,-0.487558, -0.876098,-0.482191, -0.879040,-0.476807
float	-0.881949,-0.471404, -0.884825,-0.465983, -0.887668,-0.460546
float	-0.890477,-0.455090, -0.893253,-0.449618, -0.895995,-0.444129
float	-0.898703,-0.438623, -0.901378,-0.433100, -0.904018,-0.427561
float	-0.906625,-0.422006, -0.909197,-0.416435, -0.911735,-0.410849
float	-0.914239,-0.405247, -0.916708,-0.399630, -0.919143,-0.393997
float	-0.921544,-0.388350, -0.923909,-0.382688, -0.926240,-0.377012
float	-0.928536,-0.371322, -0.930797,-0.365617, -0.933023,-0.359899
float	-0.935214,-0.354168, -0.937369,-0.348423, -0.939490,-0.342664
float	-0.941575,-0.336894, -0.943624,-0.331110, -0.945638,-0.325314
float	-0.947616,-0.319505, -0.949559,-0.313685, -0.951466,-0.307852
float	-0.953337,-0.302009, -0.955172,-0.296154, -0.956972,-0.290287
float	-0.958735,-0.284410, -0.960462,-0.278522, -0.962153,-0.272623
float	-0.963808,-0.266715, -0.965426,-0.260796, -0.967008,-0.254867
float	-0.968554,-0.248929, -0.970063,-0.242981, -0.971536,-0.237024
float	-0.972972,-0.231059, -0.974372,-0.225084, -0.975734,-0.219102
float	-0.977061,-0.213110, -0.978350,-0.207111, -0.979602,-0.201104
float	-0.980818,-0.195090, -0.981997,-0.189068, -0.983138,-0.183039
float	-0.984243,-0.177003, -0.985311,-0.170961, -0.986341,-0.164912
float	-0.987334,-0.158857, -0.988291,-0.152796, -0.989210,-0.146729
float	-0.990091,-0.140656, -0.990936,-0.134578, -0.991743,-0.128496
float	-0.992513,-0.122408, -0.993245,-0.116316, -0.993941,-0.110219
float	-0.994598,-0.104118, -0.995218,-0.098014, -0.995801,-0.091905
float	-0.996346,-0.085794, -0.996854,-0.079678, -0.997324,-0.073560
float	-0.997757,-0.067440, -0.998152,-0.061316, -0.998510,-0.055190
float	-0.998830,-0.049063, -0.999112,-0.042933, -0.999357,-0.036802
float	-0.999564,-0.030669, -0.999733,-0.024535, -0.999865,-0.018401
float	-0.999959,-0.012265, -1.000016,-0.006129
_fftWend:

/* bit reversed address table ------------------------------------ */

	.rsect ".rom3"
revaddr:
int24	495
fltbits	0x08000004, 0x04000008, 0x0c00000c, 0x02000010, 0x0a000014
fltbits	0x06000018, 0x0e00001c, 0x01000020, 0x09000024, 0x05000028
fltbits	0x0d00002c, 0x03000030, 0x0b000034, 0x07000038, 0x0f00003c
fltbits	0x00800040, 0x08800044, 0x04800048, 0x0c80004c, 0x02800050
fltbits	0x0a800054, 0x06800058, 0x0e80005c, 0x01800060, 0x09800064
fltbits	0x05800068, 0x0d80006c, 0x03800070, 0x0b800074, 0x07800078
fltbits	0x0f80007c, 0x08400084, 0x04400088, 0x0c40008c, 0x02400090
fltbits	0x0a400094, 0x06400098, 0x0e40009c, 0x014000a0, 0x094000a4
fltbits	0x054000a8, 0x0d4000ac, 0x034000b0, 0x0b4000b4, 0x074000b8
fltbits	0x0f4000bc, 0x08c000c4, 0x04c000c8, 0x0cc000cc, 0x02c000d0
fltbits	0x0ac000d4, 0x06c000d8, 0x0ec000dc, 0x01c000e0, 0x09c000e4
fltbits	0x05c000e8, 0x0dc000ec, 0x03c000f0, 0x0bc000f4, 0x07c000f8
fltbits	0x0fc000fc, 0x08200104, 0x04200108, 0x0c20010c, 0x02200110
fltbits	0x0a200114, 0x06200118, 0x0e20011c, 0x09200124, 0x05200128
fltbits	0x0d20012c, 0x03200130, 0x0b200134, 0x07200138, 0x0f20013c
fltbits	0x08a00144, 0x04a00148, 0x0ca0014c, 0x02a00150, 0x0aa00154
fltbits	0x06a00158, 0x0ea0015c, 0x01a00160, 0x09a00164, 0x05a00168
fltbits	0x0da0016c, 0x03a00170, 0x0ba00174, 0x07a00178, 0x0fa0017c
fltbits	0x08600184, 0x04600188, 0x0c60018c, 0x02600190, 0x0a600194
fltbits	0x06600198, 0x0e60019c, 0x096001a4, 0x056001a8, 0x0d6001ac
fltbits	0x036001b0, 0x0b6001b4, 0x076001b8, 0x0f6001bc, 0x08e001c4
fltbits	0x04e001c8, 0x0ce001cc, 0x02e001d0, 0x0ae001d4, 0x06e001d8
fltbits	0x0ee001dc, 0x09e001e4, 0x05e001e8, 0x0de001ec, 0x03e001f0
fltbits	0x0be001f4, 0x07e001f8, 0x0fe001fc, 0x08100204, 0x04100208
fltbits	0x0c10020c, 0x0a100214, 0x06100218, 0x0e10021c, 0x09100224
fltbits	0x05100228, 0x0d10022c, 0x03100230, 0x0b100234, 0x07100238
fltbits	0x0f10023c, 0x08900244, 0x04900248, 0x0c90024c, 0x02900250
fltbits	0x0a900254, 0x06900258, 0x0e90025c, 0x09900264, 0x05900268
fltbits	0x0d90026c, 0x03900270, 0x0b900274, 0x07900278, 0x0f90027c
fltbits	0x08500284, 0x04500288, 0x0c50028c, 0x0a500294, 0x06500298
fltbits	0x0e50029c, 0x095002a4, 0x055002a8, 0x0d5002ac, 0x035002b0
fltbits	0x0b5002b4, 0x075002b8, 0x0f5002bc, 0x08d002c4, 0x04d002c8
fltbits	0x0cd002cc, 0x0ad002d4, 0x06d002d8, 0x0ed002dc, 0x09d002e4
fltbits	0x05d002e8, 0x0dd002ec, 0x03d002f0, 0x0bd002f4, 0x07d002f8
fltbits	0x0fd002fc, 0x08300304, 0x04300308, 0x0c30030c, 0x0a300314
fltbits	0x06300318, 0x0e30031c, 0x09300324, 0x05300328, 0x0d30032c
fltbits	0x0b300334, 0x07300338, 0x0f30033c, 0x08b00344, 0x04b00348
fltbits	0x0cb0034c, 0x0ab00354, 0x06b00358, 0x0eb0035c, 0x09b00364
fltbits	0x05b00368, 0x0db0036c, 0x03b00370, 0x0bb00374, 0x07b00378
fltbits	0x0fb0037c, 0x08700384, 0x04700388, 0x0c70038c, 0x0a700394
fltbits	0x06700398, 0x0e70039c, 0x097003a4, 0x057003a8, 0x0d7003ac
fltbits	0x0b7003b4, 0x077003b8, 0x0f7003bc, 0x08f003c4, 0x04f003c8
fltbits	0x0cf003cc, 0x0af003d4, 0x06f003d8, 0x0ef003dc, 0x09f003e4
fltbits	0x05f003e8, 0x0df003ec, 0x0bf003f4, 0x07f003f8, 0x0ff003fc
fltbits	0x08080404, 0x0c08040c, 0x0a080414, 0x06080418, 0x0e08041c
fltbits	0x09080424, 0x05080428, 0x0d08042c, 0x0b080434, 0x07080438
fltbits	0x0f08043c, 0x08880444, 0x04880448, 0x0c88044c, 0x0a880454
fltbits	0x06880458, 0x0e88045c, 0x09880464, 0x05880468, 0x0d88046c
fltbits	0x0b880474, 0x07880478, 0x0f88047c, 0x08480484, 0x0c48048c
fltbits	0x0a480494, 0x06480498, 0x0e48049c, 0x094804a4, 0x054804a8
fltbits	0x0d4804ac, 0x0b4804b4, 0x074804b8, 0x0f4804bc, 0x08c804c4
fltbits	0x0cc804cc, 0x0ac804d4, 0x06c804d8, 0x0ec804dc, 0x09c804e4
fltbits	0x05c804e8, 0x0dc804ec, 0x0bc804f4, 0x07c804f8, 0x0fc804fc
fltbits	0x08280504, 0x0c28050c, 0x0a280514, 0x06280518, 0x0e28051c
fltbits	0x09280524, 0x0d28052c, 0x0b280534, 0x07280538, 0x0f28053c
fltbits	0x08a80544, 0x0ca8054c, 0x0aa80554, 0x06a80558, 0x0ea8055c
fltbits	0x09a80564, 0x05a80568, 0x0da8056c, 0x0ba80574, 0x07a80578
fltbits	0x0fa8057c, 0x08680584, 0x0c68058c, 0x0a680594, 0x06680598
fltbits	0x0e68059c, 0x096805a4, 0x0d6805ac, 0x0b6805b4, 0x076805b8
fltbits	0x0f6805bc, 0x08e805c4, 0x0ce805cc, 0x0ae805d4, 0x06e805d8
fltbits	0x0ee805dc, 0x09e805e4, 0x0de805ec, 0x0be805f4, 0x07e805f8
fltbits	0x0fe805fc, 0x08180604, 0x0c18060c, 0x0a180614, 0x0e18061c
fltbits	0x09180624, 0x0d18062c, 0x0b180634, 0x07180638, 0x0f18063c
fltbits	0x08980644, 0x0c98064c, 0x0a980654, 0x06980658, 0x0e98065c
fltbits	0x09980664, 0x0d98066c, 0x0b980674, 0x07980678, 0x0f98067c
fltbits	0x08580684, 0x0c58068c, 0x0a580694, 0x0e58069c, 0x095806a4
fltbits	0x0d5806ac, 0x0b5806b4, 0x075806b8, 0x0f5806bc, 0x08d806c4
fltbits	0x0cd806cc, 0x0ad806d4, 0x0ed806dc, 0x09d806e4, 0x0dd806ec
fltbits	0x0bd806f4, 0x07d806f8, 0x0fd806fc, 0x08380704, 0x0c38070c
fltbits	0x0a380714, 0x0e38071c, 0x09380724, 0x0d38072c, 0x0b380734
fltbits	0x0f38073c, 0x08b80744, 0x0cb8074c, 0x0ab80754, 0x0eb8075c
fltbits	0x09b80764, 0x0db8076c, 0x0bb80774, 0x07b80778, 0x0fb8077c
fltbits	0x08780784, 0x0c78078c, 0x0a780794, 0x0e78079c, 0x097807a4
fltbits	0x0d7807ac, 0x0b7807b4, 0x0f7807bc, 0x08f807c4, 0x0cf807cc
fltbits	0x0af807d4, 0x0ef807dc, 0x09f807e4, 0x0df807ec, 0x0bf807f4
fltbits	0x0ff807fc, 0x0c04080c, 0x0a040814, 0x0e04081c, 0x09040824
fltbits	0x0d04082c, 0x0b040834, 0x0f04083c, 0x08840844, 0x0c84084c
fltbits	0x0a840854, 0x0e84085c, 0x09840864, 0x0d84086c, 0x0b840874
fltbits	0x0f84087c, 0x0c44088c, 0x0a440894, 0x0e44089c, 0x094408a4
fltbits	0x0d4408ac, 0x0b4408b4, 0x0f4408bc, 0x0cc408cc, 0x0ac408d4
fltbits	0x0ec408dc, 0x09c408e4, 0x0dc408ec, 0x0bc408f4, 0x0fc408fc
fltbits	0x0c24090c, 0x0a240914, 0x0e24091c, 0x0d24092c, 0x0b240934
fltbits	0x0f24093c, 0x0ca4094c, 0x0aa40954, 0x0ea4095c, 0x09a40964
fltbits	0x0da4096c, 0x0ba40974, 0x0fa4097c, 0x0c64098c, 0x0a640994
fltbits	0x0e64099c, 0x0d6409ac, 0x0b6409b4, 0x0f6409bc, 0x0ce409cc
fltbits	0x0ae409d4, 0x0ee409dc, 0x0de409ec, 0x0be409f4, 0x0fe409fc
fltbits	0x0c140a0c, 0x0e140a1c, 0x0d140a2c, 0x0b140a34, 0x0f140a3c
fltbits	0x0c940a4c, 0x0a940a54, 0x0e940a5c, 0x0d940a6c, 0x0b940a74
fltbits	0x0f940a7c, 0x0c540a8c, 0x0e540a9c, 0x0d540aac, 0x0b540ab4
fltbits	0x0f540abc, 0x0cd40acc, 0x0ed40adc, 0x0dd40aec, 0x0bd40af4
fltbits	0x0fd40afc, 0x0c340b0c, 0x0e340b1c, 0x0d340b2c, 0x0f340b3c
fltbits	0x0cb40b4c, 0x0eb40b5c, 0x0db40b6c, 0x0bb40b74, 0x0fb40b7c
fltbits	0x0c740b8c, 0x0e740b9c, 0x0d740bac, 0x0f740bbc, 0x0cf40bcc
fltbits	0x0ef40bdc, 0x0df40bec, 0x0ff40bfc, 0x0e0c0c1c, 0x0d0c0c2c
fltbits	0x0f0c0c3c, 0x0c8c0c4c, 0x0e8c0c5c, 0x0d8c0c6c, 0x0f8c0c7c
fltbits	0x0e4c0c9c, 0x0d4c0cac, 0x0f4c0cbc, 0x0ecc0cdc, 0x0dcc0cec
fltbits	0x0fcc0cfc, 0x0e2c0d1c, 0x0f2c0d3c, 0x0eac0d5c, 0x0dac0d6c
fltbits	0x0fac0d7c, 0x0e6c0d9c, 0x0f6c0dbc, 0x0eec0ddc, 0x0fec0dfc
fltbits	0x0f1c0e3c, 0x0e9c0e5c, 0x0f9c0e7c, 0x0f5c0ebc, 0x0fdc0efc
fltbits	0x0fbc0f7c
revaddrE:

