/* AT&T DSP32C,  256 Point FFT				  Sep 29, 1988  */
/*									*/
/* fftc8.s  -- 256 point in-place decimation-in-time, radix 2		*/
/*	       complex FFT for DSP32C					*/
/*	    -- Use stored bit reversal table in same bank as routine	*/
/*	    -- First and second stages combined and separate also	*/
/*	       the last stage from the intermediate stages.		*/
/*	    -- Adapted from Rabiner and Gold, Theory and Application	*/
/*	       of Digital Signal Processing, p.367.			*/
/*	    -- Routine requires 1008 bytes of memory			*/
/*	       Input data requires 2048 bytes				*/
/*          -- Execution time for the 50 MHz DSP32C is 0.715 ms.	*/


/* Driver Test Program -------------------------------------------	*/

 .global  main, pause, _fftIO
 .global  _fftend, _fftc8, _fft1KY

	.rsect ".bank0"

	main:	call _fftc8 (r14)
		nop

	int	_fftIO    /* arguments */
	.align  4

		pir = r1    	 /* next instr. executed after routine call */
	pause:	if(pif) goto pause
		nop

_fftIO:	32*float 0.707,0.,2.,0.,0.707,0.,-1.,0.,-0.707,0.,0.,0.,-0.707,0.,-1.,0.
/* -------------------------------------------------------------------------- */

/* The fftc8.s routine */

.rsect ".bank1"
_fftc8:

#define  N	256	/* length of FFT */
#define  M	8	/* log2(N) */
#define  MM2	4	/* counter for M-2 loops */
#define  L	120	/* number of address pairs */
#define  INC	(-2040)

#define i r1
#define j r2
#define k r3
#define iz r4
#define kz r5
#define w r6
#define le1 r7
#define leo r8
#define jcnt r9
#define lx r10
#define Mcnt r12
#define lem r15
#define cnt r16
#define ur a2
#define ui a3
#define ONE *r13++
#define _ERO *r13--
#define _fftTWO a1

	r11e = *r14		/* address of data */

/* Performs bit-reversal prior to FFT, using table lookup */

		r3e = _fft1KY	/* pointer to first address */
		r4e = r3 + 2	/* pointer to second address in pair */
		r5 = L - 1	/* loop counter */
		r15 = 4		/* increment */

	do 8,r5
       	r1 = *r3++r15	/* get first address (A) */
        r2 = *r4++r15	/* get second address (B) */
	r1e = r1 + r11	/* optional offsets to array */
	r2e = r2 + r11
	r6e = r1

	*r1++=	a0 = *r2++	/* ReB -> ReA */
	*r1--=	a0 = *r2--	/* ImB -> ImA */
	*r2++=	a0 = *r6++	/* ReA -> ReB */
	*r2--=	a0 = *r6--	/* ImA -> ImB */

/* fft - Calculates 256 point complex in-place DIT radix 2 FFT*/

/* Stage 1 and 2 combined, i.e., use N/4 butterflies */

		r16 = N/4 - 1	/* loop counter */
		r1e = r11
		r2e = r1 + 8	/* data item B */
		r3e = r1 + 16	/* data item C */
		r4e = r1 + 24	/* data item D */

		r5e = r1		/* pointer for new A (A') */
		r6e = r2		/* B' */
		r7e = r3		/* C' */
		r8e = r4		/* D' */

		r15 = 28	/* increment for next group of four */

		do 15, r16
_fft8B:	        a0 = *r1++ + *r2++	/* ReA + ReB */
		a1 = *r3++ + *r4++	/* ReC + ReD */
		a2 = *r1-- + *r2--	/* ImA + ImB */
		a3 = *r3-- + *r4--	/* ImC + ImD */

	*r5++=	a0 = a0 + a1		/* ReA' = ReA+ReB+ReC+ReD */
		a0 = *r1++ - *r2++	/* ReA - ReB */
	*r7++=	a1 = -a1 + a0		/* ReC' = -(ReC+ReD)+(ReA+ReB) */
  	*r5++r15=  a2 = a2 + a3		/* ImA' = ImA+ImB+ImC+ImD */
		a2 = *r3++ - *r4++	/* ReC - ReD */
        *r7++r15=  a3 = -a3 + a2	/* ImC' = -(ImC+ImD)+(ImA+ImB) */
		a3 = *r1++r15 - *r2++r15	/* ImA - ImB */
		a1 = *r3++r15 - *r4++r15	/* ImC - ImD */

	*r6++=	a0 = a1 + a0		/* ReB' = (ImC-ImD)+(ReA-ReB) */
	*r8++=  a1 = -a1 + a0		/* ReD' = -(ImC-ImD)+(ReA-ReB) */
        *r6++r15 = a3 = -a2 + a3	/* ImB' = -(ReC-ReD)+(ImA-ImB) */
	*r8++r15 = a2 = a2 + a3		/* ImD' = (ReC-ReD)+(ImA-ImB) */

/* Stage 3 to M, radix 2 ---------------------------------------------- */

	Mcnt = M + (-5)	        /* M - 5, counter for M (stages) */
        r13e=_ffttwo
        _fftTWO = *r13++
        r6e=_fftW		/* Pointer to twiddle table */
        leo=16			/* Butterfly index difference */
        le1=4			/* Number of twiddles for the stage */

	lx = N/4              /* N/4, length of FFT *************** */

/* Stage 3 to M-1 --------------------------------- */

_fft30: lx=lx/2			/* Number of Butterflies */
        leo=leo*2
        lem=leo*2
        lem=lem-4
	r17 = lem + INC		/* Increment for the next twiddle */
        r2e=*r14			/* _fftIO */
        ur=*w++			/* ************ */
        ui=*w--			/* ************ */
        jcnt=le1+(-3)		/* Counter for number of twiddles */
        le1=le1*2		/* Number of twiddles for next stage */

        r1e=j
	r4e=i
        r3e=i
        r3e=k+leo
	r5e=k
        cnt=lx+(-2)		/* Counter for number of butterflies */

/* Stages 3 to M-1, Butterfly # 1 --------------------------------- */

	do 3, cnt
_fft10d:
        *iz++=a0=*i + *k++		/* RE1 + RE2 */
        *kz++=a0=-a0 + *i++ * _fftTWO	/* RE1 - RE2 */
   *iz++lem=a0=*i + *k++lem		/* IM1 + IM2 */
   *kz++lem=a0= -a0 + *i++lem  *_fftTWO	/* IM1 - IM2 */

        *iz++=a0=*i + *k++		/* RE1 + RE2 */
        *kz++=a0=-a0 + *i++ * _fftTWO	/* RE1 - RE2 */
   *iz++r17=a0=*i + *k++r17		/* IM1 + IM2 */
   *kz++r17=a0= -a0 + *i++r17  *_fftTWO	/* IM1 - IM2 */


/* Stages 3 to M-1, Butterfly # 2 --------------------------------- */

_fft20:
	do 5, cnt
_fft10: a0 = *i  + ur * *k++       /* T=_fftIO(i)r+Ur*_fftIO(k)r */
        *iz++=a0=a0- *k-- * ui    /* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
        *kz++=a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)r=-T+2*_fftIO(i)r */
        a0 = *i  + ui * *k++       /* T=_fftIO(i)i+Ui*_fftIO(k)r */
   *iz++lem=a0=a0+ *k++lem * ur    /* _fftIO(i)i=T=T+Ur*_fftIO(k)i */
   *kz++lem=a0=-a0+ *i++lem * _fftTWO    /* _fftIO(k)i=-T+2*_fftIO(i)i */

         a0 = *i  + ur * *k++       /* T=_fftIO(i)r+Ur*_fftIO(k)r */
        *iz++=a0=a0- *k-- * ui    /* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
        *kz++=a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)r=-T+2*_fftIO(i)r */
        a0 = *i  + ui * *k++       /* T=_fftIO(i)i+Ui*_fftIO(k)r */
   *iz++r17=a0=a0+ *k++r17 * ur    /* _fftIO(i)i=T=T+Ur*_fftIO(k)i */
   *kz++r17=a0=-a0+ *i++r17 * _fftTWO    /* _fftIO(k)i=-T+2*_fftIO(i)i */

        a0 = ur * *w++          /* Compute new twiddle */
        ur = a0 - ui * *w         /*   U = U * _fftW       */
        a0 = ur * *w--
        ui = a0 + ui * *w
        if (jcnt-- >= 0) goto _fft20
        r2e=j+8

        if (Mcnt-- >= 0) goto _fft30
        r6e=w+8

/* Stage M -------------------------------------------- */
	leo=leo*2
        r1e=*r14			/* _fftIO */
        ur=ONE
        ui=_ERO
        jcnt=le1+(-2)

	r4e=i
        r3e=i
        r3e=k+leo
	r5e=k

	do 9, jcnt
_fft20A: a0 = *i  + ur * *k++       /* T=_fftIO(i)r+Ur*_fftIO(k)r */
        *iz++=a0=a0- *k-- * ui    /* _fftIO(i)r=T=T-Ui*_fftIO(k)i */
        *kz++=a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)r=-T+2*_fftIO(i)r */
        a0 = *i  + ui * *k++       /* T=_fftIO(i)i+Ui*_fftIO(k)r */
   *iz++ =a0=a0+ *k++ * ur    /* _fftIO(i)i=T=T+Ur*_fftIO(k)i */
   *kz++ =a0=-a0+ *i++ * _fftTWO    /* _fftIO(k)i=-T+2*_fftIO(i)i */

        a0 = ur * *w++          /* Compute new twiddle */
        ur = a0 - ui * *w         /*   U = U * _fftW       */
        a0 = ur * *w--
        ui = a0 + ui * *w

/* --------------------------------------------------- */

 	goto r14+4
	nop

/* Constant table ------------------------------------ */

_ffttwo:    float 2.0, 1.0, 0.0

/* Twiddle table ------------------------------------- */

_fftW:  float  0.7071068, -0.7071068   /* cos(pi/4),-sin(pi/4) */
  float  0.9238795, -0.3826834   /* cos(pi/8),-sin(pi/8) */
  float  0.9807853, -0.1950903   /*cos(pi/16),-sin(pi/16)*/
  float  0.9951847, -0.0980171   /*cos(pi/32),-sin(pi/32)*/
  float 0.9987955, -4.9067674e-2  /* cos and -sin(pi/64) */
  float 0.9996988, -2.4541228e-2 /* cos and -sin(pi/128) */

/* ---------------------------------------------------------- */

/* Bit reveral table */
_fft1KY:

fltbits 0x04000008, 0x02000010, 0x06000018, 0x01000020, 0x05000028
fltbits 0x03000030, 0x07000038, 0x00800040, 0x04800048, 0x02800050
fltbits 0x06800058, 0x01800060, 0x05800068, 0x03800070, 0x07800078
fltbits 0x04400088, 0x02400090, 0x06400098, 0x014000a0, 0x054000a8
fltbits 0x034000b0, 0x074000b8, 0x04c000c8, 0x02c000d0, 0x06c000d8
fltbits 0x01c000e0, 0x05c000e8, 0x03c000f0, 0x07c000f8, 0x04200108
fltbits 0x02200110, 0x06200118, 0x05200128, 0x03200130, 0x07200138
fltbits 0x04a00148, 0x02a00150, 0x06a00158, 0x01a00160, 0x05a00168
fltbits 0x03a00170, 0x07a00178, 0x04600188, 0x02600190, 0x06600198
fltbits 0x056001a8, 0x036001b0, 0x076001b8, 0x04e001c8, 0x02e001d0
fltbits 0x06e001d8, 0x05e001e8, 0x03e001f0, 0x07e001f8, 0x04100208
fltbits 0x06100218, 0x05100228, 0x03100230, 0x07100238, 0x04900248
fltbits 0x02900250, 0x06900258, 0x05900268, 0x03900270, 0x07900278
fltbits 0x04500288, 0x06500298, 0x055002a8, 0x035002b0, 0x075002b8
fltbits 0x04d002c8, 0x06d002d8, 0x05d002e8, 0x03d002f0, 0x07d002f8
fltbits 0x04300308, 0x06300318, 0x05300328, 0x07300338, 0x04b00348
fltbits 0x06b00358, 0x05b00368, 0x03b00370, 0x07b00378, 0x04700388
fltbits 0x06700398, 0x057003a8, 0x077003b8, 0x04f003c8, 0x06f003d8
fltbits 0x05f003e8, 0x07f003f8, 0x06080418, 0x05080428, 0x07080438
fltbits 0x04880448, 0x06880458, 0x05880468, 0x07880478, 0x06480498
fltbits 0x054804a8, 0x074804b8, 0x06c804d8, 0x05c804e8, 0x07c804f8
fltbits 0x06280518, 0x07280538, 0x06a80558, 0x05a80568, 0x07a80578
fltbits 0x06680598, 0x076805b8, 0x06e805d8, 0x07e805f8, 0x07180638
fltbits 0x06980658, 0x07980678, 0x075806b8, 0x07d806f8, 0x07b80778
_fftend:

