;****************************************************************************
;                FINITE IMPULSE RESPONSE FILTER PROGRAM
;
;               For the TMS320C25 Digital Signal Processor
;
; This program implements a 15-tap FIR Low Pass filter.  It assumes that
; a TLC32044 Analog Interface Circuit is connected to the TMS320C25's
; serial port.  A receive interrupt (RXINT) occurs with every new A/D
; sample and is read through the receive serial port register, DRR (loc 0 of
; data page 0).  Outputs to the D/A converter are performed through writes
; to the serial data transmit register, DXR (location 1 of data page 0).
;****************************************************************************
;
; PAGE 7 VARIABLES
XN      EQU     113              ; STARTING LOCATION IN PAGE 7 WHERE INPUT
        ;                          SAMPLES ARE STORED
        ;                          X(N-1)= LOC 114...X(N-14)= LOC 127
YN      EQU     112
        ;
LENGTH  EQU     14
B0COFS  EQU     0FF00H           ; BEGINNING OF BLOCK B0 FILTER COEFFICIENTS
        ;                          h(15) = LOC FF00H..h(1) = LOC FFOEH
; PAGE 0 VARIABLES
TMP0    EQU     112              ; used for temporary storage
; SETUP INTERRUPT VECTORS FOR THE DXP-25 BOARD
        ORG     8000H
        B       START
        ORG     8002H          ;INT0
        NOP
        NOP
        ORG     8004H          ;TIMER
        NOP
        NOP
        ORG     8006H          ;SERIAL PORT RX
        B       RINT
        ORG     8008H          ;SERIAL PORT TX
        B       TINT
FCOFS:
        ; FIR FILTER COEFFICIENTS FOR A .03125 Fs Low Pass Filter
        ; N = 15 (15 taps)
        ; Q15 FORMAT (FIXED 16 BIT TWO'S COMPLEMENT)
        DW      0FED9H           ; h1
        DW      0102H            ; h2
        DW      044AH            ; h3
        DW      085DH            ; h4
        DW      0CA9H            ; h5
        DW      01075H           ; h6
        DW      01312H           ; h7
        DW      01400H           ; h8
        DW      01312H           ; h9
        DW      01075H           ; h10
        DW      0CA9H            ; h11
        DW      085DH            ; h12
        DW      044AH            ; h13
        DW      0102H            ; h14
        DW      0FED9H           ; h15
START:  SOVM                     ;SET OVERFLOW (SATURATION) MODE
        LDPK    0                ; SELECT PAGE 0
        LALK    0E00H            ; INITIALIZE
        SACL    TMP0             ; ST0 TO
        LST     TMP0             ; 0E00H
        LALK    003F0H           ; INITIALIZE
        SACL    TMP0             ; ST1 TO
        LST1    TMP0             ; 03F0H
        LDPK    7                ; SELECT PAGE 7
        ZAC
        LARP    0
        LRLK    0,03FFH          ; POINT AR0 TO B1
        RPTK    LENGTH           ; CLEAR INPUT BUFFER
        SACL    *-
        CNFD                     ; MAKE BLOCK B0 DATA RAM
        LRLK    0,0200H          ; POINT TO B0 DATA RAM ADDRESS
        RPTK    LENGTH           ; READ COEFFICIENTS INTO B0
        BLKP    FCOFS,*+         ; LOAD FILTER COEFFICIENTS
        CNFP                     ; NOW, CONFIGURE B0 AS PROGRAM MEMORY
        LDPK    0                ; SELECT DATA PAGE 0
        LACK    020H             ; ALLOW ONLY TXINT
        SACL    IMR              ; STORE IN INTERRUPT MASK REGISTER
        EINT
        ; SETUP TLC32044 TO A 10 KHZ SAMPLING RATE
        LALK    3
        SACL    DXR              ;SIGNAL SECONDARY COM
        LALK    03202H           ; set Fs TO 10280 HZ
        IDLE                     ; WAIT HERE UNTIL TXINT
        ZAC
        EINT
        IDLE                     ; WAIT HERE UNTIL TXINT
        EINT
        LALK    3
        SACL    DXR              ; SIGNAL SECONDARY COM
        LALK    01400H           ; 1663H
        IDLE                     ; WAIT HERE UNTIL TXINT
        ZAC
        IDLE                     ; WAIT HERE UNTIL TXINT
        ;
        LACK    011H             ; ENABLE RXINT ONLY
        SACL    IMR              ; STORE IN INTERRUPT MASK REGISTER
        EINT
LOOP:   B       LOOP
TINT:   SACL    DXR
        EINT
        RET
RINT:   LDPK    0                ; SELECT DATA PAGE 0
        LAC     DRR              ; READ INPUT SAMPLE FROM A/D
        LDPK    7                ; SELECT DATA PAGE 7
        SACL    XN               ; STORE LATEST SAMPLE IN XN
        LRLK    1,03FFH          ; POINT AR1 TO BOTTOM OF BLOCK B1
        LARP    1
        MPYK    0                ; CLEAR P REG
        ZAC                      ; CLEAR ACCUMULATOR
        RPTK    LENGTH           ; N - 1 FILTER STAGES
        MACD    B0COFS,*-        ; MULTIPLY FILTER STAGES BY COEFFICIENTS
        APAC                     ; ACCUMULATE FINAL PRODUCT
        SACH    YN,1             ; STORE OUTPUT IN YN
        LAC     YN,2             ; GET OUTPUT OF FILTER IN ACCUM
        LDPK    0                ; SELECT DATA PAGE 0
        ANDK    0FFFCH           ; MASK OFF BITS 0-1 FOR AIC
        SACL    DXR              ; OUTPUT TO D/A
        EINT
        RET
        END
