/* AT&T DSP32,  64 tap FIR Filter			  Sep 29, 1988  */
/*									*/
/* 64 tap DSP32 FIR filter.		fir64.s				*/
/* Execution requires 73 cycles per sample including I/0:		*/
/*									*/
/*	DSP32 (160ns) = 73 x 160ns = 11.68us;  85.6 kHz sample rate	*/
/*	DSP32C (80ns) = 73 x 80ns = 5.84us;  171.2 kHz sample rate	*/
/*	     (see separate DSP32C program using do loop: fir64c.s)	*/
/* Memory requirements:							*/
/*	Program:  24 words						*/
/*	Data:	  39 words (63 state variables + 64 coefficients)	*/
/* All instructions are single-cycle. Each line is a a single		*/
/*     assembly language instruction.					*/


init:	r1 = data		/* Initialize pointers to data and	*/
	r3 = data		/* coefficients.			*/
	r2 = coef
	r4 = 4			/* r4 is used as a loop counter (n-2).	*/
	r15 = -248		/* r15 and r16 are used as post-	*/
	r16 = -252		/* modification registers to 		*/
				/* re-initialize within the loop.	*/

loop:	a1  =  float(ibuf)	/* Input new sample and convert to 32b.	*/
	a0 = *r1++ * *r2++	/* Tap X(n-63) x coef b63.		*/

loop1:	a0 = a0 + (*r3++=*r1++) * *r2++	/* Repeat 10 MACs 6 times.	*/
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	if (r4-->=0) goto loop1		/* Test decrement and cond branch. */
	a0 = a0 + (*r3++=*r1++) * *r2++

	a0 = a0 + (*r3++=*r1++) * *r2++	/* X(n-2) x b2; update X(n-3)	*/

	a0 = a0 + (*r3++=*r1++r15) * *r2++	/* X(n-1) x b1; init r1.*/
	a0 = a0 + (*r3++r15=a1) * *r2++r16	/* X(n-0) x input; init */
						/* r3 and r2.		*/
	goto loop
	obuf = a0 = int(a0)	/* Latent instruction; convert to 16b	*/
				/* and output value.			*/

		/* Coefficients stored from b63 to b0 */

coef:	float	1.234872, 0.1794321e-2, -2.183211e-4

		/* remaining coefficients stored here */

.rsect ".hi_ram"	/* Assembler directive.				*/

data:	63*float 0.0	/* All state variables are initialized to 0.	*/

