/* AT&T DSP32,  67 tap FIR Filter			  Jan 24, 1989  */
/*									*/
/* 67 tap DSP32 FIR filter.		fir67.s				*/
/* Execution requires 76 cycles per sample including I/0:		*/
/*									*/
/*	DSP32 (160ns) = 76 x 160ns = 12.16us;  82.2 kHz sample rate	*/
/*	DSP32C (80ns) = 76 x 80ns = 6.08us;  164.5 kHz sample rate	*/
/*	     (see separate DSP32C program using do loop: fir67c.s)	*/
/* Memory requirements:							*/
/*									*/
/*	Program:  27 words						*/
/*	Data:	 133 words (66 state variables + 67 coefficients)	*/
/* All instructions are single-cycle. Each line is a a single		*/
/*     assembly language instruction.					*/


init:	r1 = data		/* Initialize pointers to data and	*/
	r3 = data		/* coefficients.			*/
	r2 = coef
	r4 = 4			/* r4 is used as a loop counter (n-2).	*/
	r15 = -260		/* r15 and r16 are used as post-	*/
	r16 = -264		/* modification registers to 		*/
				/* re-initialize within the loop.	*/

loop:	a1  =  float(ibuf)	/* Input new sample and convert to 32b.	*/
	a0 = *r1++ * *r2++	/* Tap X(n-66) x coef b66.		*/

loop1:	a0 = a0 + (*r3++=*r1++) * *r2++	/* Repeat 10 MACs 6 times.	*/
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	a0 = a0 + (*r3++=*r1++) * *r2++
	if (r4-->=0) goto loop1		/* Test, decrement and cond branch. */
	a0 = a0 + (*r3++=*r1++) * *r2++

	a0 = a0 + (*r3++=*r1++) * *r2++	   /*X(n-5) x b5; update X(n-6)	*/
	a0 = a0 + (*r3++=*r1++) * *r2++	   /*X(n-4) x b4; update X(n-5)	*/
	a0 = a0 + (*r3++=*r1++) * *r2++	   /*X(n-3) x b3; update X(n-4)	*/
	a0 = a0 + (*r3++=*r1++) * *r2++	   /*X(n-2) x b2; update X(n-3)	*/

	a0 = a0 + (*r3++=*r1++r15) * *r2++	/* X(n-1) x b1; init r1.*/
	a0 = a0 + (*r3++r15=a1) * *r2++r16	/* X(n-0) x input; init */
						/* r3 and r2.		*/
	goto loop
	obuf = a0 = int(a0)	/* Latent instruction; convert to 16b	*/
				/* and output value.			*/

		/* Coefficients stored from b66 to b0 */

coef:	float	1.234872, 0.1794321e-2, -2.183211e-4

		/* remaining coefficients stored here */

.rsect ".hi_ram"	/* Assembler directive.				*/

data:	66*float 0.0	/* All state variables are initialized to 0.	*/

