/*
     FIR_FILT.DSP -- C-callable FIR filter routine

     Prototypes:
     void fir_init( void );
     int fir( int sample, int pm coeffs[] );
*/

.module/ram   __fir_filt__;

#include <asm_sprt.h>

#define TAPS            22

.var/dm/ram/circ  state_[TAPS];   /* the filter state */
.var/dm/ram       state_ptr;      /* points to next location in state_ */

.global state_;
.entry  fir_filt_;
.entry  fir_init_;

/* filter initialization -- called only once */
fir_init_:
        leaf_entry;

        i1 = ^state_;
        dm(state_ptr) = i1;

        cntr = TAPS;            /* FIR Filter State = 0 */
        do state_init until ce;
state_init: dm(i1,m1) = 0;
           
        si = 0;
        dm(0x3ffe) = si;        /* DM Wait States = 0 */
        si = 0x0418;
        dm(0x3fff) = si;        /* PM Wait States = 0 */

        leaf_exit;

/* fir filter assembly routine */
fir_filt_:
        leaf_entry;

        /* save additional registers on the stack */
        dm(i4,m5) = mx0;
        dm(i4,m5) = my0;

        /* read parameters */
        mx0 = ar;               /* input sample */
        i6 = ay1;               /* pointer to coeff array */

        /* more setup */
        dis m_mode;             /* Fractional mode */
        i1 = dm(state_ptr);     /* Next location in circular buffer */
        l1 = TAPS;
        m5 = 1;

        dm(i1,m1) = mx0;        /* Move sample into delay line */

        /* start the fir filtering */
        mr = 0, mx0 = dm(i1,m1), my0 = pm(i6,m5);

        cntr = TAPS - 1;        /* MAC done once outside of loop */

        do fir_loop until ce;
fir_loop:  mr = mr + mx0*my0 (ss), mx0 = dm(i1,m1), my0 = pm(i6,m5);

        mr = mr + mx0*my0 (rnd);
        if mv sat mr;

        /* filtering done, set return value */
        ar = mr1;

        /* save pointer and restore registers */
        dm(state_ptr) = i1;
        l1 = 0;
        ena m_mode;             /* back to Integer mode */

        /* restore registers from the stack */
        i6 = i4; m5 = 1; modify(i6,m5);
        my0 = dm(i6,m5);
        mx0 = dm(i6,m5);

        leaf_exit;

.endmod;
