.module/boot=1/boot=2        gain_calculation; 

{ GAIN.DSP - Calculates the gain factor for a speech frame.

  Input and output is accomplished through global definitions.
  The following definitions are required:
  INPUT:
     i0 -> speech frame
  
  OUTPUT:
     sr1 = gain

  FUNCTIONS CALLED:
     sqrt  -  in file SQRT.DSP
  
  DESCRIPTION:
     The gain of a frame is calculated as:

        gain = sqrt(sum_over_frame(x(n)^2))

     A simpel no-speech detection is implemented, if the gain is lower
     than NOISE_FLOOR the gain is set to zero.
     The result is scaled appropriatly by GAIN_SCALE.
}

{Include constant definitions}
#include "lpc.h"

.const POLY_ORDER  = 10; 
.const NOISE_FLOOR = 0x0000;  {found as gain when no input is present}
.const GAIN_SCALE  = 1;       {appropriate scale value}

.entry    calc_gain;

.external overflow;
.external poly_approx;

.var/pm   sqrt_cff[2*POLY_ORDER];
.init     sqrt_cff: <sqrt.hex>;

calc_gain:
  
  {calulate energy of frame, as sum of input squared}
  mr=0;
  cntr = FRAME_LENGTH;
  do cor_data_loop until ce;
    si = dm(i0,m1);
    sr = ashift si by G_INP_SHIFT (hi);  {scale to avoid overflow}
    my0 = sr1;                  
  cor_data_loop: mr=mr+sr1*my0(ss);
  if mv call overflow;
  
  {set gain = 0 if under noise level}
  ay0 = NOISE_FLOOR;
  ar = mr1 - ay0;
  if gt jump speech;
    sr1 = 0;
    jump from_noise;
  speech:
    {calc the squareroot of gain^2}
    my0 = mr0;    {R(0) lsb}
    my1 = mr1;    {R(0) msb}
    ax0 = POLY_ORDER - 1;
    i6  = ^sqrt_cff;   l6 = 0;
    call poly_approx;  {sqrt function}
    si = mx0;
    sr = lshift si by 11 (lo);  {scale up by 2048, comes with the coeff's}
    sr = sr or ashift ar by 11 (hi);
    sr  = ashift sr1 by GAIN_SCALE (hi);  {shift to appropriate value}

{    sr1 = mr1;   R(0) msb}
  
  from_noise:

rts;
     
.endmod;