.MODULE/RAM/BOOT=0           half_triple_precision_iir_biquad; 
 
{ 
 
         IIR Filter of the Form: 
 
              y(n)=b0*x(n)+b1*x(n-1)+b2*x(n-2)+a1*y(n-1)+a2*x(n-2) 
 
         Where: 
 
              x(n), x(n-1), x(n-2), y(n-1), y(n-2) are 48-bits 
              b0, b1, b2, a1, a2 are 32-bits 
 
         Calling Parameters: 
 
              AR=lower word of input 
              AX0=middle word of input 
              AX1=upper word of input 
              I0=start address of delay line lower word in data memory stored 
                 in [x(n-1),x(n-2),y(n-1),y(n-2)] order 
              I1=start address of delay line middle word in data memory stored 
                 in [x(n-1),x(n-2),y(n-1),y(n-2)] order 
              I2=start address of delay line upper word in data memory stored 
                 in [x(n-1),x(n-2),y(n-1),y(n-2)] order 
              I4=start address of coefficients in program memory stored 
                 in MSW,LSW order [organized b0,b1,b2,a1,a2] 
              I5=start address of coefficient scaling factors in program 
                 memory (one/section) 
              L0=(2*number of sections)+2 
              L1=(2*number of sections)+2 
              L2=(2*number of sections)+2 
              L4=10*number of sections 
              L5=number of sections 
              M0=0, M1=1, M2=2, M3=-3 
              M5=1, M6=2, M7=-9 
              CNTR=number of sections 
 
         Return Values: 
 
              I0,I1,I2 -> new start address of delay line 
              SR1=upper result 
              SR0=middle result 
              AR=lower result 
 
         Altered Registers: 
 
              MX0,MY0,MR,SE,SR,SI,AF 
 
         Computation Time: 
 
              12 + (45 * number of sections) 
 
         Delay line, input sample in 1.47 (Q.47) format 
         Coefficients in 1.31 (Q.31) format 
 
} 
 
 
.ENTRY                       biq_3P; 
 
 
biq_3P: 
 
         SR1=AX1; 
         AF=PASS AR, SR0=AX0; 
         MODIFY(I4,M5); 
         MY0=PM(I4,M6); 
 
         DO biq UNTIL CE; 
 
              MR=AR*MY0(UU), MX0=DM(I0,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(UU), MX0=DM(I0,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(UU), MX0=DM(I0,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(UU), MX0=DM(I0,M3), MY0=PM(I4,M7); 
              MR=MR+MX0*MY0(UU), MY0=PM(I4,M5); 
              MR=MR(RND), SE=PM(I5,M5); 
 
              MR0=MR1; 
              MR1=MR2; 
 
              MR=MR+AR*MY0(US), MY0=PM(I4,M5); 
              MR=MR+SR0*MY0(UU), MX0=DM(I0,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I1,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(UU), MX0=DM(I0,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I1,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(UU), MX0=DM(I0,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I1,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(UU), MX0=DM(I0,M3), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I1,M3), MY0=PM(I4,M7); 
              MR=MR+MX0*MY0(UU), MX0=DM(I0,M1), MY0=PM(I4,M5); 
              MR=MR(RND); 
 
              MR0=MR1; 
              MR1=MR2; 
 
              MR=MR+SR0*MY0(US), MY0=PM(I4,M5); 
              MR=MR+SR1*MY0(SU), MX0=DM(I1,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I2,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(SU), MX0=DM(I1,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I2,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(SU), MX0=DM(I1,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I2,M1), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(SU), MX0=DM(I1,M3), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I2,M3), MY0=PM(I4,M7); 
              MR=MR+MX0*MY0(SU), MX0=DM(I1,M1), MY0=PM(I4,M6); 
 
              SR0=MR0; 
              MR0=MR1; 
              MR1=MR2; 
 
              MR=MR+SR1*MY0(SS), MX0=DM(I2,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SS), MX0=DM(I2,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SS), MX0=DM(I2,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SS), MX0=DM(I2,M3), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SS), MX0=DM(I2,M1), MY0=PM(I4,M5); 
scale_it:     SR=LSHIFT SR0 (LO), MY0=PM(I4,M6); 
              AR=SR0; 
              SR=LSHIFT SR1 BY 0 (LO); 
              DM(I0,M1)=AR, SR=SR OR LSHIFT MR0 (LO); 
              DM(I1,M1)=SR0, SR=SR OR ASHIFT MR1 (HI); 
biq:          DM(I2,M1)=SR1; 
 
         MODIFY(I0,M1); 
         MODIFY(I1,M1); 
         MODIFY(I2,M1); 
         DM(I2,M0)=AX1; 
         DM(I1,M0)=AX0, AR=PASS AF; 
         DM(I0,M0)=AR; 
 
         RTS; 
 
.ENDMOD;                                                                                                                 