.MODULE/RAM/BOOT=0           dpiir_2p2z_16_32; 
 
{ 
 
         IIR Filter of the Form: 
 
              y(n)=b0*x(n)+b1*x(n-1)+b2*x(n-2)+a1*y(n-1)+a2*x(n-2) 
 
         Where: 
 
              x(n), x(n-1), x(n-2), y(n-1), y(n-2) are 16-bits 
              b0, b1, b2, a1, a2 are 32-bits 
 
         Calling Parameters: 
 
              AX1=input sample 
              I0=start address of delay line in data memory 
                 [organized x(n-1),x(n-2),y(n-1),y(n-2)] 
              I4=start address of coefficients in program memory stored in 
                 MSW,LSW order [organized b0,b1,b2,a1,a2] 
              I5=start address of coefficient scaling factors in program 
                 memory (one/section) 
              L0=(2*number of sections)+2 
              L4=10*number of sections 
              L5=number of sections 
              M0=0, M1=1, M2=2, M3=-3 
              M4=-1, M5=1, M6=2, M7=-9 
              CNTR=number of sections 
 
         Return Values: 
 
              I0=new start address of delay line 
              SR1=MSW result 
              SR0=LSW result 
 
         Altered Registers: 
 
              MX0,MY0,MR,SE,SR 
 
         Computation Time: 
 
              8 + (15 * number of sections) 
 
         Coefficients, delay line, input sample in 1.31 (Q.31) format 
 
} 
 
 
.ENTRY                       biq_1632; 
 
biq_1632:SR1=AX1; 
         MODIFY(I4,M5); 
         MY0=PM(I4,M6); 
         SE=PM(I5,M5); 
 
         DO biq UNTIL CE; 
 
              MR=SR1*MY0(SU), MX0=DM(I0,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SU), MX0=DM(I0,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SU), MX0=DM(I0,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SU), MX0=DM(I0,M3), MY0=PM(I4,M7); 
              MR=MR+MX0*MY0(SU), MY0=PM(I4,M6); 
 
              MR0=MR1; 
              MR1=MR2; 
 
              MR=MR+SR1*MY0(SS), MX0=DM(I0,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SS), MX0=DM(I0,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SS), MX0=DM(I0,M1), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SS), MX0=DM(I0,M3), MY0=PM(I4,M6); 
              MR=MR+MX0*MY0(SS), MX0=DM(I0,M1), MY0=PM(I4,M5); 
 
              SR=ASHIFT MR1 (HI), MY0=PM(I4,M6); 
              SR=SR OR LSHIFT MR0 (LO), SE=PM(I5,M5); 
 
biq:          DM(I0,M1)=SR1; 
 
         MODIFY(I0,M1); 
         DM(I0,M0)=AX1; 
 
         RTS; 
 
.ENDMOD;                                                 