.module/boot=2/abs=0      lp2;

{  LPC_THR.DSP - talk through, encoding and decoding using LPC.

   Input: Speech samples from microphone (using autobuffering) via sport0

   Ouput: Speech samples to speeaker (using autobuffering) via sport0
   
   Modules used:
     - pre_emphasize_speech       (PREEMP.DSP)
     - gain_calculation           (GAIN.DSP)
     - autocorrelation_of_speech  (AUTOCOR.DSP)
     - durbin_double/single       (DURBIN2.DSP/DURBIN.DSP)
     - pitch_detection            (PITCH.DSP)
     - lpc_sync_synth             (SSYNTH.DSP)
     - de_emphasize_speech        (DEEMP.DSP)

     - constant header            (LPC.H)


   Description:
     This program implements a shell to demonstrate the LPC algorithm 
     on an EZ-LAB board. Speech is autobuffered in from the codec, 
     compressed, decompressed and autobuffered back out to the codec, 
     providing a "talk-through" program.


   NOTE: The framesyncs of sport0 (TFS0 & RFS0) must be tied together 
         EXTERNALY, for the autobuffering to function properly!! 
         (On the EZ-LAB's seriel connector it is pins 4 & 5)
}

{include constant definitions}
#include "lpc.h";

{Buffers used by autobuffering, input swaps between analys_buf & receive_buf,
 whereas output swaps between synth_buf & transit_buf}
.var/dm/ram/circ  analys_buf[FRAME_LENGTH];
.var/dm/ram/circ  receive_buf[FRAME_LENGTH];
.var/dm/ram/circ  synth_buf[FRAME_LENGTH];
.var/dm/ram/circ  transmit_buf[FRAME_LENGTH];

{The LPC-parameters are stored in trans_line to "simulate" transmission}
.var/dm/ram       trans_line[WORDS_PR_LPCFRAME];

{Pointers to the buffers that are NOT currently being used by autobuffering}
.var/dm/ram       p2_analysis,p2_synth;

{Intermediate variables}
.var/pm/ram       autocor_speech[FRAME_LENGTH];
.var/dm/ram/circ  k[N];
.var/dm/ram       pitch, gain;


.external   pre_emph;
.external   calc_gain;
{.external   window;}
.external   a_correlate;
.external   levinson;
.external   detect_pitch;
.external   encode;
.external   decode;
.external   clear_filter;
.external   synthesis;
.external   de_emph;

.global  k;
.global  pitch;
.global  gain;
.global  trans_line;


{load interrupt vectors}
  jump start_test; nop; nop; nop;   {reset interrupt}
  jump change_demo; rti; nop; nop;  {irq2}
  call trns_ir; rti; nop; nop;      {sport0 transmit}
  call rcv_ir; rti; nop; nop;       {sport0 receive}
  rti; nop; nop; nop;
  rti; nop; nop; nop;
  rti; nop; nop; nop;

change_demo:
  icntl = b#00111;
  ax0 = dm(0x3fff);
  ay0 = 0xfc3f; {clear bootpage settings}
  ar  = ax0 and ay0;
  ay0 = 0x0000;  {set new bootpage = 0}
  ar  = ar or ay0;
  ay0 = 0x0200;  {set bootforce bit}
  ar  = ar or ay0;
  dm(0x3fff) = ar;
  nop;
rts;


start_test:

  {configure sports etc.}
  ax0 = 0x0000;
  dm(0x3ffe) = ax0;  {dm waits = 0}
  ax0 = 0x6927;
  dm(0x3ff6) = ax0;  {set sport0 control reg}
  ax0 = 2;
  dm(0x3ff5) = ax0;  {set sclkfreq to 2.048 Mhz}
  ax0 = 255;
  dm(0x3ff4) = ax0;  {set rclkfreq to 64 khz}
  ax0 = 0x06f7;      
  dm(0x3ff3) = ax0;  {tireg = i3, tmreg = m1, tbuf enable}
                     {rireg = i7, rmreg = m5, rbuf enable} 
                     
 {Default register values, these values can always be asumed, and must
  be reset if altered}
  l0 = 0; l1 = 0; l2 = 0; l3 = 0;
  l4 = 0; l5 = 0; l6 = 0; l7 = 0;
  m0 = 0;  m4 = 0;
 
 {DEDICATED REGISTERS. These registers must NOT be altered by any
  routine at any time! (used by autobuffering)}
  m1 = 1;  m5 = 1;
  i3 = ^transmit_buf;   l3 = %transmit_buf; 
  i7 = ^receive_buf;    l7 = %receive_buf; 
  
 {Setup and clear intermediate buffers}
  i2 = ^synth_buf;      l2 = 0;
  dm(p2_synth) = i2;
  i6 = ^analys_buf;     l6 = 0;
  dm(p2_analysis) = i6;
  cntr = FRAME_LENGTH;
  ax0 = 0;
  do buf_clr until ce;
    dm(i3,m1) = ax0;
    dm(i7,m5) = ax0;
    dm(i2,m1) = ax0;
  buf_clr: dm(i6,m5) = ax0;

 {clear the synthesis filter}
  call clear_filter;


 {enable sport0}
  ax0 = 0x1038;
  dm(0x3fff) = ax0; 

 {Start transmission}
  ax0 = 0;
  tx0 = ax0;
  nop;
    
 {enable sport0}
  icntl = b#10111;
  imask = b#111000;  {Enable transmit & receive}

wait: idle;
  jump wait;

{End of main routine}

{Autobuffering interrupt routines}

trns_ir:
  {switch pointers}
  ax0 = i3;    
  ax1 = dm(p2_synth);
  i3 = ax1;   l3 = %transmit_buf;
  dm(p2_synth) = ax0;

rts;
  
rcv_ir:
  {switch pointers}
  ay0 = i7;        
  ay1 = dm(p2_analysis);
  i7 = ay1;     l7 = %receive_buf;
  dm(p2_analysis) = ay0;


 {Parameters: i0 = p2_analysis (-> speech)
  Returns   : filtered speech}
  i0 = dm(p2_analysis);  l0 = 0;
  call pre_emph;

 {Parameters: i0  = p2_analysis (-> speech)
  Returns   : sr1 = gain}
  i0 = dm(p2_analysis); l0 = 0;
  call calc_gain;
  dm(gain) = sr1;
  
  {Parameters: p2_analysis(-> speech)
   Returns   : windowed speech}
{  call window;
}  
  
 {Parameters: i0 = p2_analysis (-> speech)
  Returns   : autocor_speech[]}
  i0 = dm(p2_analysis); l0 = 0;
  i6 = ^autocor_speech; l6 = 0;
  call a_correlate;

 {Parameters: i4 -> autocor_speech[])
  Returns   : i0 -> k[]}
  i4 = ^autocor_speech; l4 = 0;
  i0 = ^k;              l0 = 0;
  call levinson;

  {Parameters: i0 -> k[], 
               i6 -> autocor_speech[]
   Returns   : si =  pitch}
  i0 = ^k;              l0 = 0;
  i6 = ^autocor_speech; l6 = 0;
  call detect_pitch;
  dm(pitch) = si; 

  {Parameters: k[], pitch, gain
   Returns   : k,pitch,gain truncated}
  call encode;

 {Parameters: k[], pitch, gain
  Returns   : k,pitch,gain unpacked}
  call decode;
 
 {Parameters: ax1 =  pitch, 
              mx1 =  gain,
              i0  -> k[]
  Returns   : i2  -> speech}
  i0 = ^k;           l0 = 0; 
  i1 = ^k + N - 1;   l1 = 0; {store k's in revers order - }
  cntr = 5; {N/2}            {required by lattice routine}
  m2 = -1;
  do reverse_ks until ce;
    ay0 = dm(i0,m0);
    ay1 = dm(i1,m0);
    dm(i0,m1) = ay1;
  reverse_ks: dm(i1,m2) = ay0;
  ax1 = dm(pitch);
  mx1 = dm(gain);
  i1 = ^k;           l1 = N;
  i2 = dm(p2_synth); l2 = 0;
  call synthesis;
  
 {Parameters: i0 = p2_analysis (-> speech)
  Returns   : filtered speech}
  i0 = dm(p2_analysis);  l0 = 0;
  call de_emph;

rts;
 {END of main code}

.endmod;


