/******************************************************************************/
/*									      */
/*		DFT based DTMF receiver for AT&T WE (R) DSP32		      */
/*									      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/*			COPYRIGHT (c) 1987 AT&T				      */
/*									      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/*	FILE = mdtmf3.s							      */
/*									      */
/*	The following code implements 16 DFT based DTMF receivers on the AT&T */
/*	WE (R) DSP32 operating with a 160nsec instruction rate.  Both the     */
/*	program and data are stored in RAM. The state variables are in bank 1 */
/*	and everything else is in bank 0.  During the 125us sample period,    */
/*	there is time to execute an additional 130 instructions.	      */
/*									      */
/*	The DSP32 receives sampled data on the serial input port in mu-law    */
/*	format using passive clocks. DMA is used to buffer the sampled data   */
/*	from each codec. An external sync pulse tells the AT&T WE (R) DSP32   */
/*	that codec 1 is beginning to transmit. The sync pulse should go high  */
/*	immediately after codec 16's sync resets and go low at least 800ns    */
/*	before the input load clock goes high.				      */
/*									      */
/*	When a digit has been validated, it is written to pdr.  The lower     */
/*	byte of pdr contains the receiver number in the upper 4 bits and the  */
/*	detected digit in the lower 4 bits.
/*									      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/*			DISCLAIMER AND NOTICE				      */
/*									      */
/*	AT&T and its SUBSIDIARIES (hereinafter AT&T) reserve the right to     */
/*	make changes in this APPLICATION NOTE and the semiconductor devices   */
/*	or device specifications to which it is directed.		      */
/*									      */
/*	AT&T makes no representations or warranties, expressly or implied.    */
/*	By way of example but not of limitation, AT&T makes no representa-    */
/*	tions or warranties of merchantability or fitness for any particular  */
/*	purpose, or that the use of the INFORMATION in this APPLICATION NOTE  */
/*	or any of it will not infringe any patent, copyright or other intel-  */
/*	lectual property right of a third party.  AT&T shall not be held to   */
/*	any liability with respect to any claim by a USER or any third party  */
/*	on account of, or arising from the use of the INFORMATION or any      */
/*	part of it.  							      */
/*									      */
/*	AT&T further does not grant any license, express or implied, under    */
/*	any patent, copyright or other intellectual property right of AT&T    */
/*	covering or relating to any process, machine or combination in which  */
/*	the INFORMATION and/or semiconductor device or devices may be or are  */
/*	used.								      */
/*									      */
/*	The software code and other information contained in this APPLICA-    */
/*	TION NOTE are copyrighted and all rights are reserved by AT&T.  The   */
/*	software code is intended for use only in an AT&T Digital Signal      */
/*	Processor.  No other use is authorized.				      */
/*									      */
/******************************************************************************/
.global _iirstvr,_iirincr,_ttdigit,inbuf,bufio

	dauc=0x0
	ioc=0x40		/* 8-bit input, passive clks, DMA on IBF */
	r15=12
	r16=4
begin:	if (syc) goto begin		/* wait for sync to go high before */
	nop				/* beginning DMA		   */
	ioc=0x2040		/* 8-bit input, passive clks, DMA on IBF */
	pin=inbuf
	r17=-60
	r5=102-2			/* DFT frame  size - 2 */
	r6=energy
	r10=_iirzero
	r7=16-2				/* 16 receivers - 2 */
clrnrg:	if (r7-- >= 0) goto clrnrg	/* zero the energy locations on a */
	*r6++ = a0 = *r10		/* frame by frame basis. */
	r7=32-2
	r2=_iirco			/* pointer to coefficients */
	r3=_iirstvr			/* pointer to state variables */
clear:	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	*r3++ = a0 = *r10
	if (r7-- >= 0) goto clear	/* zero the state variables on a */
	*r3++ = a0 = *r10		/* frame by frame basis.	 */
/*****************************************************************************/
/*									     */
/*	This section sets up the DMA for each receiver.  After 16 samples    */
/*	( one for each receiver ) have been received, they are converted to  */
/*	floating point representation and stored at bufio.  DMA is then      */
/*	re-initialized to continue receiving data.			     */
/*									     */
/*****************************************************************************/
sample: pin-ilimit
iwait:	if (le) goto iwait		/* wait for input data to be gathered */
	pin-ilimit
	pin=inbuf
	r1=inbuf
	r4=bufio
	r6=energy
	r3=14				/* 16 receivers -2 */
ioxfr:	if (r3-- >= 0) goto ioxfr
	*r4++ = a1 = ic(*r1++r16)	/* move data from inbuf to bufio */
	r11=14				/* 16 receivers - 2 */
/****************************************************************************/
/*									    */
/*	This section computes the LHS of the DFT for the 1st and 2nd order  */
/*	harmonics of each tone for each sample.	 It is performed 16 times,  */
/*	once for each receiver.	The state variables for each receiver are   */
/*	stored in a sequential stack on top of each other.		    */
/*									    */
/****************************************************************************/
	r1=bufio
	r3=_iirstvr		/* pointer to state variables */
	r4=r3+4			/* stores updated state variables */
dft:	a1=*r1
/* 697hz    1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++    	/* 2nd state var. d[1,1] */
	*r4++r15 = a0 = a0 - *r3++		/* 1st state var. d[1,2] */
/* 770hz    1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 852hz    1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 941hz    1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1209hz   1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1336hz   1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1477hz   1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1633hz   1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 697hz    2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 770hz    2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 852hz    2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 941hz    2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1209hz   2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1336hz   2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1477hz   2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1633hz   2nd harmonic   LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++r17		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
	if (r11-- >= 0) goto dft		/* 16 receiver counter */
	*r6++ = a2 = *r6 + a1 * *r1++		/* update energy */
	if (r5-- >= 0) goto sample		/* 205 sample frame counter */
	nop
/****************************************************************************/
/*									    */
/*	This section computes the RHS of the DFT for the 1st harmonics 	    */
/*	of each tone.  The magnitude is squared and stored in consecutive   */
/*	locations starting at _iirstvr.					    */
/*									    */
/****************************************************************************/

	ioc=0x40
	r16=0
	r12=energy
	r18=_iirstvr
	r19=_ttdigit
	r11=14			/* 16 receivers - 2 */
timing:	r2=_iirco
	r4=r18
	r15=8
	r1=6
	r3=r18			/* points to location of mag^2 of each tone */
right:	a2= *r4 * *r4++			/* Sn * Sn-1	 */
	a1=*r4-- * *r4			/* Sn-1^2  	 */
	a0 = a1 + *r4++r15 * *r4	/* Sn^2 + Sn-1^2 */
	if (r1-- >= 0) goto right
	*r3++ = a0 = a0 - a2 * *r2++	/* |Y|^2	 */

/****************************************************************************/
/*									    */
/*	This section finds the largest value in the low and high frequency  */
/*	band and stores the 2 values at _iirstvr+32. It also computes their */
/*	location in the low and high frequency band and stores these 2      */
/*	values at _iirincr.						    */
/*									    */
/****************************************************************************/

	r8=_iirincr
	r4=r18+32			/* loc. of largest mag. in each band */
	r3=r18				/* loc. of mag^2 of each tone */
	r7=0
lrgmag3:r6=1
	r5=0
	r1=0
	r2=0
	*r4=a0=*r3++		/* writes possible large mag. temporarily *r4 */
lrgmag4:a1=*r3++
	nop
	a2=a1-a0
	3*nop
	if (ale) goto lrgmag1		/* test which value is larger */
	nop
	*r4=a0=a1
	r2=r2+1
	r1=r2
	goto lrgmag2
	r1=r1+r5			/* contains the row & col increments */
lrgmag1:r5=r5+1
lrgmag2:if (r6-- >= 0) goto lrgmag4
	nop
	*r8++=r1			/* contains the row & col increments */
	if (r7-- >= 0) goto lrgmag3
	r4=r4+4

/****************************************************************************/
/*                                                                          */
/*	This section compares the lower of the high and low frequency	    */
/*	band magnitudes to a threshold level.  If it is lower than the	    */
/*	threshold the program updates the valid digit stack with a -1       */
/*	and retrieves another frame.  Otherwise, it continues and checks    */
/*	the twist.							    */
/*									    */
/****************************************************************************/

	r1=r18+32			/* loc. of max row & col mag's */
	a0= *r1 - *r1++			/* high mag. - low mag. */
	r10=_ttrlevl
	a2 = *r1
	a1 = *r1-- - *r10		/* lower value - 8e7 */
	if (alt) goto _ttrlow
	r2=_twisthg
_ttrhigh:a2 = *r1
	a1 = *r1++ - *r10		/* lower value - 8e7 */
	r2=_twistlw
	a1 = -*r1 + a2 * *r2		/* low mag. * 2.5 - high mag. */
	nop
	if (alt) goto notone		/* threshold check	*/
	nop
	if (alt) goto notone		/* reverse twist test */
	nop
	goto frqofset			/* continue	*/
	nop
_ttrlow:a1 = -*r1 + a2 * *r2		/* high mag. * 9.2 - low mag. */
	if (alt) goto notone		/* threshold check */
	2*nop
	if (alt) goto notone		/* normal twist test */
	nop
/********************************************************************************/
/*										*/
/*	Do largest frequencies stand out?					*/
/*	This section compares the largest magnitude (detected "tone") in each	*/
/*	frequency band with the other magnitudes in that band. If the "tone" is */
/*	not significantly larger then the other magnitudes the program returns  */
/*	to begin and samples another frame.					*/
/*										*/
/********************************************************************************/
frqofset: r4=0
	r10=_vldtonl
	r1=r18+32			/* loc. of largest mag. in each band */
	r2=r18				/* loc. of mag^2 of each tone */
freqoff2:r3=2
freqoff1:a2 = *r2
	a1 = *r1 - *r2++		/* check if mag's are the same */
	nop
	a0 = *r1 - a2 * *r10		/* highest mag. - other mags. * coeff */
	nop
	if (aeq) goto freqoff		/* if equal mag's get next mag */
	nop
	if (age) goto freqoff		/* if highest mag < other mag * coeff */
	nop				/* tone is not valid		      */
	goto notone			/* Therefore, update _ttdigit with -1 */
	nop				/* and get another frame of samples.  */
freqoff:if (r3-- >= 0) goto freqoff1
	nop
	r10=_vldtonh
	if (r4-- >= 0) goto freqoff2
	r1=r1+4
/******************************************************************************/
/*									      */
/*	This section compares the total inband energy to the largest mag. in  */
/*	the low band, the largest mag. in the high band, and the sum of the   */
/*	two magnitudes.							      */
/*									      */
/******************************************************************************/

energ:	r1=r18+32
	a2=*r1++			/* a2=largest mag in low band */
	a1=*r1				/* a1=largest mag in high band */
	r3=_nrgcoef
	a3 = a2 - *r12 * *r3++		/* a3=low band mag-energy*lowband coef */
	a2=a1+a2
	a3 = a1 - *r12 * *r3++		/* a3=highband mag-energy*highband coef*/
	a3 = a2 - *r12++ * *r3		/* a3=sum of mags-energy*sum coef */
	if (alt) goto notone
	nop
	if (alt) goto notone
	nop
	if (alt) goto notone
	nop
/********************************************************************************/
/*										*/
/*	Is the harmonic ratio > threshold?				 	*/
/*	This computes the magnitudes squared of the "tones" second harmonics    */
/*	and compares it with the "tone". If the magnitudes are within a certain */
/*	ratio, the program returns to begin and samples another frame.		*/
/*										*/
/********************************************************************************/
	r3=r18+32			/* loc. of largest mag. in each band */
	r1=0
	r5=0
	r10=_iirincr
	r7=_hrmcoef
_2ndharm: r6=*r10++			/* compute the location of the 2nd harm */
	r4=r18				/* coefficients and state variables     */
	r8=r6*2
	r8=r8*2
	r2=r8+32
	r2=r2+r5
	r8=r2*2
	r2=r2+_iirco			/* loc. of the 2nd harmonic coeff's */
	r4=r4+r8
	a2= *r4 * *r4++			/* Sn * Sn-1	 */
	a1=*r4-- * *r4			/* Sn-1^2  	 */
	a0 = a1 + *r4 * *r4		/* Sn^2 + Sn-1^2 */
	a0 = a0 - a2 * *r2		/* |Y|^2	 */
	2*nop
	a3 = *r3++ - a0 * *r7++	     /* 1st harm - 2nd harm * 2nd harm coeff */
	3*nop
	if (ale) goto notone
	r5=16
	if (r1-- >= 0) goto _2ndharm
	r9=_iirincr			/* used in decode digit section */

/********************************************************************************/
/*										*/
/*	Decode digits								*/
/*	pdr's lower byte contains the valid decoded digit in its lower 4 bits   */
/*	and the receiver number in its upper 4 bits.			  	*/
/*									    	*/
/********************************************************************************/

	r1=*r9++		/* location of _iirincr */
	r7=r16*2
	r2=r1*2
	r2=r2*2
	r3=*r9
	r2=r2*2
	r3=r3*2
	r2=r2+r3
	r2=r2+_romtbl		/* loc. of decoded digit */
	r10=*r2			/* present digit D(k) */
	r4=r19+2
	r5=*r4			/* 2nd to last digit D(k-2) */
	r7=r7*2
	r5=r5-r10
	if (eq) goto tone1	/* D(k)=D(k-2) ? */
	r4=r19
	r5=*r4			/* last digit decoded D(k-1) */
	r7=r7*2
	r5=r5-r10
	if (ne) goto tone1	/* D(k)=D(k-1) ? */
	r8=r7*2
	r8=r8^r10
	pdr=r8
	goto tone1
	nop
notone:	r10=-1			/* set digit to -1 and update stack */
tone1:	r1=r19
	r3=*r1
	*r1++ = r10		/* update D(k-1)=D(k) */
	*r1 = r3		/* update D(k-2)=D(k-1) */
	r19=r19+4
	r16=r16+1
	if (r11-- >= 0) goto timing	/* process timing for next receiver */
	r18=r18+128
	goto begin
	r15=12


/********************************************************************************/
/*										*/
/*				PROGRAM VARIABLES				*/
/*										*/
/*  First order harmonic coefficients						*/
_iirco: float	1.70773781, 1.645281, 1.568687, 1.4782
	float   1.1641, .99637, .7986184, .5685327
/*  Second order harmonic coefficients						*/
	float	.916368, .70695, .46077885, .1851
	float 	-.644862, -1.0072464, -1.36221, -1.676771
_iirzero: float 0.0
_ttrlevl: float 4e7
_twisthg: float 10.5
_twistlw: float 7.4
_vldtonl: float 5.0
_vldtonh: float 5.0
_nrgcoef: float 7.0, 5.6, 36.0
_hrmcoef: float 1.6, 60.0
_romtbl:  int 1, 2, 3, 10
	  int 4, 5, 6, 11
	  int 7, 8, 9, 12
	  int 14, 0, 15, 13
/*										*/
/********************************************************************************/
/********************************************************************************/
/*										*/
/*				STATE VARIABLES					*/
/*										*/
inbuf:	15*float 0.0
ilimit:	  float 0.0
dummy:	100*float 0.
bufio:	  16*float 0.0
_iirincr: 2*int 0
energy:	16*float 0.0
_ttdigit: 32*int 0
.rsect ".hi_ram"
_iirstvr: 512*float 0.0			/* state variables */
/*										*/
/********************************************************************************/
