/****************************************************************/
/*   INTERFACING THE WE(R) DSP16 DIGITAL SIGNAL PROCESSOR	*/
/*   TO MULTIPLE A/D AND D/A CONVERTERS				*/
/*   November, 1989  Doc# AP89-015DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1989 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*			pass4.s					*/
/****************************************************************/
/* Using interrupt driven I/O, read a sample from each A/D and	*/
/* write the sample to the corresponding D/A.  The program	*/
/* writes to the address port, reads the data port (A/D) twice	*/
/* (to compensate for latency) then writes the data port (D/A). */
/*								*/
/* This example resolves latent reads by reading the port twice.*/

#define  ADDR_PORT	pdx0
#define  DATA_PORT	pdx1
#define  PORT_0		0x001		/* bit A0 */
#define  PORT_1		0x002		/* bit A1 */
#define  PORT_2		0x004		/* bit A2 */
#define  PORT_3		0x008		/* bit A3 */
#define  NOP		a0=a1		/* used to separate PIO read/writes */

rst:	goto start			/* jump to main program		*/

intdriver: ADDR_PORT = PORT_0		/* select channel 0		*/
	NOP
	a1 = DATA_PORT			/* a1 = dummy, D0 in buffer	*/
	NOP
	a1 = DATA_PORT			/* a1 = D0, D0 in buffer	*/
	NOP
	DATA_PORT = a1			/* write D0			*/

	ADDR_PORT = PORT_1		/* select channel 1		*/
	NOP
	a1 = DATA_PORT			/* a1 = D0, D1 in buffer	*/
	NOP
	a1 = DATA_PORT			/* a1 = D1, D0 in buffer	*/
	NOP
	DATA_PORT = a1			/* write D1			*/

	ADDR_PORT = PORT_2		/* select channel 2		*/
	NOP
	a1 = DATA_PORT			/* a1 = D1, D2 in buffer	*/
	NOP
	a1 = DATA_PORT			/* a1 = D2, D2 in buffer	*/
	NOP
	DATA_PORT = a1			/* write D2			*/

	ADDR_PORT = PORT_3		/* select channel 3		*/
	NOP
	a1 = DATA_PORT			/* a1 = D2, D3 in buffer	*/
	NOP
	a1 = DATA_PORT			/* a1 = D3, D3 in buffer	*/
	NOP
	DATA_PORT = a1			/* write D3			*/

	ADDR_PORT = PORT_0		/* select channel		*/

	ireturn

start:	pioc = 0x3820		/* pids, pods active, int active	*/
	auc = 0x00

noploop: NOP
	goto noploop

