****************************************************************************** 
****************************************************************************** 
 
This file contains the modules for a 256 point radix 4 FFT.  The modules  
included are: 
 
     1)   The architecture description file  IN2101.SYS 
     2)   A main calling shell  M4N256.DSP 
     3)   The actual FFT routines  F4N256.DSP 
     4)   The coefficient data file   COS256.DAT 
 
These files have been concantenated into 1 large file for the purpose of 
the bulletin board file transfer.  These modules will need to be separated 
before trying to assemble and link with the ADSP-2100 family cross software  
tools. 
 
If you have any questions or comments, feel free to contact Analog Devices 
DSP Applications Staff at (617) 461-3672.   Field Applications Offices are 
also located in Belgium and California. 
 
****************************************************************************** 
 
The following information is the contents of the architecture description file 
This file is referenced as  IN2101.SYS 
 
 
{ADSP-2101 boot memory mapped program memory} 
 
.system				ex2101; 
.adsp2101; 
.mmap0; 
 
{_______Program Memory Section_______} 
.seg/rom/boot=0			 boot_mem[h#0800]; 
.seg/ram/abs=h#0000/pm/code/data PM_DATA[h#0800]; 
 
{_______Data Memory Section_______} 
.seg/ram/abs=0/dm/data		DM_DATA[15360]; 
.endsys; 
 
 
******************************************************************************* 
 
 
 {_______________________________________________________________________ 
                            m4n256.dsp                                    
 
 Calling shell for the 256_pt FFT radix_4 with built_in digit_reversal 
 
        *F4N256.dsp ---->FFT routine 
 
_________________________________________________________________________} 
 
.MODULE/RAM/boot=0/ABS=0        main; 
.CONST          N=256; 
.CONST          Nx2=512; 
.VAR/DM         inplace[Nx2];           {inplace array contains original input} 
                                        {and also hold intermediate results} 
.VAR/DM/abs=0   output[Nx2];            
.INIT           inplace:<input.dat>;    {load inplace array  with data} 
.GLOBAL         inplace,output; 
.EXTERNAL       fft;               {the 2 routines used to perfortm FFT} 
 
 
 
                JUMP start;             {this code is 2100/2101 compatible} 
                RTI;                    {the 2101 initial pc value is 0,} 
                RTI;                    {2100, however, has pc=4 as start add.} 
                RTI; 
start:          SI=0;                   {these 3 lines reset the system } 
                DM(0X3FFF)=SI;          {control register and the data } 
                DM(0X3FFE)=SI;          {memory control register to allow} 
                                        {zero state memory access} 
                CALL fft; 
 
TRAPPER:        JUMP TRAPPER;                                                  
.endmod; 
 
 
 
******************************************************************************** 
 
 
 
 
{_____________________________________________________________________ 
                         F4N256.DSP 
 
        Optimized complex 256_Point Radix-4 DIF FFT Program for the 
        Analog Devices ADSP-210X Digital Signal Processor Family 
 
        This program uses in_order input data and produces in_order 
        output data without using an additional unscrambling routines. 
 
        Complex data stored X0,Y0, X1, Y1,... (real, imag, real, imag) 
 
        The following terminology will be used to describe 
        the  "legs" of the radix-4 butterfly: 
        xa = 1st real input leg        x'a = 1st real output leg 
        xb = 2nd real input leg        x'b = 2nd real output leg 
        xc = 3rd real input leg        x'c = 3rd real output leg  
        xd = 4th real input leg        x'd = 4th real output leg 
        ya = 1st imag input leg        y'a = 1st imag output leg 
        yb = 2nd imag input leg        y'b = 2nd imag output leg 
        yc = 3rd imag input leg        y'c = 3rd imag output leg 
        yd = 4th imag input leg        y'd = 4th imag output leg 
 
        The index registers of the 210X will be used as pointers 
        to the following: 
        I0 --> xa,xc 
        I1 --> xb,xd I2 --> ya,yc I3 --> yb,yd                 
               w0 (= Ca = Sa = 0) 
        I5 --> w1 (1st Cb, - pi/4 for Sc) 
        I6 --> w2 (2nd Cc, - pi/4 for Sc) 
        I7 --> w3 (3rd Cd, - pi/4 for Sd) 
 
        input: inplace[2*N] normal order, interleaved real,imag. 
        output: inplace[2*N] digit reversed, interleaved real,imag. 
 
        Benchmark: 
          setup  =     9 
          stage 1=     1940 = 20+64(30) 
          stage 2=     1997 = 17+4(15+16(30)) 
          stage 3=     2177 = 17+16(15+4(30)) 
          stage 4=     1300 = 20+64(20) 
 
          Total        7423 cycles * 80ns = 0.59384ms 
 
_____________________________________________________________________} 
 
.MODULE/boot=0   fft_sub; 
.CONST           N=256; 
.CONST           Nx2=512; 
.CONST           Nov2=128; 
.CONST           Nov4=64; 
.CONST           N3ov8=96; 
.VAR/DM/CIRC     cos_table[N]; 
 
.VAR/DM          M3_space;      {memory space used to store M3 values} 
                                {when M3  is loaded with its alternate  
                                 value} 
 
.VAR/DM          bfy_count; 
.INIT            cos_table:<cos256.dat>;        {N cosine values} 
.ENTRY           fft; 
.EXTERNAL        inplace; 
 
fft:             M4=-Nov4;          {-N/4 = -90 degrees for sine} 
     L0=0; 
     L1=0; 
     L2=0; 
     L3=0; 
     L5=%cos_table; 
     L6=%cos_table; 
     L7=%cos_table; 
     SE=0; 
 
{_____________________________Stage 1_______________________________} 
 
 
 
stage1: I0=^inplace;                   {in       ->Xa,Xc} 
        I1=^inplace+Nov2;              {in+N/2   ->Xb,Xd} 
        I2=^inplace+1;                 {in+1     ->Ya,Yc} 
        I3=^inplace+Nov2+1;            {in+N/2+1 ->Yb,Yd} 
        I5=^cos_table; 
        I6=^cos_table; 
        I7=^cos_table; 
        M0=N;               {N,    skip forward to dual node} 
        M1=-N;              {-N,   skip back to primary node} 
        M2=-N+2;            {-N+2, skip to next butterfly} 
 
        M3=-2;              {because we have modified the middle branches} 
                            {of bfly, pointers for I0 have to be treated a} 
                            {little more carefully.  This requires a more} 
                            {complex pointer manipulation.  M3 is used in } 
                            {doing this} 
 
        M5=Nov4+1;          {N/4 + groups/stage*1, Cb Sb offset} 
 
        M6=Nov4+2;          {N/4 + groups/stage*2, Cc Sc offset} 
        M7=Nov4+3;          {N/4 + groups/stage*3, Cd Sc offset} 
        AX0=DM(I0,M0);                        {get first Xa} 
        AY0=DM(I0,M1);                        {get first Xc} 
        AR=AX0-AY0, AX1=DM(I2,M0);            {Xa-Xc,get first Ya} 
        SR=LSHIFT AR(LO), AY1=DM(I2,M1);      {SR1=Xa-Xc,get first Yc} 
        CNTR=Nov4;                            {Bfly/group, stage one} 
 
 
 
  {This radix_4 bfly algorithm is a modified one.  It has it's middle 2 } 
  {Branches reversed.  This alteration, done to every stage, results in } 
  {a bit_reversed output instead of a digit reversed output             } 
 
 
 
        DO stg1bfy  UNTIL CE; 
                 AR=AX0+AY0, AX0=DM(I1,M0); 
                           {AR=xa+xc, AX0=xb } 
                 MR0=AR, AR=AX1+AY1; 
                           {MR0=xa+xc, AR=ya+yc} 
                 MR1=AR, AR=AX1-AY1; 
                           {MR1=ya+yc, AR=ya-yc} 
                 SR=SR OR LSHIFT AR (HI), AY0=DM(I1,M1); 
                           {SR1=ya-yc, AY0=xd} 
                 AF=AX0+AY0, AX1=DM(I3,M0); 
                           {AF=xb+xd, AX1=yb} 
                 AR=MR0+AF, AY1=DM(I3,M1); 
                           {AR=xa+xb+xc+xd, AY1=yd} 
                 DM(I0,M0)=AR, AR=MR0-AF; 
                           {output x'a=(xa+xb+xc+xd), AR=xa+xc-xb-xd} 
                 AF=AX1+AY1, MX0=AR; 
                           {AF=yb+yd, MX0=xa+xc-xb-xd} 
                 AR=MR1+AF, MY0=DM(I6,M4); 
                           {AR=ya+yc+yb+yd, MY0=(Cc)} 
                 DM(I2,M0)=AR, AR=MR1-AF; 
                           {output y'a, AR=ya+yc-yb-yd} 
                 MR=MX0*MY0(SS), MY1=DM(I6,M6); 
                           {MR=(xa+xc-xb-xd)(Cc), MY1=(Sc)} 
 
                 MR=MR+AR*MY1(RND),SI=DM(I0,M2);     
                           {MR=(xa-xb+xc-xd)(Cc)+(ya-yb+yc-yd)(Sc)} 
                           {si here is a dummy to perform a modify(I0,M2)} 
                 DM(I1,M0)=MR1, MR=AR*MY0(SS); 
                           {output x'c to  positionx'b} 
                           {MR=(ya+yc-yb-yd)(Cc)} 
                 MR=MR-MX0*MY1(RND), MY0=DM(I5,M4); 
                           {MR=(ya+yc-yb-yd)(Cc)-(xa+xc-xb-xd)(Sc), MY0=(Cb)} 
                 DM(I3,M0)=MR1, AR=AX0-AY0; 
                           {output y'c=to position y'b} 
                           {AR=xb-xd} 
                 AY0=AR, AF=AX1-AY1; 
                           {AY0=xb-xd, AF=yb-yd} 
                 AR=SR0-AF, MY1=DM(I5,M5); 
                           {AR=xa-xc-(yb-yd), MY1=(Sb)} 
                 MX0=AR, AR=SR0+AF; 
                           {MX0=xa-xc-yb+yd, AR=xa-xc+yb-yd} 
                 SR0=AR, AR=SR1+AY0; 
                           {SR0=xa-xc+yb-yd, AR=ya-yc+xb-xd} 
                 MX1=AR, AR=SR1-AY0; 
                           {MX1=ya-yc+xb-xd, AR=ya-yc-(xb-xd)} 
                 MR=SR0*MY0(SS),AX0=DM(I0,M0); 
                           {MR=(xa-xc+yb-yd)(Cb), AX0= xa of next bfly} 
                 MR=MR+AR*MY1(RND),AY0=DM(I0,M3); 
                           {MR=(xa-xc+yb-yd)(Cb)+ (ya-yc-xb+xd)(Sb)} 
                           {AY0= xc of next bfly} 
                 DM(I0,M2)=MR1, MR=AR*MY0(SS); 
                           {output x'b=to position x'c} 
                           {MR=ya-yc-xb+xd)(Cb)} 
                 MR=MR-SR0*MY1(RND), MY0=DM(I7,M4); 
                           {MR=(ya-yc-xb+xd)(Cb)-(xa-xc+yb-yd)(Sb)} 
                           {MY0=(Cd)} 
                 DM(I2,M2)=MR1, MR=MX0*MY0(SS); 
                           {ouput y'b= to position y'c} 
                           {MR=(xa-yb-xc+yd)(Cd)} 
                 MY1=DM(I7,M7), AR=AX0-AY0; 
                            {MY1=(Sd), AR=xa-xc }   
                 MR=MR+MX1*MY1(RND), AX1=DM(I2,M0); 
                           {MR=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)} 
                           {AX1=ya of next bfly} 
                 DM(I1,M2)=MR1, MR=MX1*MY0(SS); 
                           {output x'd=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)} 
                           {MR=(ya+yb-yc-yd)(Cd)} 
                 MR=MR-MX0*MY1(RND), AY1=DM(I2,M1); 
                           {MR=(ya+yb-yc-yd)(Cd)-(xa-xc-yb+yd)Sd} 
                           {yc of next bfly} 
stg1bfy:         DM(I3,M2)=MR1, SR=LSHIFT AR(LO); 
                           {output y'd=(ya+xb-yc-xd)Cd-(xa-xc-yb+yd)Sd} 
                           {SR0=ya-yc of next bfly} 
 
 
 
{_____________________________Stage 2_______________________________} 
 
 
 
stage2:     I0=^inplace;       {in       -> Xa,Xc} 
            I1=^inplace+32;    {in+N/8   -> Xb,Xd} 
            I2=^inplace+1;     {in+1     -> Ya,Yc} 
            I3=^inplace+33;    {in+N/8+1 -> Yb,Yd} 
            M0=Nov4;           {N/4,    skip forward to dual node} 
            M1=-Nov4;          {-N/4,   skip back to primary node} 
            M2=-Nov4+2;        {-N/4+2, skip to next butterfly} 
             
            M3=96;                      {N*3/8,  skip to next group} 
            DM(M3_space)=M3;            {M3_space is  temporary storage} 
                                        {space needed because M3 isused} 
                                        {in 2 contexts and will alternate} 
                                        {in value} 
            
            M5=Nov4+4;         {N/4 +groups/stage*1, Cb Sb offset} 
            M6=Nov4+8;         {N/4 +groups/stage*2, Cc Sc offset} 
            M7=Nov4+12;        {N/4 +groups/stage*3, Cd Sd offset} 
            SI=16;             {Bfy/group, save counter for inner loop}  
            DM(bfy_count)=SI;  {SI is used as a temporary storage dummy} 
 
            CNTR=4;            {groups/stage} 
            CALL mid_stg;      {do stage 2} 
 
{_____________________________Stage 3_______________________________} 
 
stage3:     I0=^inplace;        {in         -> Xa,Xc} 
            I1=^inplace+8;      {in+N/32    -> Xb,Xd} 
            I2=^inplace+1;      {in+1       -> Ya,Yc} 
            I3=^inplace+9;      {in+N/32+1  -> Yb,Yd} 
            M0=16;              {N/16,    skip forward to dual node} 
            M1=-16;             {-N/16,   skip back to primary node} 
            M2=-14;             {-N/16+2, skip to next butterfly} 
  
            M3=24;                      {N*3/32,  skip to next group} 
            DM(M3_space)=M3;            {M3_space is  temporary storage} 
                                        {space needed because M3 isused} 
                                        {in 2 contexts and will alternate} 
                                        {in value} 
            
            M5=Nov4+16;         {N/4 +groups/stage*1, Cb Sb offset} 
            M6=Nov4+32;         {N/4 +groups/stage*2, Cc Sc offset} 
            M7=Nov4+48;         {N/4 +groups/stage*3, Cd Sd offset} 
            SI=4;              {Bfy/group, save counter for inner loop}  
            DM(bfy_count)=SI;   {SI is a dummy temporary register here } 
  
            CNTR=16;            {groups/stage} 
            CALL mid_stg;       {do stage 3} 
 
{_________________Last Stage__No Multiplies__________________________} 
 
 
                             
laststage:  I4=^inplace;       {in         ->Xa,Xc} 
            I5=^inplace+2;     {in+N/512   ->Xb,Xd} 
            I6=^inplace+1;     {in+1       ->Ya,Yc} 
            I7=^inplace+3;     {in+N/512+1 ->Yb,Yd} 
            M4=4;              {N/256,  skip forward to dual node} 
 
            M0=h#0040;          {this magic modify value is used to  } 
                                {perform a bit reverse as the final results} 
                                { are written out.  Because of interleave} 
                                {this is not as simple as explained in } 
                                {1410 Handbook.  Derivation of these values} 
                                {is in the Apps. note of this code} 
 
            I0=h#0000;          {these  base address values are derived } 
            I2=h#2000;          {for an output at address 0000.  Output } 
            I1=h#0020;          {address can be modified by modifying m0,} 
            I3=h#2020;          {and I0..I3} 
 
            L4=0;               {this last stage has no twiddles and} 
            L5=0;               {because it is where the DAG#1 bit_} 
            L6=0;               {reverses the output, the I's M's & L's} 
            L7=0;               {have been reassigned and re_initialized} 
 
 
 
 
 
            AX0=DM(I4,M4);      {first Xa}  
            AY0=DM(I4,M4);      {first Xc} 
            CNTR=Nov4;          {groups/stage} 
            ENA BIT_REV;        {all data accesses using I0..I3 are} 
                                {bit-reversed  } 
 
 
  {This radix_4 bfly algorithm is a modified one.  It has it's middle 2 } 
  {Branches reversed.This alteration, done to every stage, results in } 
  {a bit_reversed output instead of a digit reversed output. } 
 
 
 
            DO laststgbfy UNTIL CE; 
                 AR=AX0-AY0, AX1=DM(I6,M4);       {AR=xa-xc, AX1=ya} 
                 SR=LSHIFT AR(LO), AY1=DM(I6,M4); {SR0=xa-xc, AY1=yc} 
                 AR=AX0+AY0, AX0=DM(I5,M4);       {AR=xa+xc, AX0=xb} 
                 MR0=AR, AR=AX1+AY1;              {MR0=xa+xc, AR=ya+yc} 
                 MR1=AR, AR=AX1-AY1;              {MR1=ya+yc, AR=ya-yc} 
                 SR=SR OR LSHIFT AR (HI), AY0=DM(I5,M4); 
                                                  {SR1=ya-yc, AY0 xd} 
                 AF=AX0+AY0, AX1=DM(I7,M4);       {AF=xb+xd, AX1=yb} 
                 AR=MR0+AF, AY1=DM(I7,M4);        {AR=xa+xc+xb+xd, AY1=yd} 
                 DM(I0,M0)=AR, AR=MR0-AF;         {output x'a=xa+xc+xb+xd} 
                                                  {AR=xa+xc-(xb+xd)} 
                 DM(I1,M0)=AR, AF=AX1+AY1;        {output x'c to position x'b} 
                                                  {AF=yb+yd} 
                 AR=MR1+AF;                       {AR=ya+yb+yc+yd} 
                 DM(I2,M0)=AR, AR=MR1-AF;         {output y'a=ya+yc+yb+yd} 
                                                  {AR=ya+yc-(yb+yd)} 
                 DM(I3,M0)=AR, AR=AX0-AY0;        {output y'c to position y'b} 
                                                  {AR=xb-xd} 
                 AX0=DM(I4,M4);                   {AX0=xa of next group} 
                 AF=AX1-AY1, AY1=AR;              {AF=yb-yd, AY1=xb-xd} 
                 AR=SR0+AF, AY0=DM(I4,M4);        {AR=xa-xc+yb-yd} 
                                                  {AY0=xc of next group} 
                 DM(I0,M0)=AR, AR=SR0-AF;         {output x'b to position x'c} 
                                                  {AR=xa-xc-(yb-yd)} 
                 DM(I1,M0)=AR, AR=SR1-AY1;        {output x'd=xa-xc-(yb-yd)} 
                                                  {AR=ya-yc+(xb-xd)} 
                 DM(I2,M0)=AR, AR=SR1+AY1;        {output y'b to position y'c} 
                                                  {AR=ya-yc-(xb-xd)} 
laststgbfy:      DM(I3,M0)=AR;                    {output y'd=ya-yc-(xb-xd)} 
 
                 DIS  BIT_REV;                    {shut-off bit reverse} 
                                                  {mode, not necessary} 
                 RTS;             {end and exit from FFT subroutine}  
 
 
 
{_____________________Subroutine for middle stages___________________} 
 
 
mid_stg:  DO midgrp UNTIL CE; 
              I5=^cos_table; 
              I6=^cos_table; 
              I7=^cos_table; 
              AX0=DM(I0,M0);                        {get first Xa} 
              AY0=DM(I0,M1);                        {get first Xc} 
              AR=AX0-AY0, AX1=DM(I2,M0);            {Xa-Xc,get first Ya} 
              SR=LSHIFT AR (LO), AY1=DM(I2,M1);     {SR1=Xa-Xc,get first Yc} 
              CNTR=DM(bfy_count);                   {butterflies/group} 
 
 
           M3=-2;               {M3 is loaded with the special value} 
                                {required for pointer manipulation } 
                                {as explained previously} 
 
 
 
  {This radix_4 bfly algorithm is a modified one.  It has it's middle 2  } 
  {Branches reversed.  This alteration, done to every stage, resdults in } 
  {a bit reversed output instead of a digit reversed output              } 
 
              DO midbfy UNTIL CE; 
                 AR=AX0+AY0, AX0=DM(I1,M0); 
                           {AR=xa+xc, AX0=xb } 
                 MR0=AR, AR=AX1+AY1; 
                           {MR0=xa+xc, AR=ya+yc} 
                 MR1=AR, AR=AX1-AY1; 
                           {MR1=ya+yc, AR=ya-yc} 
                 SR=SR OR LSHIFT AR (HI), AY0=DM(I1,M1); 
                           {SR1=ya-yc, AY0=xd} 
                 AF=AX0+AY0, AX1=DM(I3,M0); 
                           {AF=xb+xd, AX1=yb} 
                 AR=MR0+AF, AY1=DM(I3,M1); 
                           {AR=xa+xb+xc+xd, AY1=yd} 
                 DM(I0,M0)=AR, AR=MR0-AF; 
                           {output x'a=(xa+xb+xc+xd), AR=xa+xc-xb-xd} 
                 AF=AX1+AY1, MX0=AR; 
                           {AF=yb+yd, MX0=xa+xc-xb-xd} 
                 AR=MR1+AF, MY0=DM(I6,M4); 
                            {AR=ya+yc+yb+yd, MY0=(Cc)} 
                 DM(I2,M0)=AR, AR=MR1-AF; 
                           {output y'a, AR=ya+yc-yb-yd} 
                 MR=MX0*MY0(SS), MY1=DM(I6,M6); 
                           {MR=(xa+xc-xb-xd)(Cc), MY1=(Sc)} 
 
                 MR=MR+AR*MY1(RND), SI=DM(I0,M2); 
                           {MR=(xa-xb+xc-xd)(Cc)+(ya-yb+yc-yd)(Sc)} 
                           {SI is a dummy here to cause a modify(I0,M2)}    
                 DM(I1,M0)=MR1, MR=AR*MY0(SS); 
                           {output x'c to position x'b} 
                           {MR=(ya+yc-yb-yd)(Cc)} 
                 MR=MR-MX0*MY1(RND), MY0=DM(I5,M4); 
                           {MR=(ya+yc-yb-yd)(Cc)-(xa+xc-xb-xd)(Sc), MY0=(Cb)} 
                 DM(I3,M0)=MR1, AR=AX0-AY0; 
                           {output y'c to position y'b} 
                           {AR=xb-xd} 
                 AY0=AR, AF=AX1-AY1; 
                           {AY0=xb-xd, AF=yb-yd} 
                 AR=SR0-AF, MY1=DM(I5,M5); 
                           {AR=xa-xc-(yb-yd), MY1=(Sb)} 
                 MX0=AR, AR=SR0+AF; 
                           {MX0=xa-xc-yb+yd, AR=xa-xc+yb-yd} 
                 SR0=AR, AR=SR1+AY0; 
                           {SR0=xa-xc+yb-yd, AR=ya-yc+xb-xd} 
                 MX1=AR, AR=SR1-AY0; 
                           {MX1=ya-yc+xb-xd, AR=ya-yc-(xb-xd)} 
             
                 MR=SR0*MY0(SS),  AX0=DM(I0,M0); 
                           {MR=(xa-xc+yb-yd)(Cb), AX0= xa of next bfly} 
                 MR=MR+AR*MY1(RND),  AY0=DM(I0,M3); 
                           {MR=(xa-xc+yb-yd)(Cb)+ (ya-yc-xb+xd)(Sb)} 
                           {AY0= xc of next bfly} 
 
                 DM(I0,M2)=MR1, MR=AR*MY0(SS); 
                           {ouput x'b to position x'c} 
                           {MR=ya-yc-xb+xd)(Cb)} 
                 MR=MR-SR0*MY1(RND), MY0=DM(I7,M4); 
                           {MR=(ya-yc-xb+xd)(Cb)-(xa-xc+yb-yd)(Sb)} 
                           {MY0=(Cd)} 
                 DM(I2,M2)=MR1, MR=MX0*MY0(SS); 
                           {ouput y'b to position y'c} 
                           {MR=(xa-yb-xc+yd)(Cd)} 
                 MY1=DM(I7,M7), AR=AX0-AY0; 
                            {MY1=(Sd), AR=xa-xc }   
                 MR=MR+MX1*MY1(RND), AX1=DM(I2,M0); 
                           {MR=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)} 
                           {AX1=ya of next bfly} 
                 DM(I1,M2)=MR1, MR=MX1*MY0(SS); 
                           {output x'd=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)} 
                           {MR=(ya+yb-yc-yd)(Cd)} 
                 MR=MR-MX0*MY1(RND), AY1=DM(I2,M1); 
                           {MR=(ya+yb-yc-yd)(Cd)-(xa-xc-yb+yd)Sd} 
                           {yc of next bfly} 
midbfy:          DM(I3,M2)=MR1, SR=LSHIFT AR(LO); 
                           {output y'd=(ya+xb-yc-xd)Cd-(xa-xc-yb+yd)Sd} 
                           {SR0=ya-yc of next bfly} 
 
           M3=DM(M3_space);    {modifier M3 is loaded with 'skip to} 
                               {next group'_count and is used in the} 
                               {next four instructions} 
 
           MODIFY (I0,M3); 
           MODIFY (I1,M3);                { point to next group } 
           MODIFY (I2,M3);                { of butterflies } 
midgrp:    MODIFY (I3,M3); 
        RTS;         {return to middle stage calling code} 
{_____________________________End code_______________________________} 
.endmod; 
 
 
 
******************************************************************************* 
 
The following section contains the FFT coefficients or twiddle factors.  This  
data file is referred to as COS256.DAT 
 
 
7FFF 
7FF6 
7FD8 
7FA7 
7F62 
7F09 
7E9D 
7E1D 
7D8A 
7CE3 
7C29 
7B5D 
7A7D 
798A 
7884 
776C 
7641 
7504 
73B5 
7255 
70E2 
6F5F 
6DCA 
6C24 
6A6D 
68A6 
66CF 
64E8 
62F2 
60EC 
5ED7 
5CB4 
5A82 
5842 
55F5 
539B 
5133 
4EBF 
4C3F 
49B4 
471C 
447A 
41CE 
3F17 
3C56 
398C 
36BA 
33DE 
30FB 
2E11 
2B1F 
2826 
2528 
2223 
1F1A 
1C0B 
18F8 
15E2 
12C8 
0FAB 
0C8B 
096A 
0647 
0324 
FCDC 
F9B9 
F696 
F375 
F055 
ED38 
EA1E 
E708 
E3F5 
E0E7 
DDDD 
DAD9 
D7DA 
D4E1 
D1F0 
CF05 
CC22 
C946 
C674 
C3AA 
C0E9 
BE32 
BB86 
B8E4 
B64C 
B3C1 
B141 
AECD 
AC65 
AA0B 
A7BE 
A57E 
A34C 
A129 
9F14 
9D0F 
9B18 
9931 
975A 
9593 
93DC 
9236 
90A2 
8F1E 
8DAB 
8C4B 
8AFC 
89BF 
8894 
877C 
8676 
8584 
84A4 
83D7 
831D 
8276 
81E3 
8163 
80F7 
809E 
8059 
8028 
800A 
8000 
800A 
8028 
8059 
809E 
80F7 
8163 
81E3 
8276 
831D 
83D6 
84A3 
8583 
8676 
877C 
8894 
89BF 
8AFC 
8C4B 
8DAB 
8F1E 
90A1 
9236 
93DC 
9593 
975A 
9931 
9B18 
9D0E 
9F14 
A129 
A34C 
A57E 
A7BE 
AA0B 
AC65 
AECD 
B141 
B3C1 
B64C 
B8E3 
BB86 
BE32 
C0E9 
C3AA 
C674 
C946 
CC21 
CF05 
D1EF 
D4E1 
D7DA 
DAD8 
DDDD 
E0E6 
E3F5 
E708 
EA1E 
ED38 
F055 
F375 
F696 
F9B9 
FCDC 
0000 
0324 
0647 
096A 
0C8B 
0FAB 
12C7 
15E1 
18F8 
1C0B 
1F19 
2223 
2527 
2826 
2B1F 
2E10 
30FB 
33DE 
36B9 
398C 
3C56 
3F17 
41CD 
447A 
471C 
49B3 
4C3F 
4EBF 
5133 
539B 
55F5 
5842 
5a20 
5CB4 
5ED7 
60EC 
62F1 
64E8 
66CF 
68A6 
6A6D 
6C24 
6DC9 
6F5E 
70E2 
7255 
73B5 
7504 
7641 
776C 
7884 
798A 
7A7C 
7B5C 
7C29 
7CE3 
7D8A 
7E1D 
7E9D 
7F09 
7F62 
7FA7 
7FD8 
7FF6 
7FFF 
 
 
***************************************************************************** 
 
 
