/****************************************************************/
/*   DUAL-TONE MULTIFREQUENCY RECEIVER USING THE		*/
/*   WE(R) DSP16 DIGITAL SIGNAL PROCESSOR			*/
/*   April, 1989  Doc# AP89-000DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1989 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*		s16dtmf1.s	Program Listing 1. (modified)	*/
/******************************************************************************/
/*	The following code implements 1 DFT based DTMF receiver on the AT&T   */
/*	WE (R) DSP16.  The DSP16 receives sampled data on the serial input    */
/*	port in mu-law format using passive clocks.  When a valid digit is    */
/*	detected, it is written to location 0 of PRAM. All following detected */
/*	valid digits are appended to this location in PRAM.		      */
/*									      */
/*	Sixteen DTMF receivers can be implemented on 1 AT&T WE (R) DSP16      */
/*	running at 55 nsec using only internal memory.			      */
/******************************************************************************/

/******************************************************************************/
/*									      */
/*	The energy and twist ratio coefficients which are used to vary the    */
/*	performance of the DTMF receiver are contained below.  The frequency  */
/*	offset ratio coefficients are located at _vldtone at the end of the   */
/*	program.							      */
/*									      */
/* ENERGY RATIO COEFFICIENTS						      */
#define	lonrgcof  .9		/* 1.8/2 increase cof to increase sensitivity */
#define	hinrgcof  .3		/* .6/2 increase cof to increase sensitivity  */
#define	sumnrgco   5		/* increase coeff to increase sensitivity     */

/* 	lonrgcof effects reverse twist directly!!!			      */
/* 	hinrgcof effects normal twist directly!!!			      */
/* 	sumnrgco effects the freq. offset of the corner DTMF frequencies      */
/* TWIST RATIO COEFFICIENTS						      */
#define twstrvse   3		/* decrease coeff to decrease twist 	      */
#define twstfrwd   9		/* decrease coeff to decrease twist 	      */

/*									      */
/******************************************************************************/


		sioc=0x43	/* 8 bit I/O, passive, MSB first */
		pioc=0x1800	/* strobe=1T; PIDS,PODS active */

/****************************************************************************/
/*									    */
/*		This section is just for testing!!!!!!!!!!!		    */
/*		It clears 200 locations in PRAM.			    */

		r0=0
		a0=r0
		y=-1
		a1=y
		c1=-1
clear1:		c0=-126
clear:		pdx1=a0
		pdx0=a1
		a0h=a0h+1
		if c0lt goto clear
		if c1lt goto clear1
pause:	;

/*									    */
/****************************************************************************/

		r0=0
		r1=PRAMptr
		*r1=r0			/* initialize PRAM pointer 	*/

frame:		c0=1-15		/* set loop for 105 samples/frame	*/

/* i is being used as another counter inplace of c1 which will be used by */
/* intdriver in the multiple dtmf receiver software.			*/
		i=-7		/* check input level every 15 samples   */
		r0=energy
		r1=stvar
		r2=0
		*r0--=r2		/* zero energy value per frame   */
		do 16 {			/* zero the state variables on a */
			*r1++ = r2	/* frame by frame basis		 */
		}

		r1=scaleinp
		y=1.0
		*r1=y		/* reset input scaling factor to one  */

loop:		y=0x10
wait:		a0=pioc
		a0&y
	if eq	goto wait	/* test for IBF	  */
		a0=sdx		/* a0=0x000DD0000 */

/* This section performs a ulaw to linear conversion on the input data (a0)*/
/* using a table lookup.  The value stored in a1 is 8*linear value.	   */

	y=mtolin
	a0=a0+y
	pt=a0
			y=*r0	x=*pt++	   /* y=dummy read, x=8*linear value*/
	a1=x				   /* store 8*linear value in a1    */

/*	a1 contains the linear 16 bit value  */

/*	The following commands do an auto scale on the input to adjust for */
/*	different input levels.  This effectively creates a > 35dB dynamic */
/*	range.  							   */

	auc=0x2			/* clear low, saturate, shift p left by 2 */
	r1=scaleinp
			y=a1
			x=*r1
		p=x*y
	a0=p			/* scale input down */


		a1=a0
		a1=a1>>1	/* scale input used for energy calculation */
		a1=a1>>1	/* scale input used for energy calculation */
		*r0--=a1	/* store input used for energy calculation */

/*	The following command is used to scale the input down so that when */
/*	the mag. squared is computed the result doesn't overflow, and the  */
/*	ratio between tone present and no tone present is greatest.        */

pause1:		a0=a0>>4	/* scale input used for state variable update */
		*r0=a0		/* store input used for state variable update */

/***************************************************************************/
/*									   */
/*	This section computes the LHS of Goertzels Algorithm for the 1st   */
/*	order harmonics of each touchtone on a sample by sample basis.     */
/*									   */
/***************************************************************************/

	pt=coef
	r1=stvar+1
	r2=stvar+3
	j=0
	k=4
	do 4 {				/* optimized with 2 DFT's at a time*/
		a1=a0
					   y=*r1--	x=*pt++	   /* a11 */
				p=x*y	   *r1zp:y		   /* a12 */
	 	a0=a0-y			   y=*r2--	x=*pt++   /* a21 */
	 	a0=a0+p		p=x*y	   *r2zp:y		   /* a21 */
	 	a1=a1-y			   *r1jk:a0
	 	a1=a1+p			   a0=*r0
					   *r2jk:a1
	}

/*	update the total energy value	  */

		*r0++
		a1=*r0++
							x=a1	 /* input val */
					   y=a1			 /* input val */
				p=x*y				 /*  input^2  */
					   a1=*r0		 /*   energy  */
		a1=a1+p					     /* update energy */
					   *r0--=a1	     /*  store energy */
test:	if c0lt goto loop
	a1=i
	a1h=a1h+1			/* increments i and sets flags  */
	i=a1				/* 	update i		*/
test1:	if pl goto strtimag

scalesub:r1=stvar
	c0=-15
	y=13000
sub2:	a1=*r1++
	a1=a1
	if mi goto sub3
	goto sub4
sub3:	a1=-a1
sub4:	a1=a1-y
	if gt goto sub1
	if c0lt goto sub2
	goto endsub
sub1:	r1=stvar
	do 16 {
		a1=*r1
		a1=a1>>1
		*r1++=a1
	}
	r1=energy
		a1=*r1
		a1=a1>>1
		a1=a1>>1
		*r1++=a1
	r1=scaleinp
	x=*r1
	y=.5
	p=x*y
	a1=p
	*r1=a1
endsub:	c0=-14
	r1=input		/* reset r1 input pointer for next 20 samples */
	goto loop

/*****************************************************************************/
/*									     */
/*	This section computes the magnitude squared of the RHS of Goertzels  */
/*   	Algorithm  for the 1st harmonics of each tone: 			     */
/*		|Y|^2 = Sn^2 + S(n-1)^2 - (Sn*S(n-1))*coeff.		     */
/*	Each value is stored in consecutive locations starting at iirmag.    */
/*									     */
/*****************************************************************************/

strtimag:pt=coef
	r1=stvar
	r2=iirmag
	c0=-7
imag:		auc=0x0
						    x=*r1  	/* x=S(n-1) */
					y=*r1++		   	/* y=S(n-1) */
			    
			     p=x*y		    x=*r1      	/* x=S(n) */
		a0=p	     p=x*y	y=*r1++		   	/* y=S(n) */
		a0=a0<<1					/* 2*S(n-1)^2 */
		a1=p	     p=x*y
		a0=a0+p					     /*2*S(n-1)^2 + S(n)^2*/
		a0=a0+p			y=a1	    x=*pt++  /*2*S(n-1)^2 + 2*S(n)^2*/
		auc=0x2
			     p=x*y
		a1=p
		a1=a1<<1				     /* 2*S(n-1)*S(n)*coeff */
					y=a1
		a1=a1<<16
					yl=a1
		a0=a0-y
		a0=a0<<1
		a0=a0<<1
					*r2zp:a0
	if c0lt goto imag

/******************************************************************************/
/*									      */
/*	This section finds the largest value in each band and also their      */
/*	location (used for decoding).					      */
/*	The 2 maximum values are stored sequentially starting  at iir2mag     */
/*	and their corresponding row and col locations are stored sequentially */
/*	at iirincr.							      */
/*									      */
/******************************************************************************/

	auc=0x0
	r0=iirmag			/* loc. of 1st harmonic magnitudes */
	r1=iir2mag			/* loc. of the maxmag in each band */
	r2=iirincr			/* loc. of the exact frequency     */
	i=-1				/*i is temporary counter for 1 loop*/
maxmag1:a0=*r0++
	c0=-2
	a1=a0<<16
	*r2=a1				/* initialize to zero */
	*r1=a1				/* initialize to zero */
maxmag:	y=*r0++
	a1=a0-y				/* compare the magnitudes */
	if le goto maxmag2		/* if le update new maxmag */
	a1=*r1				/* otherwise update intermediate */
	a1h=a1h+1
	*r1=a1
	goto maxmag3
maxmag2:a0=y				/* update new max. mmagnitude */
	a1=*r2
	y=*r1
	a1h=a1h+1
	a1=a1+y				/* update loc. of exact frequency */
	*r2=a1
	a1=a1<<16
	*r1=a1				/* reset *r1 (temp iirincr) to zero */
maxmag3:if c0lt goto maxmag	
	*r1++=a0
	y=*r2++				/* just used to increment mem. ptr. */
	a1=i
	a1h=a1h+1			/* increments i and sets flags  */
	i=a1				/* 	update i		*/
	if le goto maxmag1		/* now find the maxmag in high band */

/*****************************************************************************/
/*									     */
/*	This section compares the lower of the high and low frequency band   */
/*	magnitudes to a threshold level.  If it is lower than the threshold  */
/*	the program updates the valid digit stack with a -1 and retrieves    */
/*	another frame. Otherwise, it continues and checks the twist.	     */
/*	If the row magnitude is greater than the column magnitude then       */
/*	normal twist is tested, otherwise reverse twist is tested.	     */
/*									     */
/*****************************************************************************/
pause2:	r0=iir2mag
	y=*r0++				/* y=row magnitude */
	a0=y	y=*r0--			/* y=column magnitude */
	a1=a0-y				/* compare row and col magnitude */
	if gt a0=y
	y=20
	a0=a0-y
	if le goto notone
	a1=a1
	if gt goto normtwst

/*	REVERSE TWIST TEST 	*/
revrtwst:x=twstrvse		/* 4dB of reverse twist test. x HAS TO BE INT */
				/* the bigger x is, the more twist is allowed */
			y=*r0++
		p=x*y				/* ratio is in p's lower byte */
	a0=*r0
	a0=a0>>16				/* have to shift to subtract */
	a1=a0-p
	if gt goto notone
	goto frqofset

/*	NORMAL TWIST TEST 	*/
normtwst:x=twstfrwd		/* 8 dB of normal twist test, x HAS TO BE INT */ 
				/* the bigger x is, the more twist is allowed */
	a0=*r0++
	a0=a0>>16				/* have to shift to subtract */
			y=*r0
		p=x*y				/* ratio is in p's lower byte */
	a1=a0-p
	if gt goto notone

/*****************************************************************************/
/*									     */
/*	Frequency offset test.						     */
/*	Each magnitude is compared with the largest magnitude in its band.   */
/*	If it is within a certain ratio the test fails and the program 	     */
/*	returns and samples another frame.				     */
/*									     */
/*****************************************************************************/
frqofset:r0=iir2mag
	r1=iirmag
	pt=_vldtone
	c0=-3				/* low & high band loop counters */
	i=-1				/*i is temporary counter for 1 loop*/
freqoff1:a0=*r0++
	a0=a0>>16
freqoff2:			  	    x=1
				y=*r1
		     p=x*y	y=*r1++	    x=*pt++
	a1=a0-p      p=x*y
	if eq goto contt		/* test to skip same mag. comparison */
	a1=a0-p
	if mi goto notone	       /* if ratio is too small, return begin */
contt:	if c0lt goto freqoff2
	c0=-3
	a1=i
	a1h=a1h+1			/* increments i and sets flags  */
	i=a1				/* 	update i		*/
	if le goto freqoff1		/* loop back for high band test */

/*****************************************************************************/
/*									     */
/*	Energy ratio test.						     */
/*	This section compares the total inband energy to the largest mag.    */
/*	in the low band, the largest mag. in the high band, and the sum      */
/*	of the two magnitudes.  Since the magnitudes computed are really     */
/*	|Y|^2 they are greater than the total energy therefore the total     */
/*	energy is scaled and then compared with the |Y|^2.		     */
/*									     */
/*****************************************************************************/

nrg:	r0=iir2mag
	r1=energy
	auc=0x2
	a0=*r0++					/*largest low band mag*/
	a0=a0>>1
					   x=*r1	/* y=total energy */
				y=lonrgcof		/* y=low band coeff */
		     p=x*y				/* p=x*y*2 */
	a1=a0-p
	if le goto notone
	a1=*r0--				       /*largest high band mag*/
	a1=a1>>1
				y=hinrgcof		/* y=high band coeff */
		     p=x*y				/* p=x*y*2 */
	a0=a1-p
	if le goto notone
	auc=0x0
	a1=a1<<1					/* reshift up by 1  */
	a1=a1>>16
				y=0			/* zero high 16 bits */
				yl=*r0			/*largest low band mag*/
	a1=a1+y						/* low mag + high mag */
				y=sumnrgco		/*y=sum of energy coeff*/
		     p=x*y				/* p=x*y*2 */
	a1=a1-p
	if le goto notone

/*****************************************************************************/
/*									     */
/*	Decode Digits.							     */
/*	All valid digits will be stored sequentially starting at the loc.    */
/*	in memory called output.					     */
/*									     */
/*****************************************************************************/

decode:	pt=_romtbl
	r0=iirincr
	a0=*r0++
	a0=a0<<1
	a0=a0<<1
	y=*r0
	a0=a0+y
	a1=pt
	y=a0
	a1=a1+y				/* a1=loc. of decoded digit in romtbl */
	pt=a1
	y=*r0++	x=*pt++			/* y is dummy read & x=decoded digit */

/* Perform valid digit check */
	r0=ttdigit2			/* r0=loc. of 2nd to last digit D(k-2) */
	y=*r0--
	a0=x				/* a0=present digit D(k) */
	a1=a0-y
	if eq goto tone1		/* D(k)=D(k-2) ? */
	y=*r0				/* y=loc. of past digit D(k-1) */
	a1=a0-y
	if ne goto tone1		/* D(k)=D(k-1) ? */
	r1=PRAMptr
	a0=*r1
	a1=x
	pdx1=a0
	pdx0=a1
	a0h=a0h+1
	*r1=a0
	goto tone1

/*	Update past digit stack */
notone: x=-1				/* nodigit push -1 on past digit stack */
tone1:	r0=ttdigit1
	y=*r0
	*r0++=x				/* D(k-1)=D(k) */
	*r0=y				/* D(k-2)=D(k-1) */
	goto frame


/*	First order harmonic coefficients	*/
coef:	int	1.70773781
	int	1.645281
	int	1.568687
	int	1.4782
	int	1.1641
	int	.99637
	int	.7986184
	int	.5685327

/* 		FREQUENCY OFFSET RATIO COEFFICIENTS			     */
/* the larger the coeff the smaller the range of frq offset		     */
_vldtone:int	7			/* 697Hz freq offset ratio coeff     */
	int	13			/* 770Hz freq offset ratio coeff     */
	int	17			/* 852Hz freq offset ratio coeff     */
	int	16			/* 941Hz freq offset ratio coeff     */
	int	9			/* 1209Hz freq offset ratio coeff    */
	int	12			/* 1336Hz freq offset ratio coeff    */
	int	13			/* 1477Hz freq offset ratio coeff    */
	int	17			/* 1633Hz freq offset ratio coeff    */

_romtbl:int 1, 2, 3, 10
	int 4, 5, 6, 11
	int 7, 8, 9, 12
	int 14, 0, 15, 13

.ram
stvar:	16*int				/* 1st harmonic state variables */
iirmag:	8*int				/* 1st harm's mag. squared */
iir2mag:2*int				/* max mag in low and high band */
iirincr:2*int				/* row and col. location of digit */
input:int			/* input used for state variable update */
input1:int			/* input used for energy calculation */
energy:	int				/* frame by frame energy value */
scaleinp:int				/* input scaling factor */
ttdigit1:int				/* past digit D(k-1) */
ttdigit2:int				/* 2nd to last digit D(k-2) */
PRAMptr:int
.endram

/*              4 Times   						*/
/*              Linear      mulaw	Linear				*/
/*              Value       Value	Value  				*/
/*              (Hex)      (binary)   (Decimal)				*/

mtolin: int     0x8284  /* 00000000     -8031                           */
        int     0x8684  /* 00000001     -7775                           */
        int     0x8a84  /* 00000010                                     */
        int     0x8e84  /* 00000011                                     */
        int     0x9284  /* 00000100                                     */
        int     0x9684  /* 00000101                                     */
        int     0x9a84  /* 00000110                                     */
        int     0x9e84  /* 00000111                                     */
        int     0xa284  /* 00001000                                     */
        int     0xa684  /* 00001001                                     */
        int     0xaa84  /* 00001010                                     */
        int     0xae84  /* 00001011                                     */
        int     0xb284  /* 00001100                                     */
        int     0xb684  /* 00001101                                     */
        int     0xba84  /* 00001110                                     */
        int     0xbe84  /* 00001111     -4191                           */
        int     0xc184  /* 00010000                                     */
        int     0xc384  /* 00010001                                     */
        int     0xc584  /* 00010010                                     */
        int     0xc784  /* 00010011                                     */
        int     0xc984  /* 00010100                                     */
        int     0xcb84  /* 00010101                                     */
        int     0xcd84  /* 00010110                                     */
        int     0xcf84  /* 00010111                                     */
        int     0xd184  /* 00011000                                     */
        int     0xd384  /* 00011001                                     */
        int     0xd584  /* 00011010                                     */
        int     0xd784  /* 00011011                                     */
        int     0xd984  /* 00011100                                     */
        int     0xdb84  /* 00011101                                     */
        int     0xdd84  /* 00011110                                     */
        int     0xdf84  /* 00011111     -2079                           */
        int     0xe104  /* 00100000                                     */
        int     0xe204  /* 00100001                                     */
        int     0xe304  /* 00100010                                     */
        int     0xe404  /* 00100011                                     */
        int     0xe504  /* 00100100                                     */
        int     0xe604  /* 00100101                                     */
        int     0xe704  /* 00100110                                     */
        int     0xe804  /* 00100111                                     */
        int     0xe904  /* 00101000                                     */
        int     0xea04  /* 00101001                                     */
        int     0xeb04  /* 00101010                                     */
        int     0xec04  /* 00101011                                     */
        int     0xed04  /* 00101100                                     */
        int     0xee04  /* 00101101                                     */
        int     0xef04  /* 00101110                                     */
        int     0xf004  /* 00101111     -1023                           */
        int     0xf0c4  /* 00110000                                     */
        int     0xf144  /* 00110001                                     */
        int     0xf1c4  /* 00110010                                     */
        int     0xf244  /* 00110011                                     */
        int     0xf2c4  /* 00110100                                     */
        int     0xf344  /* 00110101                                     */
        int     0xf3c4  /* 00110110                                     */
        int     0xf444  /* 00110111                                     */
        int     0xf4c4  /* 00111000                                     */
        int     0xf544  /* 00111001                                     */
        int     0xf5c4  /* 00111010                                     */
        int     0xf644  /* 00111011                                     */
        int     0xf6c4  /* 00111100                                     */
        int     0xf744  /* 00111101                                     */
        int     0xf7c4  /* 00111110                                     */
        int     0xf844  /* 00111111     -495                            */
        int     0xf8a4  /* 01000000                                     */
        int     0xf8e4  /* 01000001                                     */
        int     0xf924  /* 01000010                                     */
        int     0xf964  /* 01000011                                     */
        int     0xf9a4  /* 01000100                                     */
        int     0xf9e4  /* 01000101                                     */
        int     0xfa24  /* 01000110                                     */
        int     0xfa64  /* 01000111                                     */
        int     0xfaa4  /* 01001000                                     */
        int     0xfae4  /* 01001001                                     */
        int     0xfb24  /* 01001010                                     */
        int     0xfb64  /* 01001011                                     */
        int     0xfba4  /* 01001100                                     */
        int     0xfbe4  /* 01001101                                     */
        int     0xfc24  /* 01001110                                     */
        int     0xfc64  /* 01001111     -231                            */
        int     0xfc94  /* 01010000                                     */
        int     0xfcb4  /* 01010001                                     */
        int     0xfcd4  /* 01010010                                     */
        int     0xfcf4  /* 01010011                                     */
        int     0xfd14  /* 01010100                                     */
        int     0xfd34  /* 01010101                                     */
        int     0xfd54  /* 01010110                                     */
        int     0xfd74  /* 01010111                                     */
        int     0xfd94  /* 01011000                                     */
        int     0xfdb4  /* 01011001                                     */
        int     0xfdd4  /* 01011010                                     */
        int     0xfdf4  /* 01011011                                     */
        int     0xfe14  /* 01011100                                     */
        int     0xfe34  /* 01011101                                     */
        int     0xfe54  /* 01011110                                     */
        int     0xfe74  /* 01011111     -99                             */
        int     0xfe8c  /* 01100000                                     */
        int     0xfe9c  /* 01100001                                     */
        int     0xfeac  /* 01100010                                     */
        int     0xfebc  /* 01100011                                     */
        int     0xfecc  /* 01100100                                     */
        int     0xfedc  /* 01100101                                     */
        int     0xfeec  /* 01100110                                     */
        int     0xfefc  /* 01100111                                     */
        int     0xff0c  /* 01101000                                     */
        int     0xff1c  /* 01101001                                     */
        int     0xff2c  /* 01101010                                     */
        int     0xff3c  /* 01101011                                     */
        int     0xff4c  /* 01101100                                     */
        int     0xff5c  /* 01101101                                     */
        int     0xff6c  /* 01101110                                     */
        int     0xff7c  /* 01101111     -33                             */
        int     0xff88  /* 01110000                                     */
        int     0xff90  /* 01110001                                     */
        int     0xff98  /* 01110010                                     */
        int     0xffa0  /* 01110011                                     */
        int     0xffa8  /* 01110100                                     */
        int     0xffb0  /* 01110101                                     */
        int     0xffb8  /* 01110110                                     */
        int     0xffc0  /* 01110111                                     */
        int     0xffc8  /* 01111000                                     */
        int     0xffd0  /* 01111001                                     */
        int     0xffd8  /* 01111010                                     */
        int     0xffe0  /* 01111011                                     */
        int     0xffe8  /* 01111100                                     */
        int     0xfff0  /* 01111101                                     */
        int     0xfff8  /* 01111110                                     */
        int     0x0000  /* 01111111     0          (negative zero)      */
        /* Positive mu-law values, starting with the largest value.     */
        int     0x7d7c  /* 10000000     8031       1.5777    3.17       */
        int     0x797c  /* 10000001                1.5282    2.89       */
        int     0x757c  /* 10000010                1.4787    2.61       */
        int     0x717c  /* 10000011                1.4292    2.31       */
        int     0x6d7c  /* 10000100                1.3797    2.00       */
        int     0x697c  /* 10000101                1.3302    1.69       */
        int     0x657c  /* 10000110                1.2807    1.36       */
        int     0x617c  /* 10000111                1.2312    1.01       */
        int     0x5d7c  /* 10001000                1.1817    0.66       */
        int     0x597c  /* 10001001                1.1322    0.29       */
        int     0x557c  /* 10001010                1.0826   -0.10       */
        int     0x517c  /* 10001011                1.0331   -0.51       */
        int     0x4d7c  /* 10001100                 .9836   -0.94       */
        int     0x497c  /* 10001101                 .9341   -1.38       */
        int     0x457c  /* 10001110                 .8846   -1.86       */
        int     0x417c  /* 10001111  4191           .8351   -2.36       */
        int     0x3e7c  /* 10010000                 .7856   -2.89       */
        int     0x3c7c  /* 10010001                 .7608   -3.17       */
        int     0x3a7c  /* 10010010                 .7361   -3.45       */
        int     0x387c  /* 10010011                 .7113   -3.75       */
        int     0x367c  /* 10010100                 .6866   -4.06       */
        int     0x347c  /* 10010101                 .6618   -4.38       */
        int     0x327c  /* 10010110                 .6371   -4.71       */
        int     0x307c  /* 10010111                 .6123   -5.05       */
        int     0x2e7c  /* 10011000                 .5875   -5.41       */
        int     0x2c7c  /* 10011001                 .5628   -5.78       */
        int     0x2a7c  /* 10011010                 .5380   -6.18       */
        int     0x287c  /* 10011011                 .5133   -6.58       */
        int     0x267c  /* 10011100                 .4885   -7.01       */
        int     0x247c  /* 10011101                 .4638   -7.47       */
        int     0x227c  /* 10011110                 .4390   -7.94       */
        int     0x207c  /* 10011111  2079           .4143   -8.45       */
        int     0x1efc  /* 10100000                 .3895   -8.98       */
        int     0x1dfc  /* 10100001                 .3771   -9.26       */
        int     0x1cfc  /* 10100010                 .3647   -9.55       */
        int     0x1bfc  /* 10100011                 .3524   -9.85       */
        int     0x1afc  /* 10100100                 .3400  -10.16       */
        int     0x19fc  /* 10100101                 .3276  -10.48       */
        int     0x18fc  /* 10100110                 .3152  -10.82       */
        int     0x17fc  /* 10100111                 .3029  -11.17       */
        int     0x16fc  /* 10101000                 .2905  -11.53       */
        int     0x15fc  /* 10101001                 .2781  -11.91       */
        int     0x14fc  /* 10101010                 .2657  -12.30       */
        int     0x13fc  /* 10101011                 .2534  -12.72       */
        int     0x12fc  /* 10101100                 .2410  -13.15       */
        int     0x11fc  /* 10101101                 .2286  -13.61       */
        int     0x10fc  /* 10101110                 .2162  -14.09       */
        int     0x0ffc  /* 10101111  1023           .2038  -14.61       */
        int     0x0f3c  /* 10110000                 .1915  -15.15       */
        int     0x0ebc  /* 10110001                 .1853  -15.44       */
        int     0x0e3c  /* 10110010                 .1791  -15.73       */
        int     0x0dbc  /* 10110011                 .1729  -16.04       */
        int     0x0d3c  /* 10110100                 .1667  -16.35       */
        int     0x0cbc  /* 10110101                 .1605  -16.68       */
        int     0x0c3c  /* 10110110                 .1543  -17.02       */
        int     0x0bbc  /* 10110111                 .1481  -17.38       */
        int     0x0b3c  /* 10111000                 .1420  -17.75       */
        int     0x0abc  /* 10111001                 .1358  -18.14       */
        int     0x0a3c  /* 10111010                 .1296  -18.54       */
        int     0x09bc  /* 10111011                 .1234  -18.97       */
        int     0x093c  /* 10111100                 .1172  -19.41       */
        int     0x08bc  /* 10111101                 .1110  -19.88       */
        int     0x083c  /* 10111110                 .1048  -20.38       */
        int     0x07bc  /* 10111111   495           .0986  -20.91       */
        int     0x075c  /* 11000000                 .0924  -21.47       */
        int     0x071c  /* 11000001                 .0894  -21.77       */
        int     0x06dc  /* 11000010                 .0863  -22.08       */
        int     0x069c  /* 11000011                 .0832  -22.39       */
        int     0x065c  /* 11000100                 .0801  -22.72       */
        int     0x061c  /* 11000101                 .0770  -23.07       */
        int     0x05dc  /* 11000110                 .0739  -23.42       */
        int     0x059c  /* 11000111                 .0708  -23.79       */
        int     0x055c  /* 11001000                 .0677  -24.18       */
        int     0x051c  /* 11001001                 .0646  -24.59       */
        int     0x04dc  /* 11001010                 .0615  -25.01       */
        int     0x049c  /* 11001011                 .0584  -25.46       */
        int     0x045c  /* 11001100                 .0553  -25.94       */
        int     0x041c  /* 11001101                 .0522  -26.44       */
        int     0x03dc  /* 11001110                 .0491  -26.97       */
        int     0x039c  /* 11001111   231           .0460  -27.53       */
        int     0x036c  /* 11010000                 .0429  -28.14       */
        int     0x034c  /* 11010001                 .0414  -28.45       */
        int     0x032c  /* 11010010                 .0398  -28.79       */
        int     0x030c  /* 11010011                 .0383  -29.13       */
        int     0x02ec  /* 11010100                 .0367  -29.49       */
        int     0x02cc  /* 11010101                 .0352  -29.86       */
        int     0x02ac  /* 11010110                 .0337  -30.25       */
        int     0x028c  /* 11010111                 .0321  -30.66       */
        int     0x026c  /* 11011000                 .0306  -31.09       */
        int     0x024c  /* 11011001                 .0290  -31.54       */
        int     0x022c  /* 11011010                 .0275  -32.02       */
        int     0x020c  /* 11011011                 .0259  -32.52       */
        int     0x01ec  /* 11011100                 .0244  -33.06       */
        int     0x01cc  /* 11011101                 .0228  -33.63       */
        int     0x01ac  /* 11011110                 .0213  -34.23       */
        int     0x018c  /* 11011111    99           .0197  -34.89       */
        int     0x0174  /* 11100000                 .0182  -35.60       */
        int     0x0164  /* 11100001                 .0174  -35.98       */
        int     0x0154  /* 11100010                 .0166  -36.37       */
        int     0x0144  /* 11100011                 .0159  -36.79       */
        int     0x0134  /* 11100100                 .0151  -37.22       */
        int     0x0124  /* 11100101                 .0143  -37.68       */
        int     0x0114  /* 11100110                 .0135  -38.16       */
        int     0x0104  /* 11100111                 .0128  -38.67       */
        int     0x00f4  /* 11101000                 .0120  -39.21       */
        int     0x00e4  /* 11101001                 .0112  -39.79       */
        int     0x00d4  /* 11101010                 .0104  -40.41       */
        int     0x00c4  /* 11101011                 .0097  -41.08       */
        int     0x00b4  /* 11101100                 .0089  -41.81       */
        int     0x00a4  /* 11101101                 .0081  -42.60       */
        int     0x0094  /* 11101110                 .0073  -43.47       */
        int     0x0084  /* 11101111    33           .0066  -44.43       */
        int     0x0078  /* 11110000                 .0058  -45.52       */
        int     0x0070  /* 11110001                 .0054  -46.12       */
        int     0x0068  /* 11110010                 .0050  -46.76       */
        int     0x0060  /* 11110011                 .0046  -47.46       */
        int     0x0058  /* 11110100                 .0043  -48.21       */
        int     0x0050  /* 11110101                 .0039  -49.04       */
        int     0x0048  /* 11110110                 .0035  -49.96       */
        int     0x0040  /* 11110111                 .0031  -50.98       */
        int     0x0038  /* 11111000                 .0027  -52.14       */
        int     0x0030  /* 11111001                 .0023  -53.48       */
        int     0x0028  /* 11111010                 .0019  -55.06       */
        int     0x0020  /* 11111011                 .0015  -57.00       */
        int     0x0018  /* 11111100                 .0012  -59.50       */
        int     0x0010  /* 11111101                 .0008  -63.02       */
        int     0x0008  /* 11111110     2           .0004  -69.04       */
        int     0x0000  /* 11111111     0           (positive zero)     */
