/******************************************************************************/
/*									      */
/*		DFT based DTMF receiver for AT&T WE (R) DSP32		      */
/*									      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/*			COPYRIGHT (c) 1987 AT&T				      */
/*									      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/*	FILE = sdtmf31.s						      */
/*									      */
/*	The following code implements 1 DFT based DTMF receiver on the AT&T   */
/*	WE (R) DSP32.  The DSP32 receives sampled data on the serial input    */
/*	port in mu-law format using passive clocks.  When a valid digit is    */
/*	detected, it is written to the memory location called output. All     */
/*	following detected valid digits are appended to this location.        */
/*									      */
/*	Sixteen DTMF receivers can be implemented on 1 AT&T WE (R) DSP32      */
/*	running at 160 nsec when RAM is used to store program and data.       */
/*									      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/*			DISCLAIMER AND NOTICE				      */
/*									      */
/*	AT&T and its SUBSIDIARIES (hereinafter AT&T) reserve the right to     */
/*	make changes in this APPLICATION NOTE and the semiconductor devices   */
/*	or device specifications to which it is directed.		      */
/*									      */
/*	AT&T makes no representations or warranties, expressly or implied.    */
/*	By way of example but not of limitation, AT&T makes no representa-    */
/*	tions or warranties of merchantability or fitness for any particular  */
/*	purpose, or that the use of the INFORMATION in this APPLICATION NOTE  */
/*	or any of it will not infringe any patent, copyright or other intel-  */
/*	lectual property right of a third party.  AT&T shall not be held to   */
/*	any liability with respect to any claim by a USER or any third party  */
/*	on account of, or arising from the use of the INFORMATION or any      */
/*	part of it.  							      */
/*									      */
/*	AT&T further does not grant any license, express or implied, under    */
/*	any patent, copyright or other intellectual property right of AT&T    */
/*	covering or relating to any process, machine or combination in which  */
/*	the INFORMATION and/or semiconductor device or devices may be or are  */
/*	used.								      */
/*									      */
/*	The software code and other information contained in this APPLICA-    */
/*	TION NOTE are copyrighted and all rights are reserved by AT&T.  The   */
/*	software code is intended for use only in an AT&T Digital Signal      */
/*	Processor.  No other use is authorized.				      */
/*									      */
/******************************************************************************/
.global _iirstvr,_iirmag,_iir2mag,_iirincr,pause,_ttdigit,output

	dauc=0x0
	ioc=0x547		/* 8-bit input, passive clocks */
	r13=26			/* 14 frames -2 for digits 1,5,9,d */
	r14=output
	r15=12
begin:	r5=102-2				/* DFT frame size - 2 */
	r1=input		/* loc. of input sample used in energy calc. */
	r7=30
	r10=_iirzero			
	a2=*r10				/* zero energy frame by frame */
	r3=_iirstvr			/* pointer to state variables */
clear:	if (r7-- >= 0) goto clear	/* zero the state variables on a */
	*r3++ = a0 = *r10		/* frame by frame basis.	 */
sample:	if (ibe) goto sample
	r2=_iirco			/* pointer to coefficients */
	*r1=a1=ic(ibuf)			/* convert from ulaw to float */
/****************************************************************************/
/*									    */
/*	This section computes the LHS of the DFT for the 1st and 2nd order  */
/*	harmonics of each tone for each sample.				    */
/*									    */
/****************************************************************************/
	r3=_iirstvr		/* pointer to state variables */
	r4=r3+4			/* points to st. var's. and output y(N) */
/* 697hz    1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++    	/* 2nd state var. d[1,1] */
	*r4++r15 = a0 = a0 - *r3++		/* 1st state var. d[1,2] */
/* 770hz    1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 852hz    1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 941hz    1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1209hz   1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1336hz   1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1477hz   1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1633hz   1st harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 697hz    2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 770hz    2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 852hz    2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 941hz    2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1209hz   2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1336hz   2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1477hz   2nd harmonic  LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
/* 1633hz   2nd harmonic   LHS	*/
	a0 = a1 + (*r4-- = *r3++) * *r2++		/* d[1,1] */
	*r4++r15 = a0 = a0 - *r3++			/* d[1,2] */
	if (r5-- >= 0) goto sample
	*r1-- = a2 = a2 + *r1++ * *r1		/* energy=energy+input^2 */
/****************************************************************************/
/*									    */
/*	This section computes the RHS of the DFT for the 1st harmonics 	    */
/*	of each tone.  The magnitude is squared and stored in consecutive   */
/*	locations starting at _iirmag.					    */
/*									    */
/****************************************************************************/

	r2=_iirco
	r4=_iirstvr
	r15=8
	r1=6
	r3=_iirmag		/* points to location of mag^2 of each tone */
right:	a2= *r4 * *r4++			/* Sn * Sn-1	 */
	a1=*r4-- * *r4			/* Sn-1^2  	 */
	a0 = a1 + *r4++r15 * *r4	/* Sn^2 + Sn-1^2 */
	if (r1-- >= 0) goto right
	*r3++ = a0 = a0 - a2 * *r2++	/* |Y|^2	 */

/***************************************************************************/
/*									   */
/*	This section finds the largest value in the low and high frequency */
/*	band and stores the 2 values at _iirmag. It also computes their	   */
/*	location in the low and high frequency band and stores these 2     */
/*	values at _iirincr.						   */
/*									   */
/***************************************************************************/

	r8=_iirincr
	r4=_iir2mag		/* loc. of largest mag. in each band */
	r3=_iirmag		/* loc. of mag^2 of each tone */
	r7=0
loop3:	r6=1
	r5=0
	r1=0
	r2=0
	*r4=a0=*r3++		/* writes possible large mag temporarily *r4 */
loop4:	a1=*r3++
	nop
	a2=a1-a0
	3*nop
	if (ale) goto loop1		/* test which value is larger */
	nop
	*r4=a0=a1
	r2=r2+1
	r1=r2
	goto loop2
	r1=r1+r5			/* contains the row & col increments */
loop1:	r5=r5+1
loop2:	if (r6-- >= 0) goto loop4
	nop
	*r8++=r1			/* contains the row & col increments */
	if (r7-- >= 0) goto loop3
	r4=r4+4

/**************************************************************************/
/*                                                                        */
/*	This section compares the lower of the high and low frequency	  */
/*	band magnitudes to a threshold level.  If it is lower than the	  */
/*	threshold the program updates the valid digit stack with a -1     */
/*	and retrieves another frame.  Otherwise, it continues and checks  */
/*	the twist.							  */
/*									  */
/**************************************************************************/

	r1=_iir2mag			/* loc. of max row & col mag's */
	a0= *r1 - *r1++			/* high mag. - low mag. */
	3*nop
	if (alt) goto _ttrlow
	r10=_ttrlevl
	r1=r1-4
_ttrhigh: a1 = *r1 - *r10		/* lower value - 8e7 */
	a2 = *r1++
	2*nop
	if (alt) goto notone		/* threshold check */
	r2=_twistlw
	a1 = -*r1 + a2 * *r2		/* low mag. * 2.5 - high mag. */
	3*nop
	if (alt) goto notone		/* reverse twist test */
	nop
	goto frqofset			/* continue	*/
	nop
_ttrlow: a1 = *r1 - *r10		/* lower value - 8e7 */
	a2 = *r1--
	2*nop
	if (alt) goto notone		/* threshold check */
	r2=_twisthg
	a1 = -*r1 + a2 * *r2		/* high mag. * 9.2 - low mag. */
	3*nop
	if (alt) goto notone		/* normal twist test */
	nop
/******************************************************************************/
/*									      */
/*	Do largest frequencies stand out ?				      */
/*	This section compares the largest magnitude (detected "tone") in each */
/*	frequency band with the other magnitudes in that band. If the "tone"  */
/*	is not significantly larger then the other magnitudes, the program    */
/*	returns to begin and samples another frame.			      */
/*									      */
/******************************************************************************/
frqofset:r4=0
	r10=_vldtonl
	r1=_iir2mag		/* loc. of largest mag. in each freq. band */
	r2=_iirmag		/* loc. of mag^2 of each tone */
freqoff2:r3=2
freqoff1: a1 = *r1 - *r2		/* check if mag's are the same */
	a2 = *r2++
	2*nop
	if (aeq) goto freqoff		/* if equal mag's get next mag */
	nop
	a0 = *r1 - a2 * *r10		/* highest mag. - other mags. * coeff */
	3*nop
	if (age) goto freqoff		/* if highest mag < other mag * coeff */
	nop				/* tone is not valid		      */
	goto notone			/* Therefore, update _ttdigit with -1 */
	nop				/* and get another frame of samples.  */
freqoff:if (r3-- >= 0) goto freqoff1
	nop
	r10=_vldtonh
	if (r4-- >= 0) goto freqoff2
	r1=r1+4
/******************************************************************************/
/*									      */
/*	This section compares the total inband energy to the largest mag. in  */
/*	the low band, the largest mag. in the high band, and the sum of the   */
/*	two magnitudes.							      */
/*									      */
/******************************************************************************/

energ:	r1=_iir2mag
	a2=*r1++			/* a2=largest mag in low band */
	a1=*r1				/* a1=largest mag in high band */
	r2=energy
	r3=_nrgcoef
	a3 = a2 - *r2 * *r3++		/* a3=low band mag-energy*lowband coef */
	a2=a1+a2
	a3 = a1 - *r2 * *r3++		/* a3=highband mag-energy*highband coef*/
	a3 = a2 - *r2 * *r3		/* a3=sum of mags-energy*sum coef */
	if (alt) goto notone
	nop
	if (alt) goto notone
	nop
	if (alt) goto notone
	nop
/******************************************************************************/
/*									      */
/*	Is the harmonic ratio > threshhold?				      */
/*	This computes the magnitude squared of the "tones" second harmonics   */
/* 	and compares it with the "tone". If the magnitudes are within a       */
/*	certain ratio, the program returns to begin and samples another frame.*/
/*									      */
/******************************************************************************/
	r3=_iir2mag		/* loc. of largest mag. in each freq. band */
	r1=0
	r6=0
	r5=0
	r10=_iirincr
	r7=_hrmcoef
_2ndharm: r9=*r10++			/* compute the loc. of the 2nd harm */
	nop				/* coefficients and state variables. */
	r8=r9*2
	r8=r8*2
	r2=r8+32
	r2=r2+r5
	r2=r2+_iirco			/* loc. of the 2nd harmonic coeff's */
	r8=r8*2
	r8=r8+64
	r8=r8+r6
	r4=r8+_iirstvr			/* loc. of the 2nd harm state var's */
	a2= *r4 * *r4++			/* Sn * Sn-1	 */
	a1=*r4-- * *r4			/* Sn-1^2  	 */
	a0 = a1 + *r4 * *r4		/* Sn^2 + Sn-1^2 */
	a0 = a0 - a2 * *r2		/* |Y|^2	 */
	2*nop
	a3 = *r3++ - a0 * *r7++	     /* 1st harm - 2nd harm * 2nd harm coeff */
	3*nop
	if (ale) goto notone
	r5=16
	if (r1-- >= 0) goto _2ndharm
	r6=32

/******************************************************************************/
/*									      */
/*	Decode digits							      */
/*	pdr's lower byte contains the valid decoded digit		      */
/*									      */
/******************************************************************************/

	r9=_iirincr
	r1=*r9++
	nop
	r2=r1*2
	r2=r2*2
	r2=r2*2
	r3=*r9
	nop
	r3=r3*2
	r2=r2+r3
	r2=r2+_romtbl		/* loc. of decoded digit */
	r10=*r2			/* present digit D(k) */
	r4=_ttdigit+2
	r5=*r4			/* 2nd to last digit D(k-2) */
	nop
	r5=r5-r10
	if (eq) goto tone1	/* D(k)=D(k-2) ? */
	r4=_ttdigit
	r5=*r4			/* last digit decoded D(k-1) */
	nop
	r5=r5-r10
	if (ne) goto tone1	/* D(k)=D(k-1) ? */
	nop
	*r14++=r10
	goto tone1
	nop
notone:	r10=-1			/* set digit to -1 and update stack */
tone1:  r1=_ttdigit
	r3=*r1			/* old 1st value */
	*r1++ = r10
	*r1 = r3
	if (r13-- >= 0) goto contt
	nop
pause:	nop
contt:	goto begin
	r15=12
/******************************************************************************/
/*									      */
/*				PROGRAM VARIABLES			      */
/*									      */
/*  First order harmonic coefficients					      */
_iirco: float	1.70773781, 1.645281, 1.568687, 1.4782
	float   1.1641, .99637, .7986184, .5685327
/*  Second order harmonic coefficients					      */
	float	.916368, .70695, .46077885, .1851
	float 	-.644862, -1.0072464, -1.36221, -1.676771
_iirzero: float 0.0
_ttrlevl: float 4e7
_twisthg: float 10.5
_twistlw: float 7.4
_vldtonl: float 5.0
_vldtonh: float 5.0
_nrgcoef: float 7.0, 5.6, 36.0
_hrmcoef: float 1.6, 60.0
_romtbl:  int 1, 2, 3, 10
	  int 4, 5, 6, 11
	  int 7, 8, 9, 12
	  int 14, 0, 15, 13
/*									      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/*				STATE VARIABLES				      */
/*									      */
input:	float 0.0			/* contains present input sample */
energy:	float 0.0			/* contains present energy for frame */
_iirstvr: 32*float 0.0			/* state variables */
_iirmag: 8*float 0.0
_iir2mag:2*float 0.0
_iirincr:2*int 0
_ttdigit: 2*int 0
output:   200*int 0
/*									      */
/******************************************************************************/
