/****************************************************************/
/*   DISK-DRIVE SERVO CONTROL WITH THE				*/
/*   WE(R) DSP16/DSP16A DIGITAL SIGNAL PROCESSOR		*/
/*   June, 1989 Doc# AP89-007DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1989 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*								*/
/*	servo16.s	APPENDIX A - Program Listing 1.		*/
/*								*/
/*     DSP16 code for Adaptive Servo Controller.		*/
/* The "set" mnemonic is optional.  It forces a single-cycle,	*/
/* single-word immediate load of YAAU registers for the DSP16A	*/
/* ("set" is default for the DSP16).  If the RAM variables are	*/
/* at addresses higher than 511, the "set" mnemonic should not	*/
/* be used since the address will be truncated to 9 bits,	*/
/* i.e., 511 maximum.						*/
/****************************************************************/
	
	auc=0x02			/* p <-- (x*y)*4  */
  set	r1=mem5
  set	r2=mem3
	pt=weight
	y=1.0				/* Input is step  */

			 *r2++=y	/* xn=1.0	   */
	do 40 {
			 y=a0     x=*pt++	/* Move initial weight -  */
			 *r1++=x		/* vector from ROM to RAM */
	}

	y=0x0
	r3=mem1
	do 78 {
			 *r3++=y		/* Initialize RAM to zero  */
	}

			 *r2++=y		/* Initialize mem4 to zero */
			 *r2++=y
			 *r1++=y		/* Initialize tmp1 & u(n)  */
			 *r1++=y

start:
  set	r0=mem4				/* points to u(n-1)	*/
  set	r1=mem2				/* points to c(n-2)	*/
  set	r2=mem1				/* points to c(n-39)	*/
  set	r3=mem1				/* points to c(n-39)	*/
	pt=coef				/* points to state coef.*/
	i=-1				/* pointer increment	*/
	j=39
	k=-38

/* This section of the code computes the plant output */

			  y=*r0++  x=*pt++	/* y=u(n-1); x=0.004837)   */
		 p=x*y    y=*r0++  x=*pt++	/* y=u(n-1); x=0.004678    */
	a0=p     p=x*y    y=*r1++  x=*pt++	/* y=cn(n-2); x=-0.9094    */
	a0=a0+p  p=x*y    y=*r1++  x=*pt++	/* y=cn(n-1); x=1.9	   */
	a0=a0+p  p=x*y    x=*r0++	/* x=w40; prefetch for adap. model */
	a0=a0+p		  y=*r2++   /* y=c(n-39); prefetch for adap. model */

	sdx=a0			    /* Plant output via serial port	   */

/* This section of the code computes the adaptive reference inverse model  */

		p=x*y	*r1++=a0	/* store output at cn	*/
	a1=p		y=*r1++		/* y=xn(n-39)		*/
		p=x*y	x=*r0++		/* x=w(39)		*/
	a0=p		y=*r1++		/* y=xn(n-38)		*/

	do 39 {
		 p=x*y  *r2jk:y		/* move data		*/
	a0=a0+p  p=x*y  x=*r0++		/* x=state coef.	*/
	a1=a1+p		y=*r1++		/* y=state variables	*/
	}

			*r0++=a0	/* store plant input	*/

/* Compute reference plant output */

	a0=y				/* save y		*/
			y=a0     x=*pt++	/* retrieve y; x=0.01524 */

		 p=x*y  *r1zp:y  x=*pt++	/* x=0.0147		*/
	a0=p     p=x*y  y=*r0++  x=*pt++	/* x=1.68476		*/
	a0=a0+p  p=x*y  *r0zp:y  x=*pt++	/* x=-0.7147		*/
	a0=a0+p  p=x*y  y=a1     x=*pt++i
	a0=a0+p		*r2--
	a1=a0-y		y=*r0    x=*pt++	/* compute error	*/
			y=a1     x=*pt++
		p=x*y	y=*r2
	a1=p
	j=-5
			*r0=a1
			x=*r0++j

	j=0
	k=-1

	do 40 {
		p=x*y   *r2jk:y		/* update weight vector */
			a1=*r0
	a1=a1+p		*r1++
			*r0--=a1
	}

			*r0--
			*r1++
			y=*r1++
			*r0=y
			*r1=a0

end:	goto start

/* coefficients & state variables on next page */

/* state variables */

.ram
mem1:	37*int		/* c(n-39),cn(n-38), ...,cn(n-3)		     */
mem2:	43*int		/* cn(n-2),cn(n-1),cn,xn(n-39),xn(n-38),..,xn(n-1)   */
mem3:	int		/* xn						     */
mem4:	2*int		/* un(n-1), un(n-2)				     */
mem5:	44*int		/* w40, w39, ..., w1, tmp1, un, r(n-1), r(n-2), tmp  */
.endram

/* fixed coefficients */

coef:	int  0.004837, 0.004678, -0.9094, 1.9
	int  0.01524, 0.0147, 1.68476, -.7147
	int  0.05

weight: int  -.000823, .000508, .001698, .002718   /* Initial weight value  */
	int  .003550, .004199, .004694, .005092
	int  .005479, .005973, .006722, .007896
	int  .009681, .012263, .015807, .020418
	int  .026095, .032651, .039643, .046294
	int  .051458, .053657, .051201, .042415
	int  .025951, .001145, -.031634, -.070701
	int  -.112902, -.153631, -.187024, -.206343
	int  -.204497, -.174659, -.110923, -.008907
	int  .133761, .317147, .538981, .794912

